@@ -1,17 +1,20 @@
 #!/bin/sh
 
+# glame requires at least automake 1.6 and autoconf 2.5
+
 rm -f config.cache libtool ltconfig ltmain.sh po/Makefile.in.in
 rm -rf intl/ ABOUT-NLS libltdl
-if test -L configure.ac -o ! -e configure.ac; then
-	ln -fs configure.in configure.ac
-fi
-yes n | gettextize --copy
+autopoint --force
 aclocal -I . -I macros
 libtoolize --copy --automake --ltdl
-if test -L libltdl/configure.ac -o ! -e libltdl/configure.ac; then
-	ln -fs configure.in libltdl/configure.ac
-fi
 autoheader
 touch ltconfig
 automake --copy --add-missing
 autoconf
+
+cd libltdl
+aclocal -I .
+autoheader
+automake
+autoconf
+cd ..
