@@ -1,6 +1,6 @@
 /*
  * filter.c
- * $Id: filter.c,v 1.62 2001/12/05 15:42:02 richi Exp $
+ * $Id: filter.c,v 1.64.2.1 2004/01/11 00:14:22 richi Exp $
  *
  * Copyright (C) 1999, 2000 Richard Guenther
  *
@@ -61,7 +61,7 @@ int _GLAME_WBUFSIZE = 1024;
         filter_t, list)
 
 /* Allocate pristine filternode/network. */
-filter_t *_filter_alloc()
+filter_t *_filter_alloc(void)
 {
 	filter_t *f;
 
@@ -386,7 +386,7 @@ filter_t *filter_collapse(const char *name, filter_t **nodes)
 	filter_port_t *port;
 	filter_pipe_t *pipe;
 
-	if (!name || !nodes)
+	if (!name || !nodes || !(*nodes))
 		return NULL;
 
 	/* Whats the nodes (and new net) parent - is it the same? */
@@ -525,7 +525,7 @@ filter_t *filter_collapse(const char *name, filter_t **nodes)
 
 /* I would like to have the following syntax:
  * (let* ((net (filter_creat))
- *        (nodename (filter_add_node net (plugin_get "filter") "nodename")))
+ *        (nodename (filter_add_node net (plugin-get "filter") "nodename")))
  *    (filternode_set_param ....)
  *    (filternetwork_add_input ...)
  *    (filternetwork_add_output ...)
@@ -569,7 +569,7 @@ char *filter_to_string(filter_t *net)
 				       n->name, subnet, n->name);
 			free(subnet);
 		} else
-			len += sprintf(&buf[len], "\t(%s (filter-add-node net (filter-new (plugin_get \"%s\")) \"%s\"))\n",
+			len += sprintf(&buf[len], "\t(%s (filter-add-node net (filter-new (plugin-get \"%s\")) \"%s\"))\n",
 				       n->name, plugin_name(n->plugin), n->name);
 	}
 	/* ((net .. */
