@@ -1,5 +1,5 @@
 ; glame.scm
-; $Id: glame.scm,v 1.74 2001/12/13 14:39:47 mainzelm Exp $
+; $Id: glame.scm,v 1.79.2.1 2003/04/19 18:23:19 richi Exp $
 ;
 ; Copyright (C) 2000, 2001 Richard Guenther, Martin Gasbichler
 ;
@@ -133,7 +133,7 @@
 	  (car default)))))
 
 ; GLAME 0.2.0 compatibility
-(define (plugin_description p) (plugin_query p "desc"))
+(define (plugin_description p) (plugin-query p "desc"))
 
 (define (help-helper maybe-command)
   (let ((commands-w/help (commands-w/help)))
@@ -209,7 +209,7 @@
 ; (net-add-node net "node")
 (define net-add-node
   (lambda (net node . params)
-    (let ((n (filter-new (plugin_get node))))
+    (let ((n (filter-new (plugin-get node))))
       (filter-add-node net n node)
       (apply node-set-params n params)
       n)))
@@ -273,8 +273,10 @@
 
 ; run the net, wait for completion and delete it
 (define net-run
-  (lambda (net)
-    (filter-launch net)
+  (lambda (net . bufsize)
+    (if (null? bufsize)
+	(filter-launch net)
+	(filter-launch net (car bufsize)))
     (filter-start net)
     (filter-wait net)))
 
@@ -282,7 +284,7 @@
 	  "run the net, wait for completion and delete it.")
 
 ; start the net (in the background, not deleting it afterwards),
-; i.e. you can use filter-(pause|start|terminate) on it.
+; i.e. you can use filter-(start|terminate) on it.
 (define net-run-bg
   (lambda (net)
     (filter-launch net)
@@ -384,296 +386,6 @@
       (display (sw-stat fd)) (newline)
       (display (sw-contents fd)) (newline))))
 
-(define swtest
-  (lambda (name test result)
-    (let ((res (test)))
-      (begin
-	(display name)
-	(cond ((equal? res result) (begin (display " passed.\n") #t))
-	      (else (begin
-		      (display " failed.\n")
-		      (display res) (newline)
-		      (display "Should be\n")
-		      (display result) (newline)
-		      (display "*** Test not passed. ***\n")
-		      #f)))))))
-
-(define swtest-truncate
-  (lambda ()
-    (let ((test (lambda ()
-		  (let ((fd (sw-creat 999)))
-		    (sw-write fd "Hello World!")
-		    (sw-ftruncate fd 5)
-		    (sw-ftruncate fd 64)
-		    (let ((res (sw-contents fd)))
-		      (sw-close fd)
-		      (sw-unlink 999)
-		      res)))))
-      (swtest "Truncate" test "Hello"))))
-
-(define swtest-rw-simple
-  (lambda ()
-    (let ((test (lambda ()
-		  (let ((fd (sw-creat 999)))
-		    (sw-write fd "Hello")
-		    (sw-write fd "World!")
-		    (sw-lseek fd 5 SEEK_SET)
-		    (sw-write fd " World!")
-		    (let ((res (sw-contents fd)))
-		      (sw-close fd)
-		      (sw-unlink 999)
-		      res)))))
-      (swtest "Simple read-write" test "Hello World!"))))
-
-(define swtest-cut-aligned
-  (lambda ()
-    (let ((test (lambda ()
-		  (let ((fd (sw-creat 999 "Hallo " "Leute, " "wie gehts?")))
-		    (sw-lseek fd 6 SEEK_SET)
-		    (sw-sendfile SW_NOFILE fd 7 SWSENDFILE_CUT)
-		    (let ((result (sw-contents fd)))
-		      (sw-close fd)
-		      (sw-unlink 999)
-		      result)))))
-      (swtest "Aligned cut" test "Hallo wie gehts?"))))
-
-(define swtest-cut-unaligned
-  (lambda ()
-    (let ((test (lambda ()
-		  (let ((fd (sw-creat 999 "Hallo Leute, wie gehts?")))
-		    (sw-lseek fd 6 SEEK_SET)
-		    (sw-sendfile SW_NOFILE fd 7 SWSENDFILE_CUT)
-		    (let ((result (sw-contents fd)))
-		      (sw-close fd)
-		      (sw-unlink 999)
-		      result)))))
-      (swtest "Unaligned cut" test "Hallo wie gehts?"))))
-
-(define swtest-cut-tail
-  (lambda ()
-    (let ((test (lambda ()
-		  (let ((fd (sw-creat 999 "Hallo Leute, wie gehts?")))
-		    (sw-lseek fd -6 SEEK_END)
-		    (sw-sendfile SW_NOFILE fd 6 SWSENDFILE_CUT)
-		    (let ((result (sw-contents fd)))
-		      (sw-close fd)
-		      (sw-unlink 999)
-		      result)))))
-      (swtest "Tail cut" test "Hallo Leute, wie "))))
-
-(define swtest-cut-tail-aligned
-  (lambda ()
-    (let ((test (lambda ()
-		  (let ((fd (sw-creat 999 "Hallo " "Leute, wie " "gehts?")))
-		    (sw-lseek fd -6 SEEK_END)
-		    (sw-sendfile SW_NOFILE fd 6 SWSENDFILE_CUT)
-		    (let ((result (sw-contents fd)))
-		      (sw-close fd)
-		      (sw-unlink 999)
-		      result)))))
-      (swtest "Aligned tail cut" test "Hallo Leute, wie "))))
-
-(define swtest-cut-tail-unaligned
-  (lambda ()
-    (let ((test (lambda ()
-		  (let ((fd (sw-creat 999 "Hallo " "Leute," " wie gehts?")))
-		    (sw-lseek fd -6 SEEK_END)
-		    (sw-sendfile SW_NOFILE fd 6 SWSENDFILE_CUT)
-		    (let ((result (sw-contents fd)))
-		      (sw-close fd)
-		      (sw-unlink 999)
-		      result)))))
-      (swtest "Unaligned tail cut" test "Hallo Leute, wie "))))
-
-(define swtest-cut-head
-  (lambda ()
-    (let ((test (lambda ()
-		  (let ((fd (sw-creat 999 "Hallo Leute, wie gehts?")))
-		    (sw-lseek fd 0 SEEK_SET)
-		    (sw-sendfile SW_NOFILE fd 6 SWSENDFILE_CUT)
-		    (let ((result (sw-contents fd)))
-		      (sw-close fd)
-		      (sw-unlink 999)
-		      result)))))
-      (swtest "Head cut" test "Leute, wie gehts?"))))
-
-(define swtest-cut-head-aligned
-  (lambda ()
-    (let ((test (lambda ()
-		  (let ((fd (sw-creat 999 "Hallo " "Leute, wie" " gehts?")))
-		    (sw-lseek fd 0 SEEK_SET)
-		    (sw-sendfile SW_NOFILE fd 6 SWSENDFILE_CUT)
-		    (let ((result (sw-contents fd)))
-		      (sw-close fd)
-		      (sw-unlink 999)
-		      result)))))
-      (swtest "Aligned head cut" test "Leute, wie gehts?"))))
-
-(define swtest-cut-head-unaligned
-  (lambda ()
-    (let ((test (lambda ()
-		  (let ((fd (sw-creat 999 "Hallo Leute," " wie" " gehts?")))
-		    (sw-lseek fd 0 SEEK_SET)
-		    (sw-sendfile SW_NOFILE fd 6 SWSENDFILE_CUT)
-		    (let ((result (sw-contents fd)))
-		      (sw-close fd)
-		      (sw-unlink 999)
-		      result)))))
-      (swtest "Unaligned head cut" test "Leute, wie gehts?"))))
-
-(define swtest-insert-aligned
-  (lambda ()
-    (let ((test (lambda ()
-		  (let ((fd1 (sw-creat 998 "Hallo " "wie gehts?"))
-			(fd2 (sw-creat 999 "Leute, ")))
-		    (sw-lseek fd1 6 SEEK_SET)
-		    (sw-lseek fd2 0 SEEK_SET)
-		    (sw-sendfile fd1 fd2 7 SWSENDFILE_INSERT)
-		    (let ((result (sw-contents fd1)))
-		      (sw-close fd1)
-		      (sw-close fd2)
-		      (sw-unlink 998)
-		      (sw-unlink 999)
-		      result)))))
-      (swtest "Aligned insert" test "Hallo Leute, wie gehts?"))))
-
-(define swtest-insert-unaligned
-  (lambda ()
-    (let ((test (lambda ()
-		  (let ((fd1 (sw-creat 998 "Hallo wie gehts?"))
-			(fd2 (sw-creat 999 "Bla Leute, Blubb")))
-		    (sw-lseek fd1 6 SEEK_SET)
-		    (sw-lseek fd2 4 SEEK_SET)
-		    (sw-sendfile fd1 fd2 7 SWSENDFILE_INSERT)
-		    (let ((result (sw-contents fd1)))
-		      (sw-close fd1)
-		      (sw-close fd2)
-		      (sw-unlink 998)
-		      (sw-unlink 999)
-		      result)))))
-      (swtest "Unaligned insert" test "Hallo Leute, wie gehts?"))))
-
-(define swtest-filecow
-  (lambda ()
-    (let ((test (lambda ()
-		  (let ((fd1 (sw-creat 998 "Hallo wie gehts?"))
-			(fd2 (sw-creat 999)))
-		    (sw-lseek fd1 0 SEEK_SET)
-		    (sw-ftruncate fd2 16)
-		    (sw-lseek fd2 0 SEEK_SET)
-		    (sw-sendfile fd2 fd1 16 0)
-		    (sw-lseek fd2 4 SEEK_SET)
-		    (sw-write fd2 "XXXXX")
-		    (let ((result (string-append (sw-contents fd1) (sw-contents fd2))))
-		      (sw-close fd1)
-		      (sw-close fd2)
-		      (sw-unlink 998)
-		      (sw-unlink 999)
-		      result)))))
-      (swtest "File COW" test "Hallo wie gehts?HallXXXXX gehts?"))))
-
-(define swtest-overwrite-aligned
-  (lambda ()
-    (let ((test (lambda ()
-		  (let ((fd1 (sw-creat 998 "Hallo Leute,"))
-			(fd2 (sw-creat 999 " wie gehts?")))
-		    (sw-ftruncate fd1 23)
-		    (sw-lseek fd1 12 SEEK_SET)
-		    (sw-lseek fd2 0 SEEK_SET)
-		    (sw-sendfile fd1 fd2 11 0)
-		    (let ((result (sw-contents fd1)))
-		      (sw-close fd1)
-		      (sw-close fd2)
-		      (sw-unlink 998)
-		      (sw-unlink 999)
-		      result)))))
-      (swtest "Aligned overwrite" test "Hallo Leute, wie gehts?"))))
-
-(define swtest-overwrite-unaligned
-  (lambda ()
-    (let ((test (lambda ()
-		  (let ((fd1 (sw-creat 998 "Hallo Leute, wie"))
-			(fd2 (sw-creat 999 " wie gehts?")))
-		    (sw-ftruncate fd1 23)
-		    (sw-lseek fd1 12 SEEK_SET)
-		    (sw-lseek fd2 0 SEEK_SET)
-		    (sw-sendfile fd1 fd2 11 0)
-		    (let ((result (sw-contents fd1)))
-		      (sw-close fd1)
-		      (sw-close fd2)
-		      (sw-unlink 998)
-		      (sw-unlink 999)
-		      result)))))
-      (swtest "Unaligned overwrite" test "Hallo Leute, wie gehts?"))))
-
-(define swtest-overwrite-unaligned-append
-  (lambda ()
-    (let ((test (lambda ()
-		  (let ((fd1 (sw-creat 998 "Hallo Leute, wie"))
-			(fd2 (sw-creat 999 " wie gehts?")))
-		    (sw-ftruncate fd1 20)
-		    (sw-lseek fd1 12 SEEK_SET)
-		    (sw-lseek fd2 0 SEEK_SET)
-		    (sw-sendfile fd1 fd2 11 0)
-		    (let ((result (sw-contents fd1)))
-		      (sw-close fd1)
-		      (sw-close fd2)
-		      (sw-unlink 998)
-		      (sw-unlink 999)
-		      result)))))
-      (swtest "Unaligned overwrite with append" test "Hallo Leute, wie gehts?"))))
-
-(define swtest-shared-shrink
-  (lambda ()
-    (let ((test (lambda ()
-		  (let ((fd1 (sw-creat 998 "Hallo Leute, wie gehts?"))
-			(fd2 (sw-creat 999)))
-		    (sw-sendfile fd2 fd1 23 0)
-		    (sw-ftruncate fd2 22)
-		    (let ((result (sw-contents fd2)))
-		      (sw-close fd1)
-		      (sw-close fd2)
-		      (sw-unlink 998)
-		      (sw-unlink 999)
-		      result)))))
-      (swtest "Shared file shrink" test "Hallo Leute, wie gehts"))))
-
-(define swtest-shared-grow
-  (lambda ()
-    (let ((test (lambda ()
-		  (let ((fd1 (sw-creat 998 "Hallo Leute, wie gehts?"))
-			(fd2 (sw-creat 999)))
-		    (sw-sendfile fd2 fd1 23 0)
-		    (sw-ftruncate fd2 24)
-		    (let ((result (sw-contents fd2)))
-		      (sw-close fd1)
-		      (sw-close fd2)
-		      (sw-unlink 998)
-		      (sw-unlink 999)
-		      result)))))
-      (swtest "Shared file grow" test "Hallo Leute, wie gehts?"))))
-
-(define swtest-all
-  (lambda ()
-    (and (swtest-rw-simple)
-	 (swtest-truncate)
-	 (swtest-cut-aligned)
-	 (swtest-cut-unaligned)
-	 (swtest-cut-head)
-	 (swtest-cut-head-aligned)
-	 (swtest-cut-head-unaligned)
-	 (swtest-cut-tail)
-	 (swtest-cut-tail-aligned)
-	 (swtest-cut-tail-unaligned)
-	 (swtest-insert-aligned)
-	 (swtest-insert-unaligned)
-	 (swtest-filecow)
-	 (swtest-shared-shrink)
-	 (swtest-shared-grow)
-	 (swtest-overwrite-aligned)
-	 (swtest-overwrite-unaligned)
-	 (swtest-overwrite-unaligned-append))))
-
 
 
 ;
@@ -704,7 +416,7 @@
     (let* ((net (net-new))
 	   (rf (net-add-node net read-file))
 	   (ao (net-add-node net audio-out)))
-      (filter-set-param rf "filename" fname)
+      (filter-set-param! rf "filename" fname)
       (while-not-false
        (lambda ()
 	 (let* ((eff (apply net-add-nodes net effects))
@@ -760,8 +472,8 @@
     (let* ((net (net-new))
 	   (rf (net-add-node net read-file))
 	   (wf (net-add-node net write-file)))
-      (filter-set-param rf "filename" fname)
-      (filter-set-param wf "filename" oname)
+      (filter-set-param! rf "filename" fname)
+      (filter-set-param! wf "filename" oname)
       (while-not-false
        (lambda ()
 	 (let* ((eff (apply net-add-nodes net effects))
@@ -771,7 +483,7 @@
 	   (if (eq? conn #f)
 	       (begin (apply nodes-delete eff) #f)
 	       (begin (nodes-connect (append eff (list wf))) #t)))))
-      (net-run net))))
+      (net-run net (* 64 1024)))))
 
 (add-help 'save-eff '(input-filename output-filename effect ...)
 	  "apply the effects to the input and write output")
@@ -918,6 +630,24 @@
      swfile-pipe)
     grp))
 
+(define gpsm-grp-find-item
+  (lambda (grp hpos vpos)
+    (if (and (= (gpsm-item-hposition grp) hpos)
+	     (= (gpsm-item-vposition grp) vpos))
+	grp
+	(if (gpsm-item-swfile? grp)
+	    #f
+	    (gpsm-grp-find-item-loop (gpsm-grp-items grp)
+				     (+ hpos (gpsm-item-hposition grp))
+				     (+ vpos (gpsm-item-vposition grp)))))))
+(define gpsm-grp-find-item-loop
+  (lambda (list hpos vpos)
+    (if (null? list)
+	#f
+	(let ((item (gpsm-grp-find-item (car list))))
+	  (if item
+	      item
+	      (gpsm-grp-find-item-loop (cdr list)))))))
 
 
 
@@ -940,3 +670,35 @@
 
 (add-help 'test-latency '(number)
 	  "Checks latency of a chain of number null filters")
+
+
+;
+; other stuff
+;
+
+;; Helper function, I just love mapconcat!
+(define (mapconcat proc list sep)
+  (let ((newlist (map proc list)))
+    (apply string-append
+           (cons (car newlist)
+                 (map (lambda (elt)
+                        (string-append sep elt))
+                      (cdr newlist))))))
+
+(define describe-filter-params
+  (lambda (filter)
+    (display ";; The parameters of ")
+    (display (filter-name filter))
+    (display
+     (string-append
+      " are:\n"
+      (mapconcat
+       (lambda (param)
+         (string-append "; " (param-label param) ": " (param->string param)))
+       (filter-params filter) "\n")
+      "\n"))))
+
+(add-help 'describe-filter-params '(filter)
+          "Display a simple description of filters parameters.")
+
+
