@@ -169,9 +169,9 @@ static void dump_ops(xmlNodePtr node)
 }
 
 static void insert_node_op(xmlNodePtr node);
-static void insert_node_file(gpsm_grp_t *tree, xmlNodePtr node);
-static void insert_node_grp(gpsm_grp_t *tree, xmlNodePtr node);
-static void insert_childs(gpsm_grp_t *tree, xmlNodePtr node)
+static void insert_node_file(gpsm_grp_t *tree, xmlNodePtr node, gpsm_grp_t *);
+static void insert_node_grp(gpsm_grp_t *tree, xmlNodePtr node, gpsm_grp_t *);
+static void insert_childs(gpsm_grp_t *tree, xmlNodePtr node, gpsm_grp_t *lost)
 {
 #ifndef xmlChildrenNode
         node = node->childs;
@@ -183,9 +183,9 @@ static void insert_childs(gpsm_grp_t *tree, xmlNodePtr node)
 
 	while (node) {
 		if (strcmp(node->name, "group") == 0)
-			insert_node_grp(tree, node);
+			insert_node_grp(tree, node, lost);
 		else if (strcmp(node->name, "file") == 0)
-			insert_node_file(tree, node);
+			insert_node_file(tree, node, lost);
 		else if (strcmp(node->name, "op") == 0)
 			insert_node_op(node);
 		else if (strcmp(node->name, "text") == 0)
@@ -195,7 +195,8 @@ static void insert_childs(gpsm_grp_t *tree, xmlNodePtr node)
 		node = node->next;
 	}
 }
-static void insert_node_file(gpsm_grp_t *tree, xmlNodePtr node)
+static void insert_node_file(gpsm_grp_t *tree, xmlNodePtr node,
+			     gpsm_grp_t *lost)
 {
 	char *c, *ilabel;
 	long ihposition, ivposition;
@@ -249,11 +250,15 @@ static void insert_node_file(gpsm_grp_t *tree, xmlNodePtr node)
 	swfile->item.hsize = st.size/SAMPLE_SIZE;
 	swfile->item.vsize = 1;
 	hash_add_swfile(swfile);
-	if (gpsm_item_place(tree, (gpsm_item_t *)swfile, ihposition, ivposition) == -1)
+	if (gpsm_item_place(tree, (gpsm_item_t *)swfile, ihposition, ivposition) == -1) {
 		DPRINTF("WARNING: placement of %s at %li, %li rejected\n",
 			gpsm_item_label(swfile), ihposition, ivposition);
+		gpsm_item_place(lost, (gpsm_item_t *)swfile,
+				0, gpsm_item_vsize(lost) + 1);
+	}
 }
-static void insert_node_grp(gpsm_grp_t *tree, xmlNodePtr node)
+static void insert_node_grp(gpsm_grp_t *tree, xmlNodePtr node,
+			    gpsm_grp_t *lost)
 {
         gpsm_item_t *item;
 	char *c, *ilabel;
@@ -275,12 +280,15 @@ static void insert_node_grp(gpsm_grp_t *tree, xmlNodePtr node)
 	/* Create new group and insert it into tree. */
 	item = gpsm_newitem(GPSM_ITEM_TYPE_GRP);
 	item->label = ilabel;
-	if (gpsm_item_place(tree, item, ihposition, ivposition) == -1)
+	if (gpsm_item_place(tree, item, ihposition, ivposition) == -1) {
 		DPRINTF("WARNING: placement of %s at %li, %li rejected\n",
 			gpsm_item_label(item), ihposition, ivposition);
+		gpsm_item_place(lost, item,
+				0, gpsm_item_vsize(lost) + 1);
+	}
 
 	/* Recurse down the childrens. */
-	insert_childs((gpsm_grp_t *)item, node);
+	insert_childs((gpsm_grp_t *)item, node, lost);
 }
 static void insert_node_op(xmlNodePtr node)
 {
@@ -365,7 +373,7 @@ static void insert_node_op(xmlNodePtr node)
 
 /* Scan the swapfile and add all non-xmled files to a seperate
  * group. */
-static void scan_swap()
+static void scan_swap(void)
 {
 	gpsm_swfile_t *swfile;
 	gpsm_grp_t *grp;
@@ -422,6 +430,7 @@ int gpsm_init(const char *swapfile)
 	xmlDocPtr doc;
 	char *xml;
 	int fd;
+	gpsm_grp_t *lost;
 
 	if (root)
 		return -1;
@@ -486,7 +495,16 @@ empty:
 	root->item.vposition = 0;
 	root->item.hsize = 0;
 	root->item.vsize = 0;
-        insert_childs(root, xmlDocGetRootElement(doc));
+	lost = (gpsm_grp_t *)gpsm_newitem(GPSM_ITEM_TYPE_GRP);
+	lost->item.label = strdup("(Lost on initialization)");
+	lost->item.hposition = 0;
+	lost->item.vposition = 0;
+	lost->item.hsize = 0;
+	lost->item.vsize = 0;
+	insert_childs(root, xmlDocGetRootElement(doc), lost);
+	if (gpsm_item_vsize(lost) != 0)
+		gpsm_item_place(root, (gpsm_item_t *)lost,
+				0, gpsm_item_vsize(root) + 1);
 
 	/* Search for not xml-ed swapfile. */
 	scan_swap();
@@ -512,7 +530,7 @@ int gpsm_set_max_saved_ops(int max)
 	return op_max_listsize;
 }
 
-void gpsm_sync()
+void gpsm_sync(void)
 {
 	xmlDocPtr doc;
 	xmlNodePtr docroot;
@@ -537,7 +555,7 @@ void gpsm_sync()
 	xmlFreeDoc(doc);
 }
 
-void gpsm_close()
+void gpsm_close(void)
 {
 	if (!root)
 		return;
@@ -620,7 +638,8 @@ gpsm_swfile_t *gpsm_newswfile(const char *label)
 			     O_RDWR|O_CREAT|O_EXCL)) == -1)
 		;
 	sw_close(fd);
-	swfile->samplerate = glame_config_get_long_with_default("audio_io/input_rate", GLAME_DEFAULT_SAMPLERATE);
+	swfile->samplerate = glame_config_get_long_with_default("audio_io/input_rate", 
+								GLAME_DEFAULT_SAMPLERATE);
 	swfile->position = 0.0;
 	hash_add_swfile(swfile);
 
@@ -1114,47 +1133,71 @@ int gpsm_vbox_can_insert(gpsm_grp_t *grp, gpsm_item_t *item,
 }
 
 /* Transparently handle item moving top-down / back-front. */
+static void __gpsm_item_insert_space_after(gpsm_grp_t *grp, struct glame_list_head *node, long dx, long dy)
+{
+	long old_hsize[256], old_vsize[256]; /* FIXME: should be ever enough */
+	int olds = 0, i;
+	gpsm_grp_t *parent;
+
+	if (!grp)
+		return;
+
+	/* Remember old BB sizes. */
+	parent = grp;
+	do {
+		old_hsize[olds] = gpsm_item_hsize(parent);
+		old_vsize[olds] = gpsm_item_vsize(parent);
+		parent = gpsm_item_parent(parent);
+		olds++;
+	} while (parent);
+
+	/* Go up the boxes, moving items. */
+	parent = grp;
+	i = 0;
+	do {
+		struct glame_list_head *nodei;
+		int is_hbox, is_vbox;
+		DPRINTF("Iterating through %s [%li,%li]\n", gpsm_item_label(parent), gpsm_item_hsize(parent), gpsm_item_vsize(parent));
+		is_hbox = gpsm_grp_is_hbox(parent);
+		is_vbox = gpsm_grp_is_vbox(parent);
+		nodei = parent->items.prev;
+		while (nodei != node) {
+			gpsm_item_t *item = glame_list_entry(nodei, gpsm_item_t, list);
+			DPRINTF("Considering %s\n", gpsm_item_label(item));
+			if (is_hbox) {
+				DPRINTF("X shift by %li\n", dx);
+				item->hposition += dx;
+			}
+			if (is_vbox) {
+				DPRINTF("Y shift by %li\n", dy);
+				item->vposition += dy;
+			}
+			glsig_emit(gpsm_item_emitter(item), GPSM_SIG_ITEM_CHANGED, item);
+			nodei = nodei->prev;
+		}
+		DPRINTF("%s is now [%li,%li]\n", gpsm_item_label(parent), gpsm_item_hsize(parent), gpsm_item_vsize(parent));
+		dx = gpsm_item_hsize(parent) - old_hsize[i];
+		dy = gpsm_item_vsize(parent) - old_vsize[i];
+		DPRINTF("dx/dy now %li/%li\n", dx, dy);
+		/* if (dx == 0 && dy == 0)
+		   return; */
+
+		node = &parent->item.list;
+		parent = gpsm_item_parent(parent);
+		i++;
+	} while (parent);
+}
 static void _gpsm_item_insert_space_after(gpsm_item_t *item, long dx, long dy)
 {
-	gpsm_item_t *it;
-
-	/* Top - down */
-	if (!gpsm_item_parent(item))
-		return;
-	_gpsm_item_insert_space_after((gpsm_item_t *)gpsm_item_parent(item),
-				      dx, dy);
-
-	/* Back to front. */
-	it = glame_list_gettail(&gpsm_item_parent(item)->items, gpsm_item_t, list);
-	while (it && it != item) {
-		it->hposition += dx;
-		it->vposition += dy;
-		glsig_emit(gpsm_item_emitter(it), GPSM_SIG_ITEM_CHANGED, it);
-		it = glame_list_getprev(&gpsm_item_parent(item)->items, it,
-				  gpsm_item_t, list);
-	}
+	DPRINTF("Inserting space %li/%li after %s\n", dx, dy, gpsm_item_label(item));
+	__gpsm_item_insert_space_after(gpsm_item_parent(item),
+				       &item->list, dx, dy);
 }
 static void _gpsm_item_insert_space_before(gpsm_item_t *item, long dx, long dy)
 {
-	gpsm_item_t *it;
-
-	/* Top - down */
-	if (!gpsm_item_parent(item))
-		return;
-	_gpsm_item_insert_space_after((gpsm_item_t *)gpsm_item_parent(item),
-				      dx, dy);
-
-	/* Back to front. */
-	it = glame_list_gettail(&gpsm_item_parent(item)->items, gpsm_item_t, list);
-	do {
-		it->hposition += dx;
-		it->vposition += dy;
-		glsig_emit(gpsm_item_emitter(it), GPSM_SIG_ITEM_CHANGED, it);
-		if (it == item)
-			break;
-		it = glame_list_getprev(&gpsm_item_parent(item)->items, it,
-				  gpsm_item_t, list);
-	} while (it);
+	DPRINTF("Inserting space %li/%li before %s\n", dx, dy, gpsm_item_label(item));
+	__gpsm_item_insert_space_after(gpsm_item_parent(item),
+				       item->list.prev, dx, dy);
 }
 
 int gpsm_hbox_insert(gpsm_grp_t *hbox, gpsm_item_t *item,
@@ -1165,15 +1208,33 @@ int gpsm_hbox_insert(gpsm_grp_t *hbox, gpsm_item_t *item,
 	if (!root || !gpsm_hbox_can_insert(hbox, item, hposition, vposition))
 		return -1;
 
-	/* Cut out the item first - closing the gap it leaves.
-	 * FIXME: whats the removal semantic we want to have? */
-	if (gpsm_grp_is_hbox(gpsm_item_parent(item)))
-		gpsm_hbox_cut(item);
-	else if (gpsm_grp_is_vbox(gpsm_item_parent(item)))
-		gpsm_vbox_cut(item);
-	else {
-		DPRINTF("WARNING: doing ordinary remove on hbox_insert\n");
-		gpsm_item_remove(item);
+	/* Find insertion point FIRST, because removing the item
+	 * may invalidate insertion position
+	 * - succ will point to the successor. */
+	gpsm_grp_foreach_item(hbox, succ) {
+		if (gpsm_item_hposition(succ) >= hposition)
+			break;
+	}
+	/* Insertion position == current position? Fixup for item removal. */
+	if (succ == item)
+		succ = glame_list_getnext(&hbox->items, succ, gpsm_item_t, list);
+	/* Remember insertion vposition relative to succ+space/tail. */
+	if (succ)
+		hposition = gpsm_item_hposition(succ) + gpsm_item_hsize(item) - hposition;
+	else
+		hposition = gpsm_item_hsize(hbox) - hposition;
+
+	if (gpsm_item_parent(item)) {
+		/* Cut out the item first - closing the gap it leaves.
+		 * FIXME: whats the removal semantic we want to have? */
+		if (gpsm_grp_is_hbox(gpsm_item_parent(item)))
+			gpsm_hbox_cut(item);
+		else if (gpsm_grp_is_vbox(gpsm_item_parent(item)))
+			gpsm_vbox_cut(item);
+		else {
+			DPRINTF("WARNING: doing ordinary remove on hbox_insert\n");
+			gpsm_item_remove(item);
+		}
 	}
 
 	/* Register an item changed signal handler to the new item
@@ -1182,12 +1243,6 @@ int gpsm_hbox_insert(gpsm_grp_t *hbox, gpsm_item_t *item,
 			  GPSM_SIG_ITEM_CHANGED|GPSM_SIG_ITEM_REMOVE,
 			  handle_itemchange, item);
 
-	/* Find insertion point - succ will point to the successor. */
-	gpsm_grp_foreach_item(hbox, succ) {
-		if (gpsm_item_hposition(succ) >= hposition)
-			break;
-	}
-
 	/* Move all nodes beginning at succ by item hsize. */
 	if (succ)
 		_gpsm_item_insert_space_before(succ, gpsm_item_hsize(item), 0);
@@ -1196,7 +1251,10 @@ int gpsm_hbox_insert(gpsm_grp_t *hbox, gpsm_item_t *item,
 					      gpsm_item_hsize(item), 0);
 
 	/* Do the insertion. */
-	_place_tail(hbox, item, succ, hposition, vposition);
+	_place_tail(hbox, item, succ,
+		    succ ? gpsm_item_hposition(succ) - hposition
+		    : gpsm_item_hsize(hbox) - hposition,
+		    vposition);
 
 	return 0;
 }
@@ -1209,15 +1267,33 @@ int gpsm_vbox_insert(gpsm_grp_t *vbox, gpsm_item_t *item,
 	if (!root || !gpsm_vbox_can_insert(vbox, item, hposition, vposition))
 		return -1;
 
-	/* Cut out the item first - closing the gap it leaves.
-	 * FIXME: whats the removal semantic we want to have? */
-	if (gpsm_grp_is_vbox(gpsm_item_parent(item)))
-		gpsm_vbox_cut(item);
-	else if (gpsm_grp_is_hbox(gpsm_item_parent(item)))
-		gpsm_hbox_cut(item);
-	else {
-		DPRINTF("WARNING: doing ordinary remove on vbox_insert\n");
-		gpsm_item_remove(item);
+	/* Find insertion point FIRST, because removing the item
+	 * may invalidate insertion position
+	 * - succ will point to the successor. */
+	gpsm_grp_foreach_item(vbox, succ) {
+		if (gpsm_item_vposition(succ) >= vposition)
+			break;
+	}
+	/* Insertion position == current position? Fixup for item removal. */
+	if (succ == item)
+		succ = glame_list_getnext(&vbox->items, succ, gpsm_item_t, list);
+	/* Remember insertion vposition relative to succ+space/tail. */
+	if (succ)
+		vposition = gpsm_item_vposition(succ) + gpsm_item_vsize(item) - vposition;
+	else
+		vposition = gpsm_item_vsize(vbox) - vposition;
+
+	if (gpsm_item_parent(item)) {
+		/* Cut out the item first - closing the gap it leaves.
+		 * FIXME: whats the removal semantic we want to have? */
+		if (gpsm_grp_is_vbox(gpsm_item_parent(item)))
+			gpsm_vbox_cut(item);
+		else if (gpsm_grp_is_hbox(gpsm_item_parent(item)))
+			gpsm_hbox_cut(item);
+		else {
+			DPRINTF("WARNING: doing ordinary remove on vbox_insert\n");
+			gpsm_item_remove(item);
+		}
 	}
 
 	/* Register an item changed signal handler to the new item
@@ -1226,12 +1302,6 @@ int gpsm_vbox_insert(gpsm_grp_t *vbox, gpsm_item_t *item,
 			  GPSM_SIG_ITEM_CHANGED|GPSM_SIG_ITEM_REMOVE,
 			  handle_itemchange, item);
 
-	/* Find insertion point - succ will point to the successor. */
-	gpsm_grp_foreach_item(vbox, succ) {
-		if (gpsm_item_vposition(succ) >= vposition)
-			break;
-	}
-
 	/* Move all nodes beginning at succ by item vsize. */
 	if (succ)
 		_gpsm_item_insert_space_before(succ, 0, gpsm_item_vsize(item));
@@ -1240,7 +1310,9 @@ int gpsm_vbox_insert(gpsm_grp_t *vbox, gpsm_item_t *item,
 					      0, gpsm_item_vsize(item));
 
 	/* Do the insertion. */
-	_place_tail(vbox, item, succ, hposition, vposition);
+	_place_tail(vbox, item, succ, hposition,
+		    succ ? gpsm_item_vposition(succ) - vposition
+		    : gpsm_item_vsize(vbox) - vposition);
 
 	return 0;
 }
@@ -1384,13 +1456,13 @@ void gpsm_notify_swapfile_change(long filename, long pos, long size)
 	while ((swfile = hash_find_next_swfile(filename, swfile))) {
 		/* Handle extension transparently. */
 		if (pos + size > gpsm_item_hsize(swfile))
-			gpsm_notify_swapfile_insert(filename, gpsm_item_hsize(swfile), pos + size - 1 - gpsm_item_hsize(swfile));
+			gpsm_notify_swapfile_insert(filename, gpsm_item_hsize(swfile), pos + size - gpsm_item_hsize(swfile));
 
 		glsig_emit(gpsm_item_emitter(swfile), GPSM_SIG_SWFILE_CHANGED,
 			   swfile, pos, size);
 #ifdef DEBUG
 		if (gpsm_item_hsize(swfile) != file_size)
-			DPRINTF("WARNING! Unnoticed swapfile change!\n");
+			DPRINTF("WARNING! Unnoticed swapfile change! (size differs by %li)\n", (long)(file_size - gpsm_item_hsize(swfile)));
 #endif
 	}
 }
@@ -1422,7 +1494,7 @@ void gpsm_notify_swapfile_cut(long filename, long pos, long size)
 			   GPSM_SIG_ITEM_CHANGED, swfile);
 #ifdef DEBUG
 		if (gpsm_item_hsize(swfile) != file_size)
-			DPRINTF("WARNING! Unnoticed swapfile change!\n");
+			DPRINTF("WARNING! Unnoticed swapfile change! (size differs by %li)\n", (long)(file_size - gpsm_item_hsize(swfile)));
 #endif
 	}
 }
@@ -1454,7 +1526,7 @@ void gpsm_notify_swapfile_insert(long filename, long pos, long size)
 			   GPSM_SIG_ITEM_CHANGED, swfile);
 #ifdef DEBUG
 		if (gpsm_item_hsize(swfile) != file_size)
-			DPRINTF("WARNING! Unnoticed swapfile change!\n");
+			DPRINTF("WARNING! Unnoticed swapfile change! (size differs by %li)\n", (long)(file_size - gpsm_item_hsize(swfile)));
 #endif
 	}
 }
