@@ -1,9 +1,9 @@
 /*
  * ladspa.c
  *
- * $Id: ladspa.c,v 1.12.2.2 2002/02/17 17:20:31 richi Exp $
+ * $Id: ladspa.c,v 1.18.4.3 2003/06/24 21:21:14 richi Exp $
  * 
- * Copyright (C) 2000 Richard Furse, Alexander Ehlert
+ * Copyright (C) 2000-2003 Richard Furse, Alexander Ehlert, Richard Guenther
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +28,8 @@
 #include <dirent.h>
 #include <dlfcn.h>
 #include <string.h>
+#include <ctype.h>
+#include <limits.h>
 #include <ladspa.h>
 #include "filter.h"
 #include "util.h"
@@ -42,9 +44,9 @@ static int ladspa_connect_out(filter_port_t *port, filter_pipe_t *p)
 	int rate;
 	float pos;
 
-	rate = filterparam_val_int(filterparamdb_get_param(
+	rate = filterparam_val_long(filterparamdb_get_param(
 				filter_paramdb(n), "GLAME Sample Rate"));
-	pos = filterparam_val_float(filterparamdb_get_param(
+	pos = filterparam_val_double(filterparamdb_get_param(
 				filter_paramdb(n), "GLAME Position"));
 	filterpipe_settype_sample(p, rate, pos);
 	return 0;
@@ -127,7 +129,7 @@ static int ladspa_f(filter_t * n)
 		    filterparamdb_get_param(filter_paramdb(n),
 					    "GLAME Sample Rate");
 		if (psParam)
-			lSampleRate = filterparam_val_int(psParam);
+			lSampleRate = filterparam_val_long(psParam);
 		else
 			lSampleRate = GLAME_DEFAULT_SAMPLERATE;
 	}
@@ -137,9 +139,12 @@ static int ladspa_f(filter_t * n)
 	if (psParam != NULL) {
 		/* ok, we have a sound generator there
 		 * give it some time to rumble */
-		glame_timer = TIME2CNT(unsigned long, 
-				       filterparam_val_float(psParam)*1000.0,
+		glame_timer = TIME2CNT(long,
+				       filterparam_val_double(psParam)*1000.0,
 				       lSampleRate);
+		/* 0 means infinity. */
+		if (glame_timer == 0)
+			glame_timer = LONG_MAX;
 	}
 	else 
 		glame_timer = 1;
@@ -201,13 +206,13 @@ static int ladspa_f(filter_t * n)
 				/* psParam == NULL does not happen if params were registered
 				 * appropriately. [richi] */
 				pfControlValues[lPortIndex] =
-				    filterparam_val_float(psParam);
+				    filterparam_val_double(psParam);
 			}
 			
 /* 
  * We now need to wire up the control port on the LADSPA plugin
  * to the point in the control ports array. For an input port
- * the value acquired by filterparam_val_float() will be
+ * the value acquired by filterparam_val_double() will be
  * used. For an output port the value will be written into the
  * relevant spot in the array but never read again. Less than
  * ideal, perhaps a FIXME for a later date... 
@@ -290,7 +295,7 @@ static int ladspa_f(filter_t * n)
 				/* psParam == NULL does not happen if params were registered
 				 * appropriately. [richi] */
 					pfControlValues[lPortIndex] =
-					    filterparam_val_float(psParam);
+					    filterparam_val_double(psParam);
 				}
 			
 				psDescriptor->connect_port(psLADSPAPluginInstance,
@@ -483,6 +488,7 @@ int installLADSPAPlugin(const LADSPA_Descriptor * psDescriptor,
 	LADSPA_PortDescriptor iPortDescriptor;
 	filter_port_t *psPort;
 	filter_t *psFilter;
+	char category[16];
 
 	psFilter = filter_creat(NULL);
 	if (!psFilter)
@@ -541,54 +547,108 @@ int installLADSPAPlugin(const LADSPA_Descriptor * psDescriptor,
 		if (LADSPA_IS_PORT_INPUT(iPortDescriptor)
 		    && LADSPA_IS_PORT_CONTROL(iPortDescriptor)) {
 			filter_param_t *param;
-			int bound_below = 0, bound_above = 0;
+			int bound_below = 0, bound_above = 0, isint = 0, toggled = 0;
 			fBound1 = 0;
 			fBound2 = 0;
 			if (LADSPA_IS_HINT_BOUNDED_BELOW
-			    (psDescriptor->PortRangeHints[lPortIndex].
-			     HintDescriptor)) {
+			    (psDescriptor->PortRangeHints[lPortIndex].HintDescriptor)) {
 				bound_below = 1;
 				fBound1 =
-				    psDescriptor->
-				    PortRangeHints[lPortIndex].LowerBound;
+				    psDescriptor->PortRangeHints[lPortIndex].LowerBound;
 			}
 			if (LADSPA_IS_HINT_BOUNDED_ABOVE
-			    (psDescriptor->PortRangeHints[lPortIndex].
-			     HintDescriptor)) {
+			    (psDescriptor->PortRangeHints[lPortIndex].HintDescriptor)) {
 				bound_above = 1;
 				fBound2 =
-				    psDescriptor->
-				    PortRangeHints[lPortIndex].UpperBound;
+				    psDescriptor->PortRangeHints[lPortIndex].UpperBound;
 			}
 			if (LADSPA_IS_HINT_SAMPLE_RATE
-			    (psDescriptor->PortRangeHints[lPortIndex].
-			     HintDescriptor)) {
+			    (psDescriptor->PortRangeHints[lPortIndex].HintDescriptor)) {
 				fBound1 *= lSampleRate;
 				fBound2 *= lSampleRate;
 			}
 			if (fBound1 > 0
 			    && fBound2 > 0
 			    && LADSPA_IS_HINT_LOGARITHMIC
-			    (psDescriptor->PortRangeHints[lPortIndex].
-			     HintDescriptor))
+			    (psDescriptor->PortRangeHints[lPortIndex].HintDescriptor))
 				fRecommendation = 0.0; /* -- assume db, 0 is sane default, then. (was: sqrt(fBound1 * fBound2);) */
 			else
 				fRecommendation =
 				    0.5 * (fBound1 + fBound2);
 			if (LADSPA_IS_HINT_INTEGER
-			    (psDescriptor->PortRangeHints[lPortIndex].
-			     HintDescriptor))
+			    (psDescriptor->PortRangeHints[lPortIndex].HintDescriptor)) {
+				isint = 1;
 				fRecommendation =
 				    (LADSPA_Data) (long) (fRecommendation +
 							  0.5);
-			param = filterparamdb_add_param_float(
-				filter_paramdb(psFilter),
-				psDescriptor->PortNames[lPortIndex],
-				FILTER_PARAMTYPE_FLOAT, fRecommendation,
-				FILTERPARAM_END);
-			if (bound_below && bound_above) {
+			}
+			if (LADSPA_IS_HINT_TOGGLED
+			    (psDescriptor->PortRangeHints[lPortIndex].HintDescriptor)) {
+				toggled = 1;
+				fRecommendation = 0.0;
+			}
+			if (LADSPA_IS_HINT_HAS_DEFAULT
+			    (psDescriptor->PortRangeHints[lPortIndex].HintDescriptor)) {
+				const LADSPA_PortRangeHintDescriptor *d;
+				d = &psDescriptor->PortRangeHints[lPortIndex].HintDescriptor;
+				if (LADSPA_IS_HINT_DEFAULT_MINIMUM(*d))
+					fRecommendation = fBound1;
+				else if (LADSPA_IS_HINT_DEFAULT_MAXIMUM(*d))
+					fRecommendation = fBound2;
+				else if (LADSPA_IS_HINT_DEFAULT_LOW(*d))
+					fRecommendation = LADSPA_IS_HINT_LOGARITHMIC(*d)
+						? exp(log(fBound1)*0.75 + log(fBound2)*0.25)
+						: fBound1*0.75 + fBound2*0.25;
+				else if (LADSPA_IS_HINT_DEFAULT_MIDDLE(*d))
+					fRecommendation = LADSPA_IS_HINT_LOGARITHMIC(*d)
+						? exp(0.5*(log(fBound1) + log(fBound2)))
+						: 0.5*(fBound1+fBound2);
+				else if (LADSPA_IS_HINT_DEFAULT_HIGH(*d))
+					fRecommendation = LADSPA_IS_HINT_LOGARITHMIC(*d)
+						? exp(log(fBound1)*0.25 + log(fBound2)*0.75)
+						: fBound1*0.25 + fBound2*0.75;
+				else if (LADSPA_IS_HINT_DEFAULT_0(*d))
+					fRecommendation = 0.0;
+				else if (LADSPA_IS_HINT_DEFAULT_1(*d))
+					fRecommendation = 1.0;
+				else if (LADSPA_IS_HINT_DEFAULT_100(*d))
+					fRecommendation = 100.0;
+				else if (LADSPA_IS_HINT_DEFAULT_440(*d))
+					fRecommendation = 440.0;
+			}
+			if  (isint || toggled) {
+				param = filterparamdb_add_param_long(
+					filter_paramdb(psFilter),
+					psDescriptor->PortNames[lPortIndex],
+					FILTER_PARAMTYPE_LONG, fRecommendation,
+					FILTERPARAM_END);
+			} else {
+				param = filterparamdb_add_param_double(
+					filter_paramdb(psFilter),
+					psDescriptor->PortNames[lPortIndex],
+					FILTER_PARAMTYPE_DOUBLE, fRecommendation,
+					FILTERPARAM_END);
+			}
+			if (!param) {
+				DPRINTF("Cannot add param for port %i (%s)\n",
+					(int)lPortIndex, psDescriptor->PortNames[lPortIndex]);
+				continue;
+			}
+
+			if (bound_below && bound_above && !isint) {
 				/* Use GtkHScale */
 				char xml[1024];
+				float step, page;
+				int lrange, lstep, lpage;
+				/* optimize step/page dependend on min-max range */
+				lrange = floor(log10(fBound2-fBound1));
+				lstep = MIN(0, lrange-2);
+				lpage = (lrange + lstep) / 2;
+				if (lstep == lpage)
+					lpage++;
+				step = pow(10.0, lstep);
+				page = pow(10.0, lpage);
+#if 1
 				snprintf(xml, 1023,
 "<?xml version=\"1.0\"?><GTK-Interface>"
 "  <widget>"
@@ -597,7 +657,7 @@ int installLADSPAPlugin(const LADSPA_Descriptor * psDescriptor,
 "    <can_focus>True</can_focus>"
 "    <draw_value>True</draw_value>"
 "    <value_pos>GTK_POS_LEFT</value_pos>"
-"    <digits>3</digits>"
+"    <digits>2</digits>"
 "    <policy>GTK_UPDATE_CONTINUOUS</policy>"
 "    <value>%.3f</value>"
 "    <lower>%.3f</lower>"
@@ -607,9 +667,41 @@ int installLADSPAPlugin(const LADSPA_Descriptor * psDescriptor,
 "    <page_size>%.3f</page_size>"
 "  </widget>"
 "</GTK-Interface>",
-					 fRecommendation, fBound1, fBound2, 0.1, 0.5, 0.0);
+					 fRecommendation, fBound1, fBound2, step, page, 0.0);
+#else
+				snprintf(xml, 1023,
+"<?xml version=\"1.0\"?><GTK-Interface>"
+" <widget>"
+"  <class>GtkHBox</class>"
+"  <name>root</name>"
+"  <widget>"
+"    <class>GtkKnob</class>"
+"    <name>widget</name>"
+"    <value>%.3f</value>"
+"    <lower>%.3f</lower>"
+"    <upper>%.3f</upper>"
+"  </widget>"
+"  <widget>"
+"    <class>GtkHScale</class>"
+"    <name>widget2</name>"
+"    <can_focus>True</can_focus>"
+"    <draw_value>True</draw_value>"
+"    <value_pos>GTK_POS_LEFT</value_pos>"
+"    <digits>3</digits>"
+"    <policy>GTK_UPDATE_CONTINUOUS</policy>"
+"    <value>0.0</value>"
+"    <lower>0.0</lower>"
+"    <upper>1.0</upper>"
+"    <step>0.1</step>"
+"    <page>0.1</page>"
+"    <page_size>0.0</page_size>"
+"  </widget>"
+" </widget>"
+"</GTK-Interface>",
+					 fRecommendation, fBound1, fBound2);
+#endif
 				filterparam_set_property(param, FILTERPARAM_GLADEXML, strdup(xml));
-			} else if (bound_below) {
+			} else if ((bound_below || bound_above) && !isint) {
 				/* Use GtkSpinButton */
 				char xml[1024];
 				snprintf(xml, 1023,
@@ -619,7 +711,7 @@ int installLADSPAPlugin(const LADSPA_Descriptor * psDescriptor,
 "    <name>widget</name>"
 "    <can_focus>True</can_focus>"
 "    <climb_rate>1</climb_rate>"
-"    <digits>3</digits>"
+"    <digits>2</digits>"
 "    <numeric>True</numeric>"
 "    <update_policy>GTK_UPDATE_ALWAYS</update_policy>"
 "    <snap>False</snap>"
@@ -632,9 +724,11 @@ int installLADSPAPlugin(const LADSPA_Descriptor * psDescriptor,
 "    <page_size>0.0</page_size>"
 "  </widget>"
 "</GTK-Interface>",
-					 fRecommendation, fBound1, 1e19, 0.1, 0.5);
+					 fRecommendation,
+					 bound_below ? fBound1 : -1e19,
+					 bound_above ? fBound2 : 1e19, 0.01, 0.1);
 				filterparam_set_property(param, FILTERPARAM_GLADEXML, strdup(xml));
-			} else if (bound_above) {
+			} else if ((bound_above || bound_below) && isint) {
 				/* Use GtkSpinButton */
 				char xml[1024];
 				snprintf(xml, 1023,
@@ -644,20 +738,51 @@ int installLADSPAPlugin(const LADSPA_Descriptor * psDescriptor,
 "    <name>widget</name>"
 "    <can_focus>True</can_focus>"
 "    <climb_rate>1</climb_rate>"
-"    <digits>3</digits>"
+"    <digits>0</digits>"
 "    <numeric>True</numeric>"
 "    <update_policy>GTK_UPDATE_ALWAYS</update_policy>"
 "    <snap>False</snap>"
 "    <wrap>False</wrap>"
-"    <value>%.3f</value>"
-"    <lower>%.3f</lower>"
-"    <upper>%.3f</upper>"
-"    <step>%.3f</step>"
-"    <page>%.3f</page>"
+"    <value>%.0f</value>"
+"    <lower>%.0f</lower>"
+"    <upper>%.0f</upper>"
+"    <step>%.0f</step>"
+"    <page>%.0f</page>"
 "    <page_size>0.0</page_size>"
 "  </widget>"
 "</GTK-Interface>",
-					 fRecommendation, -1e19, fBound2, 0.1, 0.5);
+					 fRecommendation,
+					 bound_below ? fBound1 : -1000,
+					 bound_above ? fBound2 : 1000,
+					 1.0, 1.0);
+				filterparam_set_property(param, FILTERPARAM_GLADEXML, strdup(xml));
+			} else if (toggled) {
+				/* Use GtkSpinButton */
+				char xml[1024];
+				snprintf(xml, 1023,
+"<?xml version=\"1.0\"?><GTK-Interface>"
+"  <widget>"
+"    <class>GtkSpinButton</class>"
+"    <name>widget</name>"
+"    <can_focus>True</can_focus>"
+"    <climb_rate>1</climb_rate>"
+"    <digits>0</digits>"
+"    <numeric>True</numeric>"
+"    <update_policy>GTK_UPDATE_ALWAYS</update_policy>"
+"    <snap>False</snap>"
+"    <wrap>False</wrap>"
+"    <value>%.0f</value>"
+"    <lower>%.0f</lower>"
+"    <upper>%.0f</upper>"
+"    <step>%.0f</step>"
+"    <page>%.0f</page>"
+"    <page_size>0.0</page_size>"
+"  </widget>"
+"</GTK-Interface>",
+					 fRecommendation,
+					 0.0,
+					 1.0,
+					 1.0, 1.0);
 				filterparam_set_property(param, FILTERPARAM_GLADEXML, strdup(xml));
 			}
 		}
@@ -672,22 +797,22 @@ int installLADSPAPlugin(const LADSPA_Descriptor * psDescriptor,
 			if (filterport_is_output(psPort))
 				psPort->connect = ladspa_connect_out;
 		
-		filterparamdb_add_param_int(filter_paramdb(psFilter),
+		filterparamdb_add_param_long(filter_paramdb(psFilter),
 					    "GLAME Sample Rate",
-					    FILTER_PARAMTYPE_INT,
+					    FILTER_PARAMTYPE_RATE,
 					    GLAME_DEFAULT_SAMPLERATE,
 					    FILTERPARAM_END);
 		
-		filterparamdb_add_param_float(filter_paramdb(psFilter),
+		filterparamdb_add_param_double(filter_paramdb(psFilter),
 					    "GLAME Position",
-					    FILTER_PARAMTYPE_FLOAT,
+					    FILTER_PARAMTYPE_POSITION,
 					    0.0,
 					    FILTERPARAM_END);
 		
-		filterparamdb_add_param_float(filter_paramdb(psFilter),
+		filterparamdb_add_param_double(filter_paramdb(psFilter),
 					    "GLAME Duration",
-					    FILTER_PARAMTYPE_FLOAT,
-					    5.0,
+					    FILTER_PARAMTYPE_TIME_S,
+					    0.0,
 					    FILTERPARAM_END);
 	}
 
@@ -706,6 +831,16 @@ int installLADSPAPlugin(const LADSPA_Descriptor * psDescriptor,
 		/* We deliberately do not call free() here. */
 	}
 
+	/* Set PLUGIN_LABEL, if desc->Name is available, set
+	 * PLUGIN_CATEGORY to first character of label/name. */
+	if (psDescriptor->Name)
+		plugin_set(psPlugin, PLUGIN_LABEL, psDescriptor->Name);
+	sprintf(category, "LADSPA/%c",
+		tolower(psDescriptor->Name ? psDescriptor->Name[0]
+                                           : psPlugin->name[0]));
+	plugin_set(psPlugin, PLUGIN_CATEGORY, strdup(category));
+
+
 	filter_register(psFilter, psPlugin);
 
 	return 0;
