@@ -1,7 +1,7 @@
 /*
  * canvasfilter.c
  *
- * $Id: canvasfilter.c,v 1.50.2.1 2002/01/16 21:08:02 richi Exp $
+ * $Id: canvasfilter.c,v 1.56.2.1 2003/04/13 18:51:02 richi Exp $
  *
  * Copyright (C) 2001 Johannes Hirche
  *
@@ -21,6 +21,11 @@
  *
  *
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <sys/param.h>
 #include <stdio.h>
 #include <math.h>
@@ -866,14 +871,13 @@ static void glame_canvas_filter_collapse_selection_cb(GtkWidget* foo, GlameCanva
 
 static void glame_canvas_filter_show_about(GtkWidget* foo, GlameCanvasFilter* filterItem)
 {
-		GtkWidget * dialog;
+	GtkWidget * dialog;
 	GtkWidget * text;
 	GtkWidget * vbox;
 	GtkWidget * notebook;
 	GtkWidget * tablabel;
 	GtkCList * list;
 	char * desc;
-	int pos=0;
 
 	filter_portdb_t * ports;
 	filter_paramdb_t * params;
@@ -893,19 +897,19 @@ static void glame_canvas_filter_show_about(GtkWidget* foo, GlameCanvasFilter* fi
 	
 	dialog = gnome_dialog_new(plugin_name(filter->plugin),GNOME_STOCK_BUTTON_OK,NULL);
 	
-
 	desc = (char*)plugin_query(filter->plugin,PLUGIN_DESCRIPTION);
-	text = gtk_text_new(NULL,NULL);
-	gtk_widget_show(text);
 	
-	if(desc)
-		gtk_editable_insert_text(GTK_EDITABLE(text),desc,strlen(desc),&pos);
-	else
-		gtk_editable_insert_text(GTK_EDITABLE(text),_("This item does not have a description"),38,&pos);
+	if(desc) 
+		text=gtk_label_new(desc);
+	else 
+		text = gtk_label_new(_("This item does not have a description"));
+	gtk_label_set_line_wrap(GTK_LABEL(text),TRUE);
+	gtk_label_set_justify(GTK_LABEL(text),GTK_JUSTIFY_LEFT);
+	gtk_misc_set_alignment(GTK_MISC(text), 0.0, 0.5);
 	tablabel = gtk_label_new(_("Description"));
 	gtk_widget_show(tablabel);
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),text,tablabel);
-
+	gtk_widget_show(text);
 	tablabel = gtk_label_new(_("Ports"));
 	list = GTK_CLIST(gtk_clist_new_with_titles(3,labels));
 	gtk_clist_set_column_auto_resize(list,0,TRUE);
@@ -952,11 +956,11 @@ static void glame_canvas_filter_show_about(GtkWidget* foo, GlameCanvasFilter* fi
 	}
 	gtk_widget_show(GTK_WIDGET(list));
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),GTK_WIDGET(list),tablabel);
-
+	
 	gtk_widget_show(notebook);
 	vbox = GNOME_DIALOG(dialog)->vbox;
 	gtk_container_add(GTK_CONTAINER(vbox),notebook);
-
+	
 	gnome_dialog_button_connect_object(GNOME_DIALOG(dialog), 0, gtk_object_destroy, GTK_OBJECT(dialog));
 	gtk_widget_show(GTK_WIDGET(dialog));
 }
@@ -1087,8 +1091,10 @@ static GnomeUIInfo node_menu[]=
 	GNOMEUIINFO_ITEM(N_("_Help"), NULL, glame_canvas_filter_help,NULL),
 	GNOMEUIINFO_END
 };
+#define NODE_MENU_PROPERTIES_INDEX 0
 #define NODE_MENU_EXPAND_INDEX 2
 #define NODE_MENU_OPEN_DOWN_INDEX 3
+#define NODE_MENU_REDIRECT_PARAMETER_INDEX 5
 #define NODE_MENU_GROUP_SELECTED_INDEX 7
 #define NODE_MENU_UNGROUP_INDEX 8
 #define NODE_MENU_COPY_SELECTED_INDEX 9
@@ -1102,10 +1108,14 @@ static GtkWidget *glame_canvas_filter_get_popup_menu(GlameCanvasFilter *filter)
 	/* Build menu, disable/enable items based on context. */
 	menu = gnome_popup_menu_new(node_menu);
 
+	gtk_widget_set_sensitive(node_menu[NODE_MENU_PROPERTIES_INDEX].widget,
+				 glame_list_empty(&filter->filter->params.db.items) ? FALSE : TRUE);
 	gtk_widget_set_sensitive(node_menu[NODE_MENU_EXPAND_INDEX].widget,
 				 FILTER_IS_NETWORK(filter->filter) ? TRUE : FALSE);
 	gtk_widget_set_sensitive(node_menu[NODE_MENU_OPEN_DOWN_INDEX].widget,
 				 FILTER_IS_NETWORK(filter->filter) ? TRUE : FALSE);
+	gtk_widget_set_sensitive(node_menu[NODE_MENU_REDIRECT_PARAMETER_INDEX].widget,
+				 glame_list_empty(&filter->filter->params.db.items) ? FALSE : TRUE);
 	gtk_widget_set_sensitive(node_menu[NODE_MENU_GROUP_SELECTED_INDEX].widget,
 				 filter->selected ? TRUE : FALSE);
 	gtk_widget_set_sensitive(node_menu[NODE_MENU_UNGROUP_INDEX].widget,
