@@ -1,7 +1,7 @@
 /*
  * canvaspipe.c
  *
- * $Id: canvaspipe.c,v 1.22.2.1 2002/01/16 21:08:02 richi Exp $
+ * $Id: canvaspipe.c,v 1.27 2002/07/29 23:47:35 xwolf Exp $
  *
  * Copyright (C) 2001 Johannes Hirche
  *
@@ -22,6 +22,9 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <sys/param.h>
 #include <stdio.h>
@@ -148,11 +151,11 @@ glame_canvas_pipe_reroute(GlameCanvasPipe *p)
 	xd = p->points->coords[10];
 	yd = p->points->coords[11];
 	
-
+	xOffset = 25.0;
 	dist = xd-xs;
 	if(dist<50.0)
 		xOffset = dist/2.0;
-	xOffset=(xOffset<5.0)?5.0:xOffset;
+	xOffset=(xOffset<12.0)?12.0:xOffset;
 	
 	
 	p->points->coords[2]=xs+xOffset+(p->sourceId*4);
@@ -161,10 +164,10 @@ glame_canvas_pipe_reroute(GlameCanvasPipe *p)
 	p->points->coords[4]=xs+xOffset+(p->sourceId*4);
 	p->points->coords[5]=(ys+yd)/2.0 + dy;
 	
-	p->points->coords[6]=xd-25.0-(p->destId*4);
+	p->points->coords[6]=xd-xOffset-(p->destId*4);
 	p->points->coords[7]=(ys+yd)/2.0+dy;
 	
-	p->points->coords[8]=xd-25.0-(p->destId*4);
+	p->points->coords[8]=xd-xOffset-(p->destId*4);
 	p->points->coords[9]=yd;
 	
 	gnome_canvas_item_set(GNOME_CANVAS_ITEM(p->line),
@@ -336,6 +339,7 @@ glame_canvas_pipe_grabbing_cb(GnomeCanvasItem* i, GdkEvent* event, GlameCanvasPi
 }
 
 
+
 gboolean 
 glame_canvas_pipe_show_properties(GlameCanvasPipe* pipe)
 {
@@ -366,59 +370,218 @@ glame_canvas_pipe_show_properties(GlameCanvasPipe* pipe)
 
 	y = 10.0 + yOffset;
 	
-	DPRINTF("foo\n");
 	group = GNOME_CANVAS_GROUP(CANVAS_ITEM_ROOT(pipe));
 	group = GNOME_CANVAS_GROUP(gnome_canvas_item_new(group,
 				      gnome_canvas_group_get_type(),
 				      NULL));
 	gnome_canvas_item_raise_to_top(GNOME_CANVAS_ITEM(group));
 	params = TRUE;
-	snprintf(buffer,255,_("Samplerate: %d"),filterpipe_sample_rate(pipe->pipe));
-	text = GNOME_CANVAS_TEXT(gnome_canvas_item_new(group,
-						       gnome_canvas_text_get_type(),
-						       "x",xOffset,
-						       "y",y,
-						       "text",buffer,
-						       "font",font,
-						       "clip_width",94.0,
-						       "clip_height",16.0,
-						       "fill_color","blue",
-						       "anchor",GTK_ANCHOR_WEST,
-						       "justification",GTK_JUSTIFY_LEFT, 
-						       "clip",0,
-						       NULL));
-	gnome_canvas_item_raise_to_top(GNOME_CANVAS_ITEM(text));
-	
-	gnome_canvas_item_get_bounds(GCI(text),&recx1,&recy1,&recx2,&recy2);
-	bmaxx = bmaxx<recx2?recx2:bmaxx;
-	bmaxy = bmaxy<recy2?recy2:bmaxy;
-	bminx = bminx>recx1?recx1:bminx;
-	bminy = bminy>recy1?recy1:bminy;
-	
-	y +=16.0;
-	snprintf(buffer,255,_("Hangle: %f"),filterpipe_sample_hangle(pipe->pipe));
-	text = GNOME_CANVAS_TEXT(gnome_canvas_item_new(group,
-						       gnome_canvas_text_get_type(),
-						       "x",xOffset,
-						       "y",y,
-						       "text",buffer,
-						       "font",font,
-						       "clip_width",94.0,
-						       "clip_height",16.0,
-						       "fill_color","blue",
-						       "anchor",GTK_ANCHOR_WEST,
-						       "justification",GTK_JUSTIFY_LEFT, 
-						       "clip",0,
-						       NULL));
-	gnome_canvas_item_raise_to_top(GNOME_CANVAS_ITEM(text));
-	
-	gnome_canvas_item_get_bounds(GCI(text),&recx1,&recy1,&recx2,&recy2);
-	bmaxx = bmaxx<recx2?recx2:bmaxx;
-	bmaxy = bmaxy<recy2?recy2:bmaxy;
-	bminx = bminx>recx1?recx1:bminx;
-	bminy = bminy>recy1?recy1:bminy;
-	
-	y +=16.0;
+
+	if (filterpipe_type(pipe->pipe) == FILTER_PIPETYPE_SAMPLE) {
+		snprintf(buffer, 255, _("Samplerate: %d"),
+			 filterpipe_sample_rate(pipe->pipe));
+		text = GNOME_CANVAS_TEXT(gnome_canvas_item_new(
+			group,
+			gnome_canvas_text_get_type(),
+			"x",xOffset,
+			"y",y,
+			"text",buffer,
+			"font",font,
+			"clip_width",94.0,
+			"clip_height",16.0,
+			"fill_color","blue",
+			"anchor",GTK_ANCHOR_WEST,
+			"justification",GTK_JUSTIFY_LEFT, 
+			"clip",0,
+			NULL));
+		gnome_canvas_item_raise_to_top(GNOME_CANVAS_ITEM(text));
+		gnome_canvas_item_get_bounds(GCI(text), &recx1, &recy1,
+					     &recx2, &recy2);
+		bmaxx = bmaxx<recx2?recx2:bmaxx;
+		bmaxy = bmaxy<recy2?recy2:bmaxy;
+		bminx = bminx>recx1?recx1:bminx;
+		bminy = bminy>recy1?recy1:bminy;
+       		y +=16.0;
+
+		snprintf(buffer, 255, _("Hangle: %f"),
+			 filterpipe_sample_hangle(pipe->pipe));
+		text = GNOME_CANVAS_TEXT(gnome_canvas_item_new(
+			group,
+			gnome_canvas_text_get_type(),
+			"x",xOffset,
+			"y",y,
+			"text",buffer,
+			"font",font,
+			"clip_width",94.0,
+			"clip_height",16.0,
+			"fill_color","blue",
+			"anchor",GTK_ANCHOR_WEST,
+			"justification",GTK_JUSTIFY_LEFT, 
+			"clip",0,
+			NULL));
+		gnome_canvas_item_raise_to_top(GNOME_CANVAS_ITEM(text));
+       		gnome_canvas_item_get_bounds(GCI(text), &recx1, &recy1,
+					     &recx2, &recy2);
+		bmaxx = bmaxx<recx2?recx2:bmaxx;
+		bmaxy = bmaxy<recy2?recy2:bmaxy;
+		bminx = bminx>recx1?recx1:bminx;
+		bminy = bminy>recy1?recy1:bminy;
+ 		y +=16.0;
+	}
+
+	else if (filterpipe_type(pipe->pipe) == FILTER_PIPETYPE_FFT) {
+		snprintf(buffer, 255, _("Samplerate: %d"),
+			 filterpipe_fft_rate(pipe->pipe));
+		text = GNOME_CANVAS_TEXT(gnome_canvas_item_new(
+			group,
+			gnome_canvas_text_get_type(),
+			"x",xOffset,
+			"y",y,
+			"text",buffer,
+			"font",font,
+			"clip_width",94.0,
+			"clip_height",16.0,
+			"fill_color","blue",
+			"anchor",GTK_ANCHOR_WEST,
+			"justification",GTK_JUSTIFY_LEFT, 
+			"clip",0,
+			NULL));
+		gnome_canvas_item_raise_to_top(GNOME_CANVAS_ITEM(text));
+		gnome_canvas_item_get_bounds(GCI(text), &recx1, &recy1,
+					     &recx2, &recy2);
+		bmaxx = bmaxx<recx2?recx2:bmaxx;
+		bmaxy = bmaxy<recy2?recy2:bmaxy;
+		bminx = bminx>recx1?recx1:bminx;
+		bminy = bminy>recy1?recy1:bminy;
+		y +=16.0;
+
+		snprintf(buffer, 255, _("Hangle: %f"),
+			 filterpipe_fft_hangle(pipe->pipe));
+		text = GNOME_CANVAS_TEXT(gnome_canvas_item_new(
+			group,
+			gnome_canvas_text_get_type(),
+			"x",xOffset,
+			"y",y,
+			"text",buffer,
+			"font",font,
+			"clip_width",94.0,
+			"clip_height",16.0,
+			"fill_color","blue",
+			"anchor",GTK_ANCHOR_WEST,
+			"justification",GTK_JUSTIFY_LEFT, 
+			"clip",0,
+			NULL));
+		gnome_canvas_item_raise_to_top(GNOME_CANVAS_ITEM(text));
+       		gnome_canvas_item_get_bounds(GCI(text), &recx1, &recy1,
+					     &recx2, &recy2);
+		bmaxx = bmaxx<recx2?recx2:bmaxx;
+		bmaxy = bmaxy<recy2?recy2:bmaxy;
+		bminx = bminx>recx1?recx1:bminx;
+		bminy = bminy>recy1?recy1:bminy;
+ 		y +=16.0;
+
+		snprintf(buffer, 255, _("FFT blocksize: %i"),
+			 filterpipe_fft_bsize(pipe->pipe));
+		text = GNOME_CANVAS_TEXT(gnome_canvas_item_new(
+			group,
+			gnome_canvas_text_get_type(),
+			"x",xOffset,
+			"y",y,
+			"text",buffer,
+			"font",font,
+			"clip_width",94.0,
+			"clip_height",16.0,
+			"fill_color","blue",
+			"anchor",GTK_ANCHOR_WEST,
+			"justification",GTK_JUSTIFY_LEFT, 
+			"clip",0,
+			NULL));
+		gnome_canvas_item_raise_to_top(GNOME_CANVAS_ITEM(text));
+		gnome_canvas_item_get_bounds(GCI(text), &recx1, &recy1,
+					     &recx2, &recy2);
+		bmaxx = bmaxx<recx2?recx2:bmaxx;
+		bmaxy = bmaxy<recy2?recy2:bmaxy;
+		bminx = bminx>recx1?recx1:bminx;
+		bminy = bminy>recy1?recy1:bminy;
+		y +=16.0;
+
+		snprintf(buffer, 255, _("FFT oversampling: %i"),
+			 filterpipe_fft_osamp(pipe->pipe));
+		text = GNOME_CANVAS_TEXT(gnome_canvas_item_new(
+			group,
+			gnome_canvas_text_get_type(),
+			"x",xOffset,
+			"y",y,
+			"text",buffer,
+			"font",font,
+			"clip_width",94.0,
+			"clip_height",16.0,
+			"fill_color","blue",
+			"anchor",GTK_ANCHOR_WEST,
+			"justification",GTK_JUSTIFY_LEFT, 
+			"clip",0,
+			NULL));
+		gnome_canvas_item_raise_to_top(GNOME_CANVAS_ITEM(text));
+		gnome_canvas_item_get_bounds(GCI(text), &recx1, &recy1,
+					     &recx2, &recy2);
+		bmaxx = bmaxx<recx2?recx2:bmaxx;
+		bmaxy = bmaxy<recy2?recy2:bmaxy;
+		bminx = bminx>recx1?recx1:bminx;
+		bminy = bminy>recy1?recy1:bminy;
+		y +=16.0;
+	}
+
+	else if (filterpipe_type(pipe->pipe) == FILTER_PIPETYPE_SSP) {
+		snprintf(buffer, 255, _("Samplerate: %d"),
+			 filterpipe_ssp_rate(pipe->pipe));
+		text = GNOME_CANVAS_TEXT(gnome_canvas_item_new(
+			group,
+			gnome_canvas_text_get_type(),
+			"x",xOffset,
+			"y",y,
+			"text",buffer,
+			"font",font,
+			"clip_width",94.0,
+			"clip_height",16.0,
+			"fill_color","blue",
+			"anchor",GTK_ANCHOR_WEST,
+			"justification",GTK_JUSTIFY_LEFT, 
+			"clip",0,
+			NULL));
+		gnome_canvas_item_raise_to_top(GNOME_CANVAS_ITEM(text));
+		gnome_canvas_item_get_bounds(GCI(text), &recx1, &recy1,
+					     &recx2, &recy2);
+		bmaxx = bmaxx<recx2?recx2:bmaxx;
+		bmaxy = bmaxy<recy2?recy2:bmaxy;
+		bminx = bminx>recx1?recx1:bminx;
+		bminy = bminy>recy1?recy1:bminy;
+		y +=16.0;
+
+		snprintf(buffer, 255, _("SSP blocksize: %i"),
+			 filterpipe_ssp_bsize(pipe->pipe));
+		text = GNOME_CANVAS_TEXT(gnome_canvas_item_new(
+			group,
+			gnome_canvas_text_get_type(),
+			"x",xOffset,
+			"y",y,
+			"text",buffer,
+			"font",font,
+			"clip_width",94.0,
+			"clip_height",16.0,
+			"fill_color","blue",
+			"anchor",GTK_ANCHOR_WEST,
+			"justification",GTK_JUSTIFY_LEFT, 
+			"clip",0,
+			NULL));
+		gnome_canvas_item_raise_to_top(GNOME_CANVAS_ITEM(text));
+       		gnome_canvas_item_get_bounds(GCI(text), &recx1, &recy1,
+					     &recx2, &recy2);
+		bmaxx = bmaxx<recx2?recx2:bmaxx;
+		bmaxy = bmaxy<recy2?recy2:bmaxy;
+		bminx = bminx>recx1?recx1:bminx;
+		bminy = bminy>recy1?recy1:bminy;
+ 		y +=16.0;
+	}
 	
 	if(filterparamdb_nrparams(filterpipe_sourceparamdb(pipe->pipe))){
 		params = TRUE;
