@@ -1,7 +1,7 @@
 /*
  * canvasport.c
  *
- * $Id: canvasport.c,v 1.27.2.1 2002/01/16 21:08:02 richi Exp $
+ * $Id: canvasport.c,v 1.31 2002/07/14 12:41:19 richi Exp $
  *
  * Copyright (C) 2001 Johannes Hirche
  *
@@ -21,6 +21,11 @@
  *
  *
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <sys/param.h>
 #include <stdio.h>
 #include <math.h>
@@ -620,7 +625,7 @@ glame_canvas_port_show_properties(GlameCanvasPort* port)
 							 NULL));
 	
 	sprintf(buffer,"Label: %s",filterport_label(port->port));
-	fprintf(stderr,"%f %f %f %f\n",xs,ys,xe,ye);
+	DPRINTF("%f %f %f %f\n",xs,ys,xe,ye);
 	text = GNOME_CANVAS_TEXT(gnome_canvas_item_new(group,
 							     gnome_canvas_text_get_type(),
 							     "x",0.0, //xs + filterport_is_input(port->port)?-4.0:20.0,
