@@ -1,7 +1,7 @@
 /*
  * filtereditgui.c
  *
- * $Id: filtereditgui.c,v 1.48.2.3 2002/01/27 15:43:13 richi Exp $
+ * $Id: filtereditgui.c,v 1.59.2.1 2003/04/19 18:23:19 richi Exp $
  *
  * Copyright (C) 2001 Johannes Hirche
  *
@@ -21,10 +21,18 @@
  *
  *
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <sys/param.h>
 #include <stdio.h>
 #include <math.h>
 #include <gnome.h>
+#ifdef HAVE_LIBSTROKE
+#include <stroke.h>
+#endif
 #include "glscript.h"
 #include "util/glame_gui_utils.h"
 #include "glame_accelerator.h"
@@ -48,13 +56,13 @@ static GlameCanvas *glcanvas;
 
 
 static void glame_canvas_execute_cb(GtkObject*foo, FiltereditGui *gui);
-static void glame_canvas_register_cb(GtkWidget* ignore, GlameCanvas* canv);
-static void glame_canvas_save_as_cb(GtkWidget* ignore, GlameCanvas* canv);
-static void glame_canvas_property_dialog_cb(GtkObject*foo, GlameCanvas* canv);
-static void glame_canvas_zoom_in_cb(GtkObject*foo,GlameCanvas* canv);
-static void glame_canvas_zoom_out_cb(GtkObject*foo, GlameCanvas* canv);
-static void glame_canvas_view_all_cb(GtkObject*foo, GlameCanvas* canv);
-static void glame_canvas_add_last_cb(GtkObject* foo, GlameCanvas* canv);
+static void glame_canvas_register_cb(GtkWidget* ignore, FiltereditGui *gui);
+static void glame_canvas_save_as_cb(GtkWidget* ignore, FiltereditGui *gui);
+static void glame_canvas_property_dialog_cb(GtkObject*foo, FiltereditGui *gui);
+static void glame_canvas_zoom_in_cb(GtkObject*foo, FiltereditGui *gui);
+static void glame_canvas_zoom_out_cb(GtkObject*foo, FiltereditGui *gui);
+static void glame_canvas_view_all_cb(GtkObject*foo, FiltereditGui *gui);
+static void glame_canvas_add_last_cb(GtkObject* foo,  FiltereditGui *gui);
 static void window_close(GtkWidget *dummy, GtkWidget* window);
 
 
@@ -69,6 +77,38 @@ add_filter_by_plugin_cb(GtkWidget*wid, plugin_t *plugin)
 	glame_canvas_filter_move(filter,eventx, eventy);
 }
 
+/* static gboolean */
+/* button_on_item_event(GlameCanvas* canvas, GdkEvent* event, GnomeCanvasItem* item) */
+/* { */
+/* 	switch(event->button.button){ */
+/* 	case 1: */
+
+#ifdef HAVE_LIBSTROKE		
+static guint button_down_sighandlerid;
+
+static gboolean 
+button_down_cb(GnomeCanvas * canvas, GdkEvent *event, GlameCanvas* glCanv)
+{
+	char translation[50];
+	switch(event->type){
+	case GDK_BUTTON_RELEASE:
+		if(stroke_trans(translation)){
+			DPRINTF("translation: %s\n",translation);
+		}else{
+			DPRINTF("failed translation: %s\n",translation);
+		}
+		gtk_signal_disconnect(canvas,button_down_sighandlerid);
+		break;
+	case GDK_MOTION_NOTIFY:
+		stroke_record(event->button.x, event->button.y);
+		break;
+	default:
+		break;
+	}
+	return FALSE;
+}
+#endif
+		
 #if 0
 static void group_all(GlameCanvas* canv)
 {
@@ -199,12 +239,19 @@ root_event(GnomeCanvas * canvas, GdkEvent *event, GlameCanvas* glCanv)
 				}
 			}
 			break;
+		case 2:
+			if(!onItem){
+#ifdef HAVE_LIBSTROKE			  
+			button_down_sighandlerid = gtk_signal_connect(GTK_OBJECT(canvas),"event",GTK_SIGNAL_FUNC(button_down_cb),glCanv);
+#endif
+			}
+			break;
 		case 3:
 			if(!onItem){
 				GtkWidget* edit;
 				menu = GTK_WIDGET(glame_gui_build_plugin_menu(NULL, add_filter_by_plugin_cb));
 				if (glCanv->clipBoard) {
-					edit = gtk_menu_item_new_with_label(_("Paste"));
+					edit = gtk_menu_item_new_with_label(_("_Paste"));
 					gtk_widget_show(edit);
 					gtk_signal_connect(GTK_OBJECT(edit),"activate",filtereditgui_paste_cb,glCanv);
 					gtk_menu_append(GTK_MENU(menu), edit);
@@ -279,8 +326,9 @@ static void redirection_list_param_cb(GtkCList *list, gint row, gint column,
 
 }
 
-void glame_canvas_property_dialog_cb(GtkObject* foo, GlameCanvas *canvas)
+void glame_canvas_property_dialog_cb(GtkObject* foo, FiltereditGui *window)
 {
+	GlameCanvas *canvas = window->canvas;
 	GtkWidget * dialog;
 	GtkWidget * vbox;
 	GtkWidget * notebook;
@@ -393,22 +441,18 @@ glame_filtereditgui_install_accels(GtkWidget* window)
 
 static SCM gls_editfilter_zoom_in(SCM parms)
 {
-	if(gh_number_p(parms)){
-		glame_canvas_set_zoom(glcanvas,GNOME_CANVAS(glcanvas)->pixels_per_unit*gh_scm2double(parms));
-	} else {
-		/* choosing default */
-		glame_canvas_zoom_in_cb(NULL,glcanvas);
-	}
+	double factor = 1.5;
+	if (gh_number_p(parms))
+		factor = gh_scm2double(parms);
+	glame_canvas_set_zoom(glcanvas,GNOME_CANVAS(glcanvas)->pixels_per_unit*factor);
 	return SCM_UNSPECIFIED;
 }
 static SCM gls_editfilter_zoom_out(SCM parms)
 {
-	if(gh_number_p(parms)){
-		glame_canvas_set_zoom(glcanvas,GNOME_CANVAS(glcanvas)->pixels_per_unit/gh_scm2double(parms));
-	} else {
-		/* choosing default */
-		glame_canvas_zoom_out_cb(NULL,glcanvas);
-	}
+	double factor = 1.5;
+	if (gh_number_p(parms))
+		factor = gh_scm2double(parms);
+	glame_canvas_set_zoom(glcanvas,GNOME_CANVAS(glcanvas)->pixels_per_unit/factor);
 	return SCM_UNSPECIFIED;
 }
 		
@@ -436,11 +480,11 @@ static SCM gls_editfilter_delete_selection()
 static SCM gls_editfilter_get_selection()
 {
 	GList *selected;
-	SCM s_items = SCM_LIST0;
+	SCM s_items = SCM_EOL;
 	selected = glame_canvas_get_selected_items(glcanvas);
 	while(selected){
 		filter_t *filter = (filter_t *)selected->data;
-		s_items = gh_cons(filter2scm(filter), s_items);
+		s_items = scm_cons(filter2scm(filter), s_items);
 		selected = g_list_next(selected);
 	}
 	return s_items;
@@ -597,6 +641,93 @@ GtkType filteredit_gui_get_type(void)
 	return filteredit_gui_type;
 }	
 
+
+static void glame_canvas_copy_selected_w_cb(GtkWidget *w, FiltereditGui *gui)
+{
+	glame_accel_widget_data_cb(NULL, "filteredit/y");
+}
+static void glame_canvas_paste_selected_w_cb(GtkWidget *w, FiltereditGui *gui)
+{
+	glame_accel_widget_data_cb(NULL, "filteredit/p");
+}
+static void glame_canvas_delete_selected_w_cb(GtkWidget *w, FiltereditGui *gui)
+{
+	glame_gh_safe_eval_str("(editfilter-delete-selection)");
+}
+static void glame_canvas_group_selected_w_cb(GtkWidget *w, FiltereditGui *gui)
+{
+	glame_gh_safe_eval_str("(editfilter-group-selection)");
+}
+static void glame_canvas_collapse_selected_w_cb(GtkWidget *w, FiltereditGui *gui)
+{
+	glame_gh_safe_eval_str("(editfilter-collapse-selection)");
+}
+static void help_cb(GtkWidget *menu, void *blah)
+{
+	gnome_help_goto(NULL, "info:glame#The_Filternetwork_Editor");
+}
+static void list_keybindings_cb(GtkWidget *menu, void *blah)
+{
+	glame_accel_widget_data_cb(NULL, "list_keybindings_filteredit");
+}
+
+static GnomeUIInfo window_file_menu[] = {
+        GNOMEUIINFO_ITEM(N_("Save as..."), NULL, glame_canvas_save_as_cb, NULL),
+        GNOMEUIINFO_ITEM(N_("Register..."), NULL, glame_canvas_register_cb, NULL),
+        GNOMEUIINFO_ITEM(N_("Properties..."), NULL, glame_canvas_property_dialog_cb, NULL),
+        GNOMEUIINFO_SEPARATOR,
+        GNOMEUIINFO_ITEM(N_("Execute"), NULL, glame_canvas_execute_cb, NULL),
+        GNOMEUIINFO_SEPARATOR,
+        GNOMEUIINFO_MENU_CLOSE_ITEM(window_close, NULL),
+        GNOMEUIINFO_END
+};
+static GnomeUIInfo window_edit_menu[] = {
+        GNOMEUIINFO_ITEM(N_("Copy"), NULL, glame_canvas_copy_selected_w_cb, NULL),
+        GNOMEUIINFO_ITEM(N_("Paste"), NULL, glame_canvas_paste_selected_w_cb, NULL),
+        GNOMEUIINFO_ITEM(N_("Delete"), NULL, glame_canvas_delete_selected_w_cb, NULL),
+	GNOMEUIINFO_SEPARATOR,
+        GNOMEUIINFO_ITEM(N_("Group"), NULL, glame_canvas_group_selected_w_cb, NULL),
+        GNOMEUIINFO_ITEM(N_("Collapse"), NULL, glame_canvas_collapse_selected_w_cb, NULL),
+        GNOMEUIINFO_END
+};
+static GnomeUIInfo window_view_menu[] = {
+        GNOMEUIINFO_ITEM(N_("Zoom in"), NULL, glame_canvas_zoom_in_cb, NULL),
+        GNOMEUIINFO_ITEM(N_("Zoom out"), NULL, glame_canvas_zoom_out_cb, NULL),
+        GNOMEUIINFO_ITEM(N_("View all"), NULL, glame_canvas_view_all_cb, NULL),
+        GNOMEUIINFO_END
+};
+static GnomeUIInfo window_help_menu[] = {
+	GNOMEUIINFO_ITEM(N_("_Help"),N_("Opens a gnome help browser"), help_cb, NULL),
+	GNOMEUIINFO_ITEM(N_("List key-bindings"), N_("Lists the current key-bindings"), list_keybindings_cb, NULL),
+	GNOMEUIINFO_END
+};
+static GnomeUIInfo window_menu[] = {
+	{
+            GNOME_APP_UI_SUBTREE, N_("_Network"),
+            NULL,
+            window_file_menu, NULL, NULL,
+            GNOME_APP_PIXMAP_NONE, NULL,
+            0, 0, NULL
+        },
+	{
+            GNOME_APP_UI_SUBTREE, N_("_Edit"),
+            NULL,
+            window_edit_menu, NULL, NULL,
+            GNOME_APP_PIXMAP_NONE, NULL,
+            0, 0, NULL
+        },
+	{
+            GNOME_APP_UI_SUBTREE, N_("_View"),
+            NULL,
+            window_view_menu, NULL, NULL,
+            GNOME_APP_PIXMAP_NONE, NULL,
+            0, 0, NULL
+        },
+	GNOMEUIINFO_MENU_HELP_TREE (window_help_menu),
+	GNOMEUIINFO_END
+};
+
+
 GtkWidget * 
 glame_filtereditgui_new(filter_t *net, gboolean protected)
 {
@@ -611,7 +742,7 @@ glame_filtereditgui_new(filter_t *net, gboolean protected)
 		name = "Untitled";
 
 	window = FILTEREDIT_GUI(gtk_type_new(filteredit_gui_get_type()));
-	gnome_app_construct(GNOME_APP(window), "glame0.5", name);
+	gnome_app_construct(GNOME_APP(window), "glame0.7", name);
 
 	glame_filtereditgui_install_accels(GTK_WIDGET(window));
 	dock = GNOME_DOCK(GNOME_APP(window)->dock);
@@ -639,36 +770,49 @@ glame_filtereditgui_new(filter_t *net, gboolean protected)
 
 	gtk_container_add(GTK_CONTAINER(sw),canvas);
 	
-	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),_("Execute"),_("Executes Filternetwork"),"foo",gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_EXEC),glame_canvas_execute_cb,window);
+	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),_("Execute"),_("Executes Filternetwork"),"foo",gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_EXEC),
+		glame_canvas_execute_cb, window);
 	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
-	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),_("Register"),_("Registers actual filternetwork"),"foo",gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_CONVERT),glame_canvas_register_cb,canvas);
+	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),_("Register"),_("Registers actual filternetwork"),"foo",gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_CONVERT),
+		glame_canvas_register_cb, window);
 	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
 	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),_("Save"),_("Saves Filternetwork"),"foo",
 				glame_load_icon_widget("save.png",24,24),
-				glame_canvas_save_as_cb,canvas);
+				glame_canvas_save_as_cb, window);
 	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
-	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),_("Properties"),_("Edit Filternetwork Properties"),"foo",gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_PROPERTIES),glame_canvas_property_dialog_cb,canvas);
+	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),_("Properties"),_("Edit Filternetwork Properties"),"foo",gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_PROPERTIES),
+		glame_canvas_property_dialog_cb, window);
 	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
 	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),_("Zoom in"),_("Zooms in"),"foo",
 				glame_load_icon_widget("zoom_in.png",24,24),
-				glame_canvas_zoom_in_cb,canvas);
+				glame_canvas_zoom_in_cb, window);
 	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),_("Zoom out"),_("Zooms out"),"foo",
 				glame_load_icon_widget("zoom_out.png",24,24),
-				glame_canvas_zoom_out_cb,canvas);
-	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),_("View all"),_("Adjusts scroll region"),"foo",gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_REFRESH),glame_canvas_view_all_cb,canvas);
+				glame_canvas_zoom_out_cb, window);
+	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),_("View all"),_("Adjusts scroll region"),"foo",gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_REFRESH),glame_canvas_view_all_cb, window);
 	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
 
 	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),_("Add last"),_("Adds last filter"),"foo",gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_MULTIPLE),
-				glame_canvas_add_last_cb, canvas);
+				glame_canvas_add_last_cb, window);
 
 	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
 	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),_("Close"),_("Close"),"foo",gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_CLOSE),window_close,window);
 	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
 	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),_("Help"),_("Help"),"foo",gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_HELP),gnome_help_goto,"info:glame#The_Filternetwork_Editor");
+
+        /* Create menubar - FIXME copy all uiinfos, restructure to
+	 * match nice menu layout, etc.
+	 * menu hints not w/o status bar */
+        gnome_app_create_menus_with_data(GNOME_APP(window), window_menu, window);
+        //gnome_app_install_menu_hints(GNOME_APP(window), window_menu);
+
+	gnome_app_set_toolbar(GNOME_APP(window), GTK_TOOLBAR(toolbar));
+#if 0
 	gnome_app_add_toolbar(GNOME_APP(window), GTK_TOOLBAR(toolbar),
 			      "canvas::toolbar",
 			      GNOME_DOCK_ITEM_BEH_EXCLUSIVE|GNOME_DOCK_ITEM_BEH_NEVER_FLOATING,
 			      GNOME_DOCK_TOP, 0, 0, 0);
+#endif
 
 	gnome_app_set_contents(GNOME_APP(window),sw);	
 	gtk_widget_show(GTK_WIDGET(dock));
@@ -676,7 +820,9 @@ glame_filtereditgui_new(filter_t *net, gboolean protected)
 	gtk_window_set_default_size(GTK_WINDOW(window),400,300);
 	gtk_widget_show_all(GTK_WIDGET(window));
 	gtk_signal_connect(GTK_OBJECT(canvas),"event",GTK_SIGNAL_FUNC(root_event),canvas);
-
+#ifdef HAVE_LIBSTROKE
+	stroke_init();
+#endif
 	return GTK_WIDGET(window);
 }
 
@@ -719,7 +865,7 @@ static void execute_cleanup(glsig_handler_t *handler, long sig, va_list va)
 		return;
 
 	gui->pm_playing = 0;
-	filter_terminate(gui->canvas->net);
+	filter_terminate(gui->canvas->net->launch_context);
 
 	/* Scan network for swapfile_out nodes and issue gpsm invalidate
 	 * signals. */
@@ -734,10 +880,10 @@ static void execute_cleanup(glsig_handler_t *handler, long sig, va_list va)
 			continue;
 		DPRINTF("Found swapfile_out node, issuing invalidate\n");
 		gpsm_notify_swapfile_change(
-			filterparam_val_int(filename),
-			filterparam_val_int(changed_start),
-			filterparam_val_int(changed_end)
-			- filterparam_val_int(changed_start) + 1);
+			filterparam_val_long(filename),
+			filterparam_val_long(changed_start),
+			filterparam_val_long(changed_end)
+			- filterparam_val_long(changed_start) + 1);
 	}
 
 	gtk_widget_destroy(g_list_nth(gtk_container_children(
@@ -757,7 +903,7 @@ static void glame_canvas_execute_cb(GtkObject* foo, FiltereditGui *gui)
 	if (gui->pm_playing) {
 		/* Playing - abort the network, cleanup will
 		 * happen automagically. */
-		filter_terminate(gui->canvas->net);
+		filter_terminate(gui->canvas->net->launch_context);
 		return;
 	}
 
@@ -784,8 +930,9 @@ static void glame_canvas_execute_cb(GtkObject* foo, FiltereditGui *gui)
 				glame_canvas_execute_cb, gui, 0);
 }
 
-static void glame_canvas_save_as_cb(GtkWidget*ignore,GlameCanvas *canvas)
+static void glame_canvas_save_as_cb(GtkWidget*ignore, FiltereditGui *window)
 {
+	GlameCanvas *canvas = window->canvas;
 	GtkWidget *dialog;
 	GtkWidget *dialogVbox;
 	char filenamebuffer[256] = "";
@@ -819,12 +966,12 @@ static void glame_canvas_save_as_cb(GtkWidget*ignore,GlameCanvas *canvas)
 	outf = fopen(filenamebuffer,"w");
 	buffer = filter_to_string(GLAME_CANVAS(canvas)->net);
 	DPRINTF("Network .scm is:\n%s\n", buffer);
-	fprintf(outf, "(let ((newplugin (glame_plugin_define %s \"%s\")\n)) (if (filter? newplugin) newplugin (plugin_set newplugin PLUGIN_CATEGORY \"%s\")))", buffer, filternamebuffer, categorynamebuffer);
+	fprintf(outf, "(let ((newplugin (glame_plugin_define %s \"%s\")\n)) (if (filter? newplugin) newplugin (plugin-set! newplugin PLUGIN_CATEGORY \"%s\")))", buffer, filternamebuffer, categorynamebuffer);
 	free(buffer);
 	fclose(outf);
 }
 
-static void glame_canvas_register_as_cb(gchar* name, GlameCanvas* glCanv)
+static void glame_canvas_register_as_cb(gchar* name, GlameCanvas *glCanv)
 {
 	plugin_t *newplug;
 	filter_t *copy;
@@ -835,36 +982,41 @@ static void glame_canvas_register_as_cb(gchar* name, GlameCanvas* glCanv)
 	filter_register(copy,newplug);
 }
 
-static void glame_canvas_register_cb(GtkWidget*ignore, GlameCanvas* canvas)
+static void glame_canvas_register_cb(GtkWidget*ignore, FiltereditGui *window)
 {	
+	GlameCanvas *canvas = window->canvas;
 	gnome_request_dialog(0, _("Filtername"), "filter", 16,
 			     (GnomeStringCallback)glame_canvas_register_as_cb,
 			     canvas, NULL);
 }
 
-static void glame_canvas_zoom_in_cb(GtkObject*foo,GlameCanvas* canv)
+static void glame_canvas_zoom_in_cb(GtkObject*foo, FiltereditGui *window)
 {
+	GlameCanvas *canv = window->canvas;
 	glame_canvas_set_zoom(canv,GNOME_CANVAS(canv)->pixels_per_unit*1.5);
 }
 
-static void glame_canvas_zoom_out_cb(GtkObject*foo, GlameCanvas* canv)
+static void glame_canvas_zoom_out_cb(GtkObject*foo, FiltereditGui *window)
 {
+	GlameCanvas *canv = window->canvas;
 	glame_canvas_set_zoom(canv,GNOME_CANVAS(canv)->pixels_per_unit/1.5);
 }
 
-static void glame_canvas_view_all_cb(GtkObject*foo, GlameCanvas* canv)
+static void glame_canvas_view_all_cb(GtkObject*foo, FiltereditGui *window)
 {
+	GlameCanvas *canv = window->canvas;
 	glame_canvas_view_all(canv);
 }
 
-static void glame_canvas_add_last_cb(GtkObject* foo, GlameCanvas* canv)
+static void glame_canvas_add_last_cb(GtkObject* foo, FiltereditGui *window)
 {
+	GlameCanvas *canv = window->canvas;
 	glame_canvas_add_last(canv);
 }
 
 static void window_close(GtkWidget *dummy, GtkWidget* window)
 {
 	if (FILTEREDIT_GUI(window)->pm_playing)
-		filter_terminate(FILTEREDIT_GUI(window)->canvas->net);
+		filter_terminate(FILTEREDIT_GUI(window)->canvas->net->launch_context);
 	gtk_widget_destroy(GTK_WIDGET(window));
 }
