@@ -1,7 +1,7 @@
 /*
  * canvasitem.c
  *
- * $Id: glamecanvas.c,v 1.45 2001/12/13 14:48:06 richi Exp $
+ * $Id: glamecanvas.c,v 1.47.2.1 2003/04/19 18:23:20 richi Exp $
  *
  * Copyright (C) 2001 Johannes Hirche
  *
@@ -22,6 +22,10 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <sys/param.h>
 #include <stdio.h>
 #include <math.h>
@@ -56,7 +60,7 @@ glame_canvas_destroy (GtkObject *object)
 	hash_remove_gcanvas(GLAME_CANVAS(object));
 	//DPRINTF("Destroying glamecanvas\n");
 	if (!GLAME_CANVAS(object)->openedUp) {
-		filter_terminate(GLAME_CANVAS(object)->net);
+		filter_terminate(GLAME_CANVAS(object)->net->launch_context);
 		filter_delete(GLAME_CANVAS(object)->net);
 	}
 	parent_class = gtk_type_class(GNOME_TYPE_CANVAS);
@@ -228,14 +232,13 @@ void glame_canvas_marshal_NONE__DOUBLE_DOUBLE(GtkObject* obj,
 }
 	
 
-/* returns a newly allocated string identifier. */
-char* 
-glame_gui_get_font(GlameCanvas* canv)
+/* Returns a font identifier in a static buffer which is ok, as
+ * the GUI is single threaded. */
+static char fontbuffer[256];
+char* glame_gui_get_font(GlameCanvas* canv)
 {
-	
-	char * fontbuffer = malloc(256);
-	sprintf(fontbuffer,"-adobe-helvetica-medium-r-normal-*-%d-120-*-*-p-*-iso8859-1",
-		(int)(GNOME_CANVAS(canv)->pixels_per_unit*12.0));
+	snprintf(fontbuffer, 256, "-adobe-helvetica-medium-r-normal-*-%d-120-*-*-p-*-iso8859-1",
+		 (int)(GNOME_CANVAS(canv)->pixels_per_unit*12.0));
 	return fontbuffer;
 }
 
