@@ -1,6 +1,6 @@
 /*
  * importexport.c
- * $Id: importexport.c,v 1.9.2.1 2002/01/23 20:27:38 richi Exp $
+ * $Id: importexport.c,v 1.19.2.6 2004/01/10 15:53:57 richi Exp $
  *
  * Copyright (C) 2001 Alexander Ehlert
  *
@@ -40,15 +40,22 @@
 #include "network_utils.h"
 #include "waveeditgui.h"
 #include "glame_audiofile.h"
+#ifdef HAVE_LIBMAD
+#include <mad.h>
+#endif
+#ifdef HAVE_LIBVORBISFILE
+#include <vorbis/codec.h>
+#include <vorbis/vorbisfile.h>
+#endif
 
 
      
 static char *ftlabel[] = { "raw", "aiffc", "aiff", "nextsnd", "wav", "sf", "ogg", "mp3" };
 static char *sflabel[] = { "16 bit signed", "24 bit signed", "32 bit signed", 
 			   "8 bit unsigned", "32 bit float", "64 bit float" };
-static int sf_format[] = { AF_SAMPFMT_TWOSCOMP, AF_SAMPFMT_TWOSCOMP, AF_SAMPFMT_TWOSCOMP,
+static long sf_format[] = { AF_SAMPFMT_TWOSCOMP, AF_SAMPFMT_TWOSCOMP, AF_SAMPFMT_TWOSCOMP,
 			   AF_SAMPFMT_UNSIGNED, AF_SAMPFMT_FLOAT, AF_SAMPFMT_DOUBLE };
-static int sf_width[] = { 16, 24, 32, 8, 32, 64 };
+static long sf_width[] = { 16, 24, 32, 8, 32, 64 };
 
 static char *rlabel[] = { "original", "mono", "stereo" };
 
@@ -75,20 +82,35 @@ static char *fproplabel[] = { "Format", "Samplerate", "Quality", "Channels",
 
 struct imp_s {
 	filter_t *net;
+	filter_launchcontext_t *context;
 	int cancelled;
 	int importing;
 	int destroyed;
 	gpsm_item_t *item;
 	GtkWidget *dialog;
-	GtkWidget *edit, *appbar, *checkresample, *rateentry;
+	GtkWidget *edit, *appbar, *checkresample, *rateentry, *getstats;
 	filter_t *readfile;
 	gchar *filename;
 	unsigned int frames;
-	int rate, chancnt, gotfile, gotstats;
+	long rate;
+	int chancnt, gotfile, gotstats;
 	float maxrms, dcoffset;
 	plugin_t *resample;
 	GtkWidget *fi_plabel[MAX_PROPS];
 	int reallydone;
+#ifdef HAVE_LIBMAD
+	int mad_length;
+	int mad_pos;
+	int mad_err;
+	int mad_err_cnt;
+	int mad_err_at_eof;
+	const char *mad_last_err;
+	char *mad_buffer;
+	int mad_channels;
+	swfd_t *mad_swfds;
+	int mad_bufsize;
+	char *mad_buf;
+#endif
 };
 
 
@@ -108,7 +130,7 @@ static void ie_cancel_cb(GtkWidget *bla, struct imp_s *ie)
 {
 	DPRINTF("cancel pressed\n");
 	if (ie->importing==1) {
-		filter_terminate(ie->net);
+		filter_terminate(ie->context);
 		ie->cancelled = 1;
 		DPRINTF("Network was running. Importing terminated.\n");
 	}
@@ -122,13 +144,288 @@ static gint ie_windowkilled(GtkWidget *bla,  GdkEventAny *event, gpointer data)
 {
 	struct imp_s *ie = (struct imp_s*)data;
 	if(ie->importing==1) {
-		filter_terminate(ie->net);
+		filter_terminate(ie->context);
 	}
 	ie->destroyed=1;
 	ie->cancelled=1;
 	return TRUE;
 }
 
+#ifdef HAVE_LIBVORBISFILE
+static void ie_import_ogg(struct imp_s *ie)
+{
+	OggVorbis_File vf;
+	vorbis_info *vi;
+        FILE *fd;
+	gpsm_swfile_t *swfile[2];
+	swfd_t swfd[2];
+	long ret, pos;
+	int i;
+
+	fd = fopen(ie->filename, "r");
+	if (fd == NULL)
+		return;
+	if (ov_open(fd, &vf, NULL, 0) < 0) {
+		fclose(fd);
+		return;
+	}
+	vi = ov_info(&vf, -1);
+	if (vi->channels > 2) {
+		ov_clear(&vf);
+		fclose(fd);
+		return;
+	}
+	/* vi->channels, vi->rate */
+	/* ov_pcm_total(&vf, -1);  decoded size in samples */
+
+	/* alloc gpsm group, etc. */
+	ie->item = (gpsm_item_t *)gpsm_newgrp(g_basename(ie->filename));
+	for (i=0; i<vi->channels; ++i) {
+		char label[16];
+		snprintf(label, 16, "channel-%i", i);
+		swfile[i] = gpsm_newswfile(label);
+		gpsm_swfile_set(swfile[i], vi->rate,
+				vi->channels == 2
+				? (i == 0
+				   ? FILTER_PIPEPOS_LEFT
+				   : FILTER_PIPEPOS_RIGHT)
+				: FILTER_PIPEPOS_CENTRE);
+		gpsm_item_place((gpsm_grp_t *)ie->item, (gpsm_item_t *)swfile[i], 0, i);
+		swfd[i] = sw_open(gpsm_swfile_filename(swfile[i]), O_RDWR);
+	}
+
+	/* process file */
+	ie->importing = 1;
+	pos = 0;
+	while (1) {
+		SAMPLE **ssbuf;
+		int current_section;
+		/* Vorbis folks, read my lips. Changing calling convention
+		 * without changing the function name is calling for trouble.
+		 */
+		ret = ov_read_float(&vf, &ssbuf,
+#if (GL_OV_READ_FLOAT_ARGS == 4)
+				    GLAME_BULK_BUFSIZE,
+#endif
+				    &current_section);
+		if (ret <= 0) /* error or EOF */
+			break;
+
+		/* append to swapfile */
+		for (i=0; i<vi->channels; ++i)
+			sw_write(swfd[i], ssbuf[i], SAMPLE_SIZE*ret);
+
+		/* show progress and check for user input */
+		if (pos/(1152*32) < (pos+ret)/(1152*32)) {
+			double percentage;
+			percentage = (pos % (1152*1024)) / (double)(1152*1024);
+			gnome_appbar_set_progress(GNOME_APPBAR(ie->appbar),
+						  percentage); 
+			while (gtk_events_pending())
+				gtk_main_iteration();
+			if (ie->cancelled)
+				break;
+		}
+		pos += ret;
+	}
+	while (gtk_events_pending())
+		gtk_main_iteration();
+
+	/* finish gpsm, if no error */
+	if (ret == 0 && !ie->cancelled) {
+		gpsm_item_t *item;
+		gpsm_grp_foreach_item(ie->item, item)
+			gpsm_invalidate_swapfile(gpsm_swfile_filename(item));
+	} else if (ie->cancelled) {
+		gpsm_item_destroy(ie->item);
+		ie->item = NULL;
+	} else /* if (ret < 0) */ {
+		gnome_dialog_run_and_close(GNOME_DIALOG(
+			gnome_error_dialog(_("Error importing Ogg file."))));
+		gpsm_item_destroy(ie->item);
+		ie->item = NULL;
+	}
+
+	/* clean up */
+	for (i=0; i<vi->channels; ++i)
+		sw_close(swfd[i]);
+	ov_clear(&vf);
+	fclose(fd);
+
+	ie->importing = 0;
+	ie_import_cleanup(ie);
+}
+#endif
+
+#ifdef HAVE_LIBMAD
+static enum mad_flow
+ie_import_mp3_error(void *data, struct mad_stream *stream,
+		    struct mad_frame *frame)
+{
+	struct imp_s *ie = data;
+	ie->mad_err = 1;
+	ie->mad_err_at_eof = 1;
+	ie->mad_last_err = mad_stream_errorstr(stream);
+	if (ie->mad_pos != 0)
+		ie->mad_err_cnt++;
+	DPRINTF("MAD error pos=%i: %s\n", ie->mad_pos, ie->mad_last_err);
+	/* Ignore recoverable errors. */
+	if (MAD_RECOVERABLE(stream->error))
+		return MAD_FLOW_IGNORE;
+	return MAD_FLOW_BREAK;
+}
+static enum mad_flow
+ie_import_mp3_input(void *data, struct mad_stream *stream)
+{
+	struct imp_s *ie = data;
+	if (!ie->mad_length)
+		return MAD_FLOW_STOP;
+	mad_stream_buffer(stream, ie->mad_buffer, ie->mad_length);
+	ie->mad_length = 0;
+	return MAD_FLOW_CONTINUE;
+}
+static enum mad_flow
+ie_import_mp3_output(void *data, struct mad_header const *header,
+		     struct mad_pcm *pcm)
+{
+	struct imp_s *ie = data;
+	int i;
+
+	/* Need to ignore frames with (recovered) error. */
+	if (ie->mad_err) {
+		ie->mad_err = 0;
+		return MAD_FLOW_CONTINUE;
+	}
+	ie->mad_err_at_eof = 0;
+
+	if (ie->mad_pos == 0) {
+		/* alloc gpsm group, etc. */
+		ie->item = (gpsm_item_t *)gpsm_newgrp(g_basename(ie->filename));
+		ie->mad_swfds = malloc(pcm->channels * sizeof(swfd_t));
+		ie->mad_channels = pcm->channels;
+		for (i=0; i<pcm->channels; ++i) {
+			char label[16];
+			gpsm_swfile_t *swfile;
+			snprintf(label, 16, "channel-%i", i);
+			swfile = gpsm_newswfile(label);
+			gpsm_swfile_set(swfile, pcm->samplerate,
+					pcm->channels == 2
+					? (i == 0
+					   ? FILTER_PIPEPOS_LEFT
+					   : FILTER_PIPEPOS_RIGHT)
+					: FILTER_PIPEPOS_CENTRE);
+			gpsm_item_place((gpsm_grp_t *)ie->item, (gpsm_item_t *)swfile, 0, i);
+			ie->mad_swfds[i] = sw_open(gpsm_swfile_filename(swfile), O_RDWR);
+		}
+	}
+
+	if (!ie->mad_buf
+	    || ie->mad_bufsize < pcm->length) {
+		if (ie->mad_buf)
+			free(ie->mad_buf);
+		DPRINTF("mad buffer size %i samples\n", pcm->length);
+		ie->mad_buf = malloc(SAMPLE_SIZE*pcm->length);
+		ie->mad_bufsize = pcm->length;
+	}
+	for (i=0; i<pcm->channels; ++i) {
+		unsigned int nsamples = pcm->length;
+		mad_fixed_t const *data = pcm->samples[i];
+		SAMPLE *b = (SAMPLE *)ie->mad_buf;
+
+		while (nsamples--)
+		    *b++ = mad_f_todouble(*data++);
+
+		sw_write(ie->mad_swfds[i], ie->mad_buf, SAMPLE_SIZE*pcm->length);
+	}
+
+	/* all 32 frames show some progress and check for user input */
+	if (ie->mad_pos/(1152*32) < (ie->mad_pos+pcm->length)/(1152*32)) {
+		double percentage;
+		percentage = (ie->mad_pos % (1152*1024)) / (double)(1152*1024);
+		gnome_appbar_set_progress(GNOME_APPBAR(ie->appbar), percentage); 
+		while (gtk_events_pending())
+			gtk_main_iteration();
+		if (ie->cancelled)
+			return MAD_FLOW_BREAK;
+	}
+	ie->mad_pos += pcm->length;
+
+	return MAD_FLOW_CONTINUE;
+}
+static void ie_import_mp3(struct imp_s *ie)
+{
+	int fd, i;
+	struct stat s;
+	struct mad_decoder decoder;
+	int result;
+
+	fd = open(ie->filename, O_RDONLY);
+	if (fd == -1)
+		return;
+	fstat(fd, &s);
+	ie->mad_err = 0;
+	ie->mad_err_at_eof = 0;
+	ie->mad_err_cnt = 0;
+	ie->mad_length = s.st_size;
+	ie->mad_last_err = NULL;
+	ie->mad_buffer = mmap(NULL, s.st_size, PROT_READ, MAP_PRIVATE, fd, 0);
+	if (ie->mad_buffer == MAP_FAILED) {
+		close(fd);
+		return;
+	}
+	ie->mad_bufsize = 0;
+	ie->mad_buf = NULL;
+
+	mad_decoder_init(&decoder, ie,
+			 ie_import_mp3_input, 0, 0,
+			 ie_import_mp3_output,
+			 ie_import_mp3_error, 0);
+	ie->mad_pos = 0;
+	ie->importing = 1;
+
+	result = mad_decoder_run(&decoder, MAD_DECODER_MODE_SYNC);
+	mad_decoder_finish(&decoder);
+
+	for (i=0; i<ie->mad_channels; ++i)
+		sw_close(ie->mad_swfds[i]);
+	if (ie->mad_buf)
+		free(ie->mad_buf);
+
+	munmap(ie->mad_buffer, s.st_size);
+	close(fd);
+
+	while (gtk_events_pending())
+		gtk_main_iteration();
+
+	if (result == 0) {
+		gpsm_item_t *item;
+		gpsm_grp_foreach_item(ie->item, item)
+			gpsm_invalidate_swapfile(gpsm_swfile_filename(item));
+		if (ie->mad_err_cnt - ie->mad_err_at_eof > 0) {
+			char errmsg[1024];
+			snprintf(errmsg, 1024, _("Had %i recoverable errors, last was: %s"),
+				 ie->mad_err_cnt, ie->mad_last_err);
+			gnome_dialog_run_and_close(GNOME_DIALOG(
+				gnome_warning_dialog(errmsg)));
+		}
+	} else if (ie->cancelled) {
+		gpsm_item_destroy(ie->item);
+		ie->item = NULL;
+	} else {
+		char errmsg[1024];
+		snprintf(errmsg, 1024, _("Failed importing mp3 file: %s"),
+			 ie->mad_last_err);
+		gnome_dialog_run_and_close(GNOME_DIALOG(
+			gnome_error_dialog(errmsg)));
+		gpsm_item_destroy(ie->item);
+		ie->item = NULL;
+	}
+
+	ie->importing = 0;
+	ie_import_cleanup(ie);
+}
+#endif
+
 static void ie_import_cb(GtkWidget *bla, struct imp_s *ie)
 {
 	filter_t *readfile, *swout, *resample;
@@ -137,7 +434,8 @@ static void ie_import_cb(GtkWidget *bla, struct imp_s *ie)
 	filter_param_t *pos_param;
 	float percentage;
 	gpsm_grp_t *group = NULL;
-	int i, newrate;
+	int i;
+	long newrate;
 	gboolean dorsmpl;
 	gpsm_item_t *it;
 	GtkWidget *ed;
@@ -150,13 +448,31 @@ static void ie_import_cb(GtkWidget *bla, struct imp_s *ie)
 	//ie->importing = 0;
 
 	if(ie->gotfile==0) {
-		ed=gnome_error_dialog("Select a file first!");
+		ed = gnome_error_dialog("Select a file first!");
 		gnome_dialog_set_parent(GNOME_DIALOG(ed), GTK_WINDOW(ie->dialog));
 		gnome_dialog_run_and_close(GNOME_DIALOG(ed));
-		DPRINTF("no file given (%p)\n", ie->filename);
+		return;
+	} else if (ie->gotfile==2) {
+#ifdef HAVE_LIBMAD
+		ie_import_mp3(ie);
+#else
+		ed = gnome_error_dialog("Sorry, no mp3 support.\nInstall libmad and rebuild.");
+		gnome_dialog_set_parent(GNOME_DIALOG(ed), GTK_WINDOW(ie->dialog));
+		gnome_dialog_run_and_close(GNOME_DIALOG(ed));
+#endif
+		return;
+	} else if (ie->gotfile==3) {
+#ifdef HAVE_LIBVORBISFILE
+		ie_import_ogg(ie);
+#else
+		ed = gnome_error_dialog("Sorry, no ogg support.\nInstall libvorbisfile and rebuild.");
+		gnome_dialog_set_parent(GNOME_DIALOG(ed), GTK_WINDOW(ie->dialog));
+		gnome_dialog_run_and_close(GNOME_DIALOG(ed));
+#endif
 		return;
 	}
-	DPRINTF("%i file %p\n", ie->gotfile, ie->filename);
+
+	DPRINTF("Importing via read-file plugin\n");
 
 	ie->net = filter_creat(NULL);
 	ie->importing = 1;
@@ -177,7 +493,7 @@ static void ie_import_cb(GtkWidget *bla, struct imp_s *ie)
 
 	if ((ie->resample) && (dorsmpl)) {
 		newrate = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ie->rateentry));
-		DPRINTF("Resample to %d Hz\n", newrate);
+		DPRINTF("Resample to %li Hz\n", newrate);
 		if(ie->rate==newrate) {
 			DPRINTF("new samplerate equals old samplerate\n");
 			dorsmpl=FALSE;
@@ -244,23 +560,24 @@ static void ie_import_cb(GtkWidget *bla, struct imp_s *ie)
 	gnome_appbar_set_status(GNOME_APPBAR(ie->appbar), "Importing");
 	
 
-	if ((filter_launch(ie->net, GLAME_BULK_BUFSIZE) == -1) ||
-	    (filter_start(ie->net) == -1))
+	if (!(ie->context = filter_launch(ie->net, GLAME_BULK_BUFSIZE)) ||
+	    (filter_start(ie->context) == -1))
 		goto ie_fail_cleanup;
 
-	while(!filter_is_ready(ie->net)) {
+	while(!filter_is_ready(ie->context)) {
 		while (gtk_events_pending())
 			gtk_main_iteration();
 		
 		usleep(40000);
-		percentage = (float)filterparam_val_pos(pos_param)/(float)ie->frames;
+		percentage = (float)filterparam_val_long(pos_param)/(float)ie->frames;
 		if(percentage>1.0)
 			percentage = 1.0;
 		gnome_appbar_set_progress(GNOME_APPBAR(ie->appbar),
 					  percentage);
 	}
-	
+	filter_wait(ie->context);
 	filter_delete(ie->net);
+	filter_launchcontext_unref(&ie->context);
 	ie->net = NULL;
 	
 	gnome_appbar_set_status(GNOME_APPBAR(ie->appbar), "Done.");
@@ -368,7 +685,8 @@ static void ie_stats_cb(GtkWidget *bla, struct imp_s *ie)
 	filter_param_t *param;
 	GtkWidget *ed;
 	float rms, mrms, percentage;
-	int i, bsize=1;
+	int i;
+	long bsize=1;
 	char buffer[128];
 
 	if(ie->gotfile==0) {
@@ -377,6 +695,8 @@ static void ie_stats_cb(GtkWidget *bla, struct imp_s *ie)
 		gnome_dialog_run_and_close(GNOME_DIALOG(ed));
 		return;
 	}
+	if (ie->gotfile==2)
+		return;
 
 	gtk_widget_set_sensitive(bla, FALSE);
 
@@ -421,18 +741,18 @@ static void ie_stats_cb(GtkWidget *bla, struct imp_s *ie)
 	
 	ie->cancelled=0;
 
-	if ((filter_launch(ie->net, GLAME_BULK_BUFSIZE) == -1) ||
-	    (filter_start(ie->net) == -1))
+	if (!(ie->context = filter_launch(ie->net, GLAME_BULK_BUFSIZE)) ||
+	    (filter_start(ie->context) == -1))
 		goto _ie_stats_cleanup;
 	
 	DPRINTF("Start network for %d samples\n",ie->frames);
 
-	while(!filter_is_ready(ie->net)) {
+	while(!filter_is_ready(ie->context)) {
 		while (gtk_events_pending())
 			gtk_main_iteration();
 		
 		usleep(40000);
-		percentage = (float)filterparam_val_pos(param)/(float)ie->frames;
+		percentage = (float)filterparam_val_long(param)/(float)ie->frames;
 		if(percentage>1.0)
 			percentage = 1.0;
 		gnome_appbar_set_progress(GNOME_APPBAR(ie->appbar),
@@ -440,7 +760,7 @@ static void ie_stats_cb(GtkWidget *bla, struct imp_s *ie)
 	}
 
 	if (ie->cancelled==0) {
-		ie->frames = filterparam_val_pos(param);
+		ie->frames = filterparam_val_long(param);
 		sprintf(buffer, "%d", ie->frames);
 		filterparam_set_property(filterparamdb_get_param(filter_paramdb(ie->readfile), "filename"),
 					 "#framecount", 
@@ -456,7 +776,7 @@ static void ie_stats_cb(GtkWidget *bla, struct imp_s *ie)
 		for(i=0; i<ie->chancnt; i++) {
 			param = filterparamdb_get_param(filter_paramdb(maxrms[i]),
 						"maxrms");
-			rms = filterparam_val_float(param);
+			rms = filterparam_val_double(param);
 			if(rms>mrms)
 				mrms = rms;
 		}
@@ -471,6 +791,7 @@ static void ie_stats_cb(GtkWidget *bla, struct imp_s *ie)
 		gnome_appbar_set_status(GNOME_APPBAR(ie->appbar), "Cancelled.");
 		gnome_appbar_set_progress(GNOME_APPBAR(ie->appbar), 0.0);
 	}
+	filter_launchcontext_unref(&ie->context);
 	
  _ie_stats_cleanup:
 	DPRINTF("Cleanup");
@@ -488,28 +809,68 @@ static void ie_stats_cb(GtkWidget *bla, struct imp_s *ie)
 static void ie_filename_cb(GtkEditable *edit, struct imp_s *ie)
 {
 	filter_param_t *fparam;
+	char *mimetype;
 	int i;
 
-	if (ie->filename) {
+	if (ie->filename)
 		g_free(ie->filename);
-		ie->filename = NULL;
-	}
 	ie->filename = gtk_editable_get_chars(edit, 0, -1);
-	DPRINTF("Got filename %s\n", ie->filename);
-	if (access(ie->filename, R_OK) == -1) {
-		ie->gotfile = 0;
-		ie->filename = NULL;
+	if (access(ie->filename, R_OK) == -1)
+		return;
+
+	/* Basic checks. Mimetype is a "hack", we dont want to introduce
+	 * another build dependency during 1.0.x. */
+	mimetype = "audio/";
+	if (strrchr(ie->filename, '.') != NULL ) {
+		const char *ext = strrchr(ie->filename, '.') + 1;
+		if (strcasecmp(ext, "mp3") == 0)
+			mimetype = "audio/x-mp3";
+		else if (strcasecmp(ext, "ogg") == 0)
+			mimetype = "audio/x-ogg";
+	}
+	gtk_label_set_text(GTK_LABEL(ie->fi_plabel[0]), mimetype);
+	DPRINTF("Got file %s with mimetype %s\n", ie->filename, mimetype);
+
+	/* check if its an mp3 */
+	if (strcmp(mimetype, "audio/x-mp3") == 0) {
+		ie->gotfile = 2;
+		ie->gotstats = 0;
+		for(i=1; i<MAX_PROPS; i++)
+			gtk_label_set_text(GTK_LABEL(ie->fi_plabel[i]), "-");
+		gtk_widget_set_sensitive(ie->checkresample, FALSE);
+		gtk_widget_set_sensitive(ie->rateentry, FALSE);
+		gtk_widget_set_sensitive(ie->getstats, FALSE);
+		return;
+	}
+	/* check if its an ogg */
+	if (strcmp(mimetype, "audio/x-ogg") == 0
+	    || strcmp(mimetype, "application/x-ogg") == 0) {
+		ie->gotfile = 3;
+		ie->gotstats = 0;
+		for(i=1; i<MAX_PROPS; i++)
+			gtk_label_set_text(GTK_LABEL(ie->fi_plabel[i]), "-");
+		gtk_widget_set_sensitive(ie->checkresample, FALSE);
+		gtk_widget_set_sensitive(ie->rateentry, FALSE);
+		gtk_widget_set_sensitive(ie->getstats, FALSE);
+		return;
+	}
+
+	if (strncmp(mimetype, "audio/", 6) != 0) {
+		for(i=1; i<MAX_PROPS; i++)
+			gtk_label_set_text(GTK_LABEL(ie->fi_plabel[i]), "-");
+		gtk_widget_set_sensitive(ie->checkresample, FALSE);
+		gtk_widget_set_sensitive(ie->rateentry, FALSE);
+		gtk_widget_set_sensitive(ie->getstats, FALSE);
 		return;
 	}
 
 	fparam = filterparamdb_get_param(filter_paramdb(ie->readfile), "filename");
-	if (filterparam_set(fparam, &(ie->filename))==-1) {
-		for(i=0; i<MAX_PROPS; i++)
-			gtk_label_set_text(GTK_LABEL(ie->fi_plabel[i]), "nan");
-		return;
-	}
-	
+	filterparam_set(fparam, &(ie->filename));
+
 	ie_update_plabels(ie);
+	gtk_widget_set_sensitive(ie->checkresample, TRUE);
+	gtk_widget_set_sensitive(ie->rateentry, TRUE);
+	gtk_widget_set_sensitive(ie->getstats, TRUE);
 	ie->gotfile = 1;
 	ie->gotstats = 0;
 }
@@ -563,7 +924,7 @@ gpsm_item_t *glame_import_dialog(GtkWindow *parent)
 
 	gtk_container_set_border_width (GTK_CONTAINER (ie->dialog), 1);
 	gtk_window_set_policy (GTK_WINDOW (ie->dialog), FALSE, FALSE, FALSE);
-	gnome_dialog_close_hides(GNOME_DIALOG(ie->dialog), TRUE);
+	gnome_dialog_close_hides(GNOME_DIALOG(ie->dialog), FALSE);
 	gnome_dialog_set_close(GNOME_DIALOG(ie->dialog), FALSE);
 	if (parent)
 		gnome_dialog_set_parent(GNOME_DIALOG(ie->dialog), parent);
@@ -581,6 +942,7 @@ gpsm_item_t *glame_import_dialog(GtkWindow *parent)
 	gtk_box_pack_start (GTK_BOX (vbox2), frame3, TRUE, TRUE, 0);
 	
 	fileentry2 = gnome_file_entry_new ("gpsmop::import::filename", "Import File");
+	gnome_file_entry_set_modal(GNOME_FILE_ENTRY(fileentry2), TRUE);
 	gtk_widget_show (fileentry2);
 	gtk_container_add (GTK_CONTAINER (frame3), fileentry2);
 	
@@ -630,7 +992,7 @@ gpsm_item_t *glame_import_dialog(GtkWindow *parent)
 		gtk_misc_set_alignment (GTK_MISC (ie->fi_plabel[i]), 0, 0.5);
 	}
 		
-	statbutton = gtk_button_new_with_label (_("Get RMS & DC-Offset"));
+	ie->getstats = statbutton = gtk_button_new_with_label (_("Get RMS & DC-Offset"));
 	gtk_signal_connect(GTK_OBJECT(statbutton), "clicked",
 				   (GtkSignalFunc)ie_stats_cb, ie);
 
@@ -746,7 +1108,7 @@ gpsm_item_t *glame_import_dialog(GtkWindow *parent)
 
 
 struct exp_s {
-	GtkWidget *dialog, *otypemenu, *compmenu, *ocompmenu;
+	GtkWidget *dialog, *otypemenu, *ocompmenu;
 	GtkWidget *cancelbutton, *appbar;
 	int typecnt, comptypes;
 	int *indices, *comparray;
@@ -754,16 +1116,21 @@ struct exp_s {
 	GtkWidget *renderbutton[MAX_RLABEL];
 	gpsm_item_t *item;
 	gchar *filename;
-	int filetype, compression;
+	long filetype, compression;
 	int running;
 	filter_t *net;
+	filter_launchcontext_t *context;
 };
 
+static long export_default_filetype = -1; /* auto */
+static long export_default_compression = AF_COMPRESSION_NONE;
+static int export_default_render_option = 0;
+static int export_default_sample_option = 0;
 
 static gint ie_comp_menu_cb(GtkMenu *menu, struct exp_s *exp)
 {
 	exp->compression = glame_menu_get_active_index(menu);
-	DPRINTF("Compression Type %d chosen\n", exp->compression);
+	DPRINTF("Compression Type %li chosen\n", exp->compression);
 
 	return TRUE;
 }
@@ -774,15 +1141,22 @@ static void make_comp_menu(struct exp_s *ie, int ftype)
 	int *sformarray;
 
 	gchar *complabel;
-	GtkWidget *menuitem;
+	GtkWidget *menuitem, *compmenu;
 
-	gtk_widget_destroy(ie->compmenu);
+	compmenu = gtk_option_menu_get_menu(GTK_OPTION_MENU(ie->ocompmenu));
+	if (compmenu)
+		gtk_widget_destroy(compmenu);
 
-	ie->compmenu = gtk_menu_new();
+	compmenu = gtk_menu_new();
 	
 	menuitem=gtk_menu_item_new_with_label("none");
 	gtk_widget_show(menuitem);
-	gtk_menu_append(GTK_MENU(ie->compmenu), menuitem);
+	gtk_menu_append(GTK_MENU(compmenu), menuitem);
+
+	if (ftype == -1) { /* auto filetype */
+		gtk_widget_set_sensitive(ie->ocompmenu, FALSE);
+		goto done;
+	}
 
 	sformtypes = afQueryLong(AF_QUERYTYPE_FILEFMT, AF_QUERY_SAMPLE_FORMATS, AF_QUERY_VALUE_COUNT, ftype, 0);
 	DPRINTF("%d sample formats\n", sformtypes);
@@ -802,16 +1176,17 @@ static void make_comp_menu(struct exp_s *ie, int ftype)
 			}
 			menuitem = gtk_menu_item_new_with_label(complabel);
 			gtk_widget_show(menuitem);
-			gtk_menu_append(GTK_MENU(ie->compmenu), menuitem);
+			gtk_menu_append(GTK_MENU(compmenu), menuitem);
 		}
 		gtk_widget_set_sensitive(ie->ocompmenu, TRUE);
-		gtk_signal_connect(GTK_OBJECT(ie->compmenu),
+		gtk_signal_connect(GTK_OBJECT(compmenu),
 				   "selection_done",
 				   (GtkSignalFunc)ie_comp_menu_cb, ie);
 	} else
 		gtk_widget_set_sensitive(ie->ocompmenu, FALSE);
-	
-	gtk_option_menu_set_menu(GTK_OPTION_MENU (ie->ocompmenu), ie->compmenu);
+
+ done:	
+	gtk_option_menu_set_menu(GTK_OPTION_MENU (ie->ocompmenu), compmenu);
 	gtk_option_menu_set_history (GTK_OPTION_MENU (ie->ocompmenu), 0);
 }
 
@@ -825,9 +1200,9 @@ static gint ie_type_menu_cb(GtkMenu *menu, struct exp_s *ie)
 		make_comp_menu(ie, ie->indices[val-1]);
 		ie->filetype=val;
 	} else {
+		make_comp_menu(ie, -1);
 		ie->filetype=-1;
 		ie->compression = AF_COMPRESSION_NONE;
-		gtk_widget_set_sensitive(ie->ocompmenu, FALSE);
 	}
 
 	return TRUE;
@@ -837,7 +1212,7 @@ static void export_cleanup(struct exp_s *e)
 {
 	//gtk_signal_disconnect_by_data(e->cancelbutton, e);
 	if (e->running==1) {
-		filter_terminate(e->net);
+		filter_terminate(e->context);
 		return;
 	}
 	if (e->dialog)
@@ -887,7 +1262,7 @@ static void export_cb(GtkWidget *bla, struct exp_s *exp)
 	filter_pipe_t *pipe;
 	gpsm_grp_t *grp;
 	gpsm_item_t *it;
-	float pos;
+	double pos;
 	GtkWidget *ed;
 	int sfi, ri;
 	int totalframes;
@@ -915,6 +1290,9 @@ static void export_cb(GtkWidget *bla, struct exp_s *exp)
 	for(ri=0; ri<MAX_RLABEL; ri++)
 		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(exp->renderbutton[ri])))
 			break;
+	/* ignore mono/stereo advice, if 1/2 tracks are going to be saved. */
+	if (ri == gpsm_grp_nritems(grp))
+		ri = 0;
 
 	/* Build basic network. */
 	net = filter_creat(NULL);
@@ -978,19 +1356,19 @@ static void export_cb(GtkWidget *bla, struct exp_s *exp)
 
 	exp->net = net;
 
-	if (filter_launch(net, GLAME_BULK_BUFSIZE) == -1
-	    || filter_start(net) == -1)
+	if (!(exp->context = filter_launch(net, GLAME_BULK_BUFSIZE))
+	    || filter_start(exp->context) == -1)
 		goto fail_cleanup;
 
 	exp->running=1;
 	totalframes = gpsm_item_hsize(grp);
 
-	while(!filter_is_ready(exp->net)) {
+	while(!filter_is_ready(exp->context)) {
 		while (gtk_events_pending())
 			gtk_main_iteration();
 		
 		usleep(40000);
-		percentage = (float)filterparam_val_pos(param)/(float)totalframes;
+		percentage = (float)filterparam_val_long(param)/(float)totalframes;
 		if(percentage>1.0)
 			percentage = 1.0;
 		gnome_appbar_set_progress(GNOME_APPBAR(exp->appbar),
@@ -998,7 +1376,14 @@ static void export_cb(GtkWidget *bla, struct exp_s *exp)
 	}
 	exp->running = 0;
 
+	/* remember new defaults */
+	export_default_filetype = exp->filetype;
+	export_default_compression = exp->compression;
+	export_default_render_option = ri;
+	export_default_sample_option = sfi;
+
 	filter_delete(net);
+	filter_launchcontext_unref(&exp->context);
 	gpsm_item_destroy((gpsm_item_t *)grp);
 	export_cleanup(exp);
 	return;
@@ -1012,7 +1397,7 @@ static void export_cb(GtkWidget *bla, struct exp_s *exp)
 }
 
 
-int glame_export_dialog(gpsm_item_t *item, GtkWindow *parent) 
+GnomeDialog *glame_export_dialog(gpsm_item_t *item, GtkWindow *parent) 
 {	
 	struct exp_s *ie;
 	GtkWidget *dialog, *menu, *mitem, *bigbox, *typecompbox, *valbox;
@@ -1026,13 +1411,13 @@ int glame_export_dialog(gpsm_item_t *item, GtkWindow *parent)
 	ie = (struct exp_s*)calloc(1,sizeof(struct exp_s));
 	ie->item = item;
 	ie->filename = NULL;
-	ie->filetype = -1; /* filetype auto */
-	ie->compression = AF_COMPRESSION_NONE;
+	ie->filetype = export_default_filetype;
+	ie->compression = export_default_compression;
 	ie->running = 0;
 	/* open new dialog window */
 	dialog = ie->dialog = gnome_dialog_new(NULL, NULL);
 	gtk_window_set_policy(GTK_WINDOW(dialog), FALSE, FALSE, FALSE);
-	gnome_dialog_close_hides(GNOME_DIALOG(dialog), TRUE);
+	gnome_dialog_close_hides(GNOME_DIALOG(dialog), FALSE);
 	gnome_dialog_set_close(GNOME_DIALOG(dialog), FALSE);
 	if (parent)
 		gnome_dialog_set_parent(GNOME_DIALOG(dialog), parent);
@@ -1042,7 +1427,6 @@ int glame_export_dialog(gpsm_item_t *item, GtkWindow *parent)
 	gtk_button_box_set_child_size (GTK_BUTTON_BOX (dialog_action_area), 150, 30);
 
 	dialog_vbox2 = GNOME_DIALOG (ie->dialog)->vbox;
-	/*gtk_widget_show (dialog_vbox2);*/
 
 	vbox = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (vbox);
@@ -1057,6 +1441,7 @@ int glame_export_dialog(gpsm_item_t *item, GtkWindow *parent)
 	gtk_box_pack_start (GTK_BOX (bigbox), typecompbox, TRUE, TRUE, 0);
 
 	fentry = gnome_file_entry_new ("gpsmop::export::filename", "Export File");
+	gnome_file_entry_set_modal(GNOME_FILE_ENTRY(fentry), TRUE);
 	gtk_widget_show(fentry);
 	gtk_box_pack_start (GTK_BOX (vbox), fentry, TRUE, TRUE, 0);
 
@@ -1094,6 +1479,8 @@ int glame_export_dialog(gpsm_item_t *item, GtkWindow *parent)
 	for(i=0; i<MAX_RLABEL; i++) {
 		ie->renderbutton[i]=gtk_radio_button_new_with_label(renderbuttons, rlabel[i]);
 		renderbuttons=gtk_radio_button_group(GTK_RADIO_BUTTON(ie->renderbutton[i]));
+		if (i == export_default_render_option)
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ie->renderbutton[i]), TRUE);
 		gtk_widget_show(ie->renderbutton[i]);
 		gtk_box_pack_start(GTK_BOX(frame4box), ie->renderbutton[i], TRUE, TRUE, 0);
 	}
@@ -1109,6 +1496,8 @@ int glame_export_dialog(gpsm_item_t *item, GtkWindow *parent)
 	for(i=0; i<MAX_SFLABEL; i++) {
 		ie->rbutton[i] = gtk_radio_button_new_with_label(rbuttons, sflabel[i]);
 		rbuttons = gtk_radio_button_group(GTK_RADIO_BUTTON(ie->rbutton[i]));
+		if (i == export_default_sample_option)
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ie->rbutton[i]), TRUE);
 		gtk_widget_show(ie->rbutton[i]);
 		gtk_box_pack_start (GTK_BOX (valbox), ie->rbutton[i], TRUE, TRUE, 0);
 	}
@@ -1135,22 +1524,20 @@ int glame_export_dialog(gpsm_item_t *item, GtkWindow *parent)
 		gtk_menu_append (GTK_MENU (menu), mitem);
 	}
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (ie->otypemenu), menu);
-	gtk_option_menu_set_history (GTK_OPTION_MENU (ie->otypemenu), 0);
-	
+	gtk_option_menu_set_history (GTK_OPTION_MENU (ie->otypemenu),
+				     ie->filetype == -1 ? 0 : ie->filetype);
+
+	/* compression combo-box, build by simulating a type choose */
 	ie->ocompmenu = gtk_option_menu_new ();
 	gtk_widget_show(ie->ocompmenu);
 	gtk_container_add(GTK_CONTAINER(frame2), ie->ocompmenu);
+	ie_type_menu_cb(GTK_MENU(gtk_option_menu_get_menu(GTK_OPTION_MENU(ie->otypemenu))), ie);
 
-	ie->compmenu = gtk_menu_new();
-
-	if (ie->typecnt>0) {
-		make_comp_menu(ie, ie->indices[0]);
-	}
-	
 	/* filetype is auto, so compression can't be set */
 	gtk_widget_set_sensitive(ie->ocompmenu, FALSE);
 
 	gnome_dialog_append_button(GNOME_DIALOG (ie->dialog), _("Export"));
+	gnome_dialog_set_default(GNOME_DIALOG(ie->dialog), OK);
 	gnome_dialog_append_button(GNOME_DIALOG (ie->dialog), GNOME_STOCK_BUTTON_CANCEL);
 	gnome_dialog_append_button(GNOME_DIALOG(ie->dialog), GNOME_STOCK_BUTTON_HELP);
 	//ie->cancelbutton = GTK_WIDGET (g_list_last (GNOME_DIALOG (ie->dialog)->buttons)->data);
@@ -1171,9 +1558,7 @@ int glame_export_dialog(gpsm_item_t *item, GtkWindow *parent)
 	gtk_signal_connect(GTK_OBJECT(ie->dialog),
 			   "delete-event",
 			   GTK_SIGNAL_FUNC(exp_windowkilled), ie);
-	gtk_widget_show(dialog);
 
-
-	return 0;
+	return (GnomeDialog *)dialog;
 }
 
