@@ -23,6 +23,7 @@
  * docs/drag_algorithm.txt
  */
 
+#include <string.h>
 #include <gtk/gtk.h>
 #include "util/glame_hruler.h"
 #include "gtkwaveview.h"
@@ -30,6 +31,10 @@
 #include "grange.h"
 #include "util.h"
 
+/* static xpms for rec/mute buttons */
+#include "rec.xpm"
+#include "mute.xpm"
+
 
 /* Stuff for new ruler.
  */
@@ -75,6 +80,8 @@ static const GlameRulerMetric frame_metric = {
 static void gtk_wave_view_class_init    (GtkWaveViewClass *klass);
 static void gtk_wave_view_init          (GtkWaveView      *waveview);
 
+static void gtk_wave_view_draw_marker   (GtkWaveView *waveview);
+
 
 static GtkVBoxClass *parent_class;
 
@@ -227,13 +234,13 @@ calc_ext_pel_win (GtkWaveView *waveview, gint32 win_pel_pos)
   return win_pel_pos + ((gint32)(GTK_ADJUSTMENT (waveview->adjust)->value / waveview->zoom));
 }
 
-
+#if 0 /* unused */
 static gint32
 calc_frame_pos_ext (GtkWaveView *waveview, gint32 ext_pel_pos)
 {
   return (gint32) (ext_pel_pos * waveview->zoom);
 }
-
+#endif
 
 static gint32
 calc_frame_pos_win (GtkWaveView *waveview, gint32 win_pel_pos)
@@ -411,7 +418,7 @@ gtk_wave_view_update_units (GtkWaveView *waveview)
 
   length = (gdouble) gtk_wave_buffer_get_length (waveview->wavebuffer);
   rate = (gdouble) gtk_wave_buffer_get_rate (waveview->wavebuffer);
-  width = GTK_WIDGET (waveview)->allocation.width;
+  width = GTK_WIDGET(waveview->area)->allocation.width;
 
   /* Set scrollbar in pixel units. */
   adj->lower = 0.0;
@@ -436,8 +443,10 @@ gtk_wave_view_update_units (GtkWaveView *waveview)
 
   j = -calc_win_pel_pos (waveview, 0);
   glame_ruler_set_range (GLAME_RULER(waveview->hruler),
-			 calc_frame_pos_ext (waveview, j) / rate,
-			 calc_frame_pos_ext (waveview, j + width) / rate,
+			 j * waveview->zoom / rate,
+			 (j + width) * waveview->zoom / rate,
+			 /* calc_frame_pos_ext (waveview, j) / rate,
+			 calc_frame_pos_ext (waveview, j + width) / rate, */
 			 0.0, 1000.0);
 /* for frames
   glame_ruler_set_range (GLAME_RULER(waveview->hruler),
@@ -446,6 +455,28 @@ gtk_wave_view_update_units (GtkWaveView *waveview)
 			 0.0, 1000.0); */
 }
 
+static void
+gtk_wave_view_update_label(GtkWaveView *waveview)
+{
+	char label[32];
+	int mins, mins2;
+	mins = (int)(waveview->marker/(double)gtk_wave_buffer_get_rate(waveview->wavebuffer)/60.0);
+	snprintf(label, 32, "%i:%.3fs", mins,
+		 waveview->marker/(double)gtk_wave_buffer_get_rate(waveview->wavebuffer) - mins*60);
+	gtk_label_set_text(GTK_LABEL(waveview->marker_label), label);
+	if (waveview->select_left <= waveview->select_right) {
+		mins = (int)(waveview->select_left/(double)gtk_wave_buffer_get_rate(waveview->wavebuffer)/60.0);
+		mins2 = (int)(waveview->select_right/(double)gtk_wave_buffer_get_rate(waveview->wavebuffer)/60.0);
+		snprintf(label, 32, "%i:%.3fs - %i:%.3fs",
+			 mins,
+		 	 waveview->select_left/(double)gtk_wave_buffer_get_rate(waveview->wavebuffer) - mins*60,
+			 mins2,
+		 	 waveview->select_right/(double)gtk_wave_buffer_get_rate(waveview->wavebuffer) - mins2*60);
+	} else
+		strcpy(label, "none");
+	gtk_label_set_text(GTK_LABEL(waveview->selection_label), label);
+}
+
 
 /* The crusty stuff to redraw all the channels of a waveview. */
 static void
@@ -455,6 +486,7 @@ gtk_wave_view_redraw_wave (GtkWaveView *waveview)
   guint32   n_samples, n_channels;
   guint32   i, j, width, start_x, offset;
   gint16    max_val, min_val;
+  gboolean  exact;
 
   if (waveview->wavebuffer == NULL)
     return;
@@ -471,9 +503,12 @@ gtk_wave_view_redraw_wave (GtkWaveView *waveview)
 
   offset = -calc_win_pel_pos (waveview, 0);
   waveview->drawn_offset = offset;
-  width = waveview->expose_width;
+  width = MIN(waveview->area->allocation.width, waveview->expose_width);
   start_x = MAX(0, calc_win_pel_ext(waveview, waveview->expose_x));
 
+  /* Hide marker. */
+  gtk_wave_view_draw_marker (waveview);
+
   /* First, paint all cached x coords. */
   /* Keep a range min_val -> max_val that contains all uncached x coords. */
   for (i = start_x; i < start_x + width; i++)
@@ -499,7 +534,7 @@ gtk_wave_view_redraw_wave (GtkWaveView *waveview)
 
       datatype = gtk_wave_buffer_get_datatype (waveview->wavebuffer);
 
-#define REDRAW_BUFFER 8192
+#define REDRAW_BUFFER 1024
       /* Allocate some temp space. */
       data = g_malloc (g_wavefile_type_width (datatype) * n_channels * REDRAW_BUFFER);
 
@@ -518,7 +553,7 @@ gtk_wave_view_redraw_wave (GtkWaveView *waveview)
 
       /* Perform clipping. */
       if (sample_offset >= n_samples)
-        return;
+        goto out;
       if (last_sample_offset >= n_samples)
         last_sample_offset = n_samples - 1;
 
@@ -529,6 +564,8 @@ gtk_wave_view_redraw_wave (GtkWaveView *waveview)
       accum = 1073741824U; /* 2147483648 / 2, round to nearest unit */
       count = 0;
       delta = (guint32) (2147483648.0 / waveview->zoom);
+      exact = (waveview->zoom < REDRAW_BUFFER);
+
       while (pos <= last_sample_offset)
         {
           size = MIN (REDRAW_BUFFER, last_sample_offset - pos + 1);
@@ -536,41 +573,68 @@ gtk_wave_view_redraw_wave (GtkWaveView *waveview)
 	  /* If by any chance we got destroyed, bail out. */
 	  if (waveview->destroyed) {
 		  DPRINTF("Whoops - coalesced with destroy event :)\n");
-		  return;
+		  goto out;
 	  }
 
-          /* Read data chunk. */
-          gtk_wave_buffer_get_samples (waveview->wavebuffer, pos, size, 0xffffffff, data);
+	  if (exact) {
+             /* Read data chunk. */
+	     gtk_wave_buffer_get_samples (waveview->wavebuffer, pos, size, 0xffffffff, data);
 
-          /* Convert data to s16 if it's not already in that format. */
-          if (datatype != G_WAVEFILE_TYPE_S16)
-            g_wavefile_type_convert (waveview->n_channels, size, G_WAVEFILE_TYPE_S16, data16, datatype, data);
+	     /* Convert data to s16 if it's not already in that format. */
+	     if (datatype != G_WAVEFILE_TYPE_S16)
+		g_wavefile_type_convert (waveview->n_channels, size, G_WAVEFILE_TYPE_S16, data16, datatype, data);
 
-          pos16 = 0;
-          for (i = 0; i < size; i++)
-            {
-              /* For each sample of a track, keep min and max values. */
-              for (j = 0; j < n_channels; j++)
-                {
-                  gint16 val = data16[pos16++];
-
-                  if (val < min[j]) min[j] = val;
-                  if (val > max[j]) max[j] = val;
+	     pos16 = 0;
+	     for (i = 0; i < size; i++) {
+		/* For each sample of a track, keep min and max values. */
+		for (j = 0; j < n_channels; j++) {
+		   gint16 val = data16[pos16++];
+		   if (val < min[j]) min[j] = val;
+		   if (val > max[j]) max[j] = val;
                 }
 
-              /* Do Bresenham's algorithm */
-              accum += delta;
+		/* Do Bresenham's algorithm */
+		accum += delta;
 
-              if (accum >= 2147483648U)
-                {
-                  accum -= 2147483648U;
-
-                  gtk_wave_view_cache_add (waveview, offset + min_val + count, min, max);
-                  gtk_wave_view_cache_paint (waveview, offset, min_val + count);
-
-                  count++;
+		if (accum >= 2147483648U) {
+		   accum -= 2147483648U;
+		   gtk_wave_view_cache_add (waveview, offset + min_val + count, min, max);
+		   gtk_wave_view_cache_paint (waveview, offset, min_val + count);
+		   count++;
                 }
-            }
+	     }
+
+	  } else {
+             /* Do Bresenham's algorithm */
+             accum += delta*size;
+
+             if (accum >= 2147483648U) {
+		accum -= 2147483648U;
+
+		/* Read data chunk. */
+		gtk_wave_buffer_get_samples (waveview->wavebuffer, pos, size, 0xffffffff, data);
+
+		/* Convert data to s16 if it's not already in that format.  */
+		if (datatype != G_WAVEFILE_TYPE_S16)
+		   g_wavefile_type_convert (waveview->n_channels, size, G_WAVEFILE_TYPE_S16, data16, datatype, data);
+
+		pos16 = 0;
+		for (i = 0; i < size; i++) {
+		   /* For each sample of a track, keep min and max values. */
+		   for (j = 0; j < n_channels; j++) {
+		      gint16 val = abs(data16[pos16++]);
+		      if (val > max[j]) { 
+			 max[j] = val;
+			 min[j] = -val;
+		      }
+		   }
+		}
+
+		gtk_wave_view_cache_add (waveview, offset + min_val + count, min, max);
+		gtk_wave_view_cache_paint (waveview, offset, min_val + count);
+		count++;
+	     }
+	  }
 
 	  /* Be nice to the user. */
 	  if (gtk_events_pending()) {
@@ -590,7 +654,7 @@ gtk_wave_view_redraw_wave (GtkWaveView *waveview)
 			  }
 			  if (notdone_x + notdone_width > waveview->expose_x + waveview->expose_width)
 				  waveview->expose_width = notdone_x + notdone_width - waveview->expose_x;
-			  return;
+			  goto out;
 		  }
 	  }
 
@@ -598,11 +662,14 @@ gtk_wave_view_redraw_wave (GtkWaveView *waveview)
           pos += size;
         }
 
+    out:
       if (data16 != (void*) data)
         g_free (data16);
-
       g_free (data);
     }
+
+  /* Show marker. */
+  gtk_wave_view_draw_marker (waveview);
 }
 
 
@@ -703,12 +770,12 @@ on_area_realize (GtkWidget *widget, gpointer userdata)
       gdk_gc_copy (waveview->marker_gc, waveview->area->style->white_gc);
       gdk_gc_set_function (waveview->marker_gc, GDK_XOR);
     }
-  gdk_gc_set_exposures(waveview->area->style->bg_gc [GTK_STATE_NORMAL],TRUE);
+  gtk_wave_view_update_label(waveview);
 }
 
 
 /* Paint selection areas then call a function to paint the wave data. */
-static void
+static gint
 on_area_expose_event (GtkWidget *widget, GdkEventExpose *event, gpointer userdata)
 {
   GtkWaveView *waveview = GTK_WAVE_VIEW (userdata);
@@ -736,7 +803,7 @@ on_area_expose_event (GtkWidget *widget, GdkEventExpose *event, gpointer userdat
   waveview->expose_count++;
 
   if (event->count > 0 || waveview->drawing)
-    return;
+    return TRUE;
 
   waveview->drawing = 1;
  again:
@@ -761,7 +828,7 @@ on_area_expose_event (GtkWidget *widget, GdkEventExpose *event, gpointer userdat
       frame_area.height = widget->allocation.height;
 
       /* hide marker */
-      gtk_wave_view_draw_marker (waveview);
+      //gtk_wave_view_draw_marker (waveview);
 
       /* Set clipping to expose region. */
       gdk_gc_set_clip_rectangle (widget->style->fg_gc [GTK_STATE_NORMAL], &frame_area);
@@ -845,17 +912,19 @@ on_area_expose_event (GtkWidget *widget, GdkEventExpose *event, gpointer userdat
 	      DPRINTF("Doing real, delayed destroy.\n");
 	      waveview->drawing = 0;
 	      gtk_object_destroy(GTK_OBJECT(waveview));
-	      return;
+	      return TRUE;
       }
 
       /* show marker */
-      gtk_wave_view_draw_marker (waveview);
+      //gtk_wave_view_draw_marker (waveview);
 
     }
 
   if (waveview->expose_count != 0)
 	  goto again;
   waveview->drawing = 0;
+
+  return TRUE;
 }
 
 
@@ -992,6 +1061,7 @@ gtk_wave_view_update_selection (GtkWaveView *waveview, gint32 x1, gint32 x2)
       waveview->select_right = x2;
       gtk_wave_view_redraw_sample_area (waveview, t, t2);
       gtk_wave_view_redraw_sample_area (waveview, x1, x2);
+      gtk_wave_view_update_label(waveview);
       return;
     }
 
@@ -1020,6 +1090,7 @@ gtk_wave_view_update_selection (GtkWaveView *waveview, gint32 x1, gint32 x2)
       waveview->select_right = x2;
       gtk_wave_view_redraw_sample_area(waveview, t + 1, x2);
     }
+  gtk_wave_view_update_label(waveview);
 }
 
 
@@ -1137,6 +1208,17 @@ gtk_wave_view_button_press_event (GtkWidget *widget,
           gtk_wave_view_update_selection (waveview, waveview->drag_start_point,
                                           frame);
         }
+      else if (FLAG_IS_SET (event->state, GDK_CONTROL_MASK) &&
+	       waveview->select_right - waveview->select_left > 0)
+        {
+	  if (frame < (waveview->select_left + waveview->select_right)/2)
+	    waveview->drag_start_point = waveview->select_right;
+	  else
+	    waveview->drag_start_point = waveview->select_left;
+          FLAG_SET (waveview->drag_flags, FLAG_DID_MOVE);
+          gtk_wave_view_update_selection (waveview, waveview->drag_start_point,
+                                          frame);
+        }
       else
         waveview->drag_start_point = frame;
     }
@@ -1145,7 +1227,7 @@ gtk_wave_view_button_press_event (GtkWidget *widget,
 }
 
 
-static void
+static gint
 gtk_wave_view_button_release_event (GtkWidget *widget,
                                     GdkEventAny *event,
                                     gpointer userdata)
@@ -1187,17 +1269,17 @@ gtk_wave_view_button_release_event (GtkWidget *widget,
       x = waveview->area->allocation.width /* - 1  no, round up this way */;
 
   frame = calc_frame_pos_win (waveview, x);
-  if (frame < 0)
-	  frame = 0;
   if (waveview->wavebuffer && frame >= gtk_wave_buffer_get_length(waveview->wavebuffer))
       frame = gtk_wave_buffer_get_length(waveview->wavebuffer) - 1; /* correct for round up */
+  if (frame < 0)
+	  frame = 0;
 
   if (FLAGS_IS_CLEAR (waveview->drag_flags))
-    return;
+    return FALSE;
   if (waveview->wavebuffer == NULL)
     {
       FLAGS_CLEAR (waveview->drag_flags);
-      return;
+      return FALSE;
     }
   if (FLAG_IS_SET (waveview->drag_flags, FLAG_DRAGGING_MARKER))
     {
@@ -1218,6 +1300,8 @@ gtk_wave_view_button_release_event (GtkWidget *widget,
     }  
   area_restore_normal_cursor (waveview);
   FLAGS_CLEAR (waveview->drag_flags);
+
+  return FALSE;
 }
 
 
@@ -1262,7 +1346,7 @@ gtk_wave_view_scroll_notify (gpointer widget)
   GtkAdjustment *adjust = GTK_ADJUSTMENT (waveview->adjust);
   gint32 width, pos, frames;
 
-  width = GTK_WIDGET (waveview)->allocation.width;
+  width = GTK_WIDGET (waveview->area)->allocation.width;
   pos = waveview->mouse_x;
 
   if (pos < 0)
@@ -1345,23 +1429,56 @@ gtk_wave_view_motion_notify_event (GtkWidget *widget,
 static void
 gtk_wave_view_init (GtkWaveView *waveview)
 {
+	GtkWidget *hbox, *hbox2, *vbox1, *vbox2, *label;
+
+	hbox = gtk_hbox_new(FALSE, 0);
+	waveview->vbox1 = vbox1 = gtk_vbox_new(FALSE, 0);
+	vbox2 = gtk_vbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(waveview), hbox, TRUE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), vbox1, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), vbox2, TRUE, TRUE, 0);
+
   waveview->hruler = glame_hruler_new();
   glame_ruler_set_metric(GLAME_RULER(waveview->hruler),
 			 &time_metric);
-  gtk_box_pack_start (GTK_BOX (waveview), waveview->hruler, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox2), waveview->hruler, FALSE, FALSE, 0);
   gtk_widget_show (waveview->hruler);
 
   waveview->area = gtk_drawing_area_new ();
   gtk_drawing_area_size (GTK_DRAWING_AREA (waveview->area), 150, 100);
   gtk_widget_show (waveview->area);
-  gtk_box_pack_start (GTK_BOX (waveview), waveview->area, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox2), waveview->area, TRUE, TRUE, 0);
   gtk_widget_show (waveview->area);
 
   waveview->adjust = gtk_adjustment_new (0.0, 0.0, 10.0, 1.0, 1.0, 5.0);
   waveview->hscroll = gtk_hscrollbar_new (GTK_ADJUSTMENT (waveview->adjust));
-  gtk_box_pack_start (GTK_BOX (waveview), waveview->hscroll, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox2), waveview->hscroll, FALSE, FALSE, 0);
   gtk_widget_show (waveview->hscroll);
 
+	hbox = gtk_hbox_new(FALSE, 0);
+	hbox2 = gtk_hbox_new(TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), hbox2, TRUE, TRUE, 0);
+	label = gtk_label_new("Marker position:");
+	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
+	gtk_box_pack_start(GTK_BOX(hbox2), label,
+			   FALSE, FALSE, 0);
+  	waveview->marker_label = gtk_widget_new(gtk_label_get_type(), NULL);
+	gtk_misc_set_alignment(GTK_MISC(waveview->marker_label), 0.0, 0.5);
+	gtk_box_pack_start(GTK_BOX(hbox2), waveview->marker_label,
+			   FALSE, FALSE, 0);
+	hbox2 = gtk_hbox_new(TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), hbox2, TRUE, TRUE, 0);
+	label = gtk_label_new("Current selection:");
+	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
+	gtk_box_pack_start(GTK_BOX(hbox2), label,
+			   FALSE, FALSE, 0);
+  	waveview->selection_label = gtk_widget_new(gtk_label_get_type(), NULL);
+	gtk_misc_set_alignment(GTK_MISC(waveview->selection_label), 0.0, 0.5);
+	gtk_box_pack_start(GTK_BOX(hbox2), waveview->selection_label,
+			   FALSE, FALSE, 0);
+	gtk_widget_show_all(hbox);
+	gtk_box_pack_start(GTK_BOX(waveview), hbox, FALSE, FALSE, 0);
+
   gtk_signal_connect (GTK_OBJECT (waveview->adjust),
                       "value_changed", gtk_wave_view_scroll,
                       GTK_OBJECT (waveview));
@@ -1533,6 +1650,7 @@ on_wave_buffer_insert_data (GtkWidget *widget, GRange *range)
     waveview->select_left += width;
 
   gtk_wave_view_update_units (waveview);
+  gtk_wave_view_update_label (waveview);
 }
 
 
@@ -1560,6 +1678,7 @@ on_wave_buffer_delete_data (GtkWidget *widget, GRange *range)
     waveview->select_right = range->left - 1;
 
   gtk_wave_view_update_units (waveview);
+  gtk_wave_view_update_label(waveview);
 }
 
 
@@ -1570,6 +1689,18 @@ gtk_wave_view_get_buffer (GtkWaveView *waveview)
 }
 
 
+/* Create/add record/mute buttons after realization of togglebuttons. */
+void on_tb_realize_cb(GtkWidget *widget, gpointer data)
+{
+	GdkPixmap *pixmap;
+	GdkBitmap *bitmap;
+	pixmap = gdk_pixmap_create_from_xpm_d(
+		widget->window, &bitmap, NULL, data);
+	gtk_container_add(GTK_CONTAINER(widget),
+			  gtk_pixmap_new(pixmap, bitmap));
+	gtk_widget_show_all(widget);
+}
+
 /* Select a new data stream, ref() it, invalidate cache, and update screen. */
 void
 gtk_wave_view_set_buffer (GtkWaveView *waveview, GtkWaveBuffer *wavebuffer)
@@ -1627,7 +1758,8 @@ gtk_wave_view_set_buffer (GtkWaveView *waveview, GtkWaveBuffer *wavebuffer)
       waveview->channels = g_new (GtkWaveViewChannelInfo, waveview->n_channels);
 
       waveview->select_left = 0;
-      waveview->select_right = gtk_wave_buffer_get_length (waveview->wavebuffer) - 1;
+      waveview->select_right = -1;
+      /* waveview->select_right = gtk_wave_buffer_get_length (waveview->wavebuffer) - 1; */
       waveview->marker = 0;
 
       gtk_wave_view_cache_create (waveview);
@@ -1643,8 +1775,103 @@ gtk_wave_view_set_buffer (GtkWaveView *waveview, GtkWaveBuffer *wavebuffer)
   gtk_widget_queue_draw (GTK_WAVE_VIEW (waveview)->area);
   gtk_wave_view_update_units (waveview);
   gtk_wave_view_calc_channel_locs (waveview);
+
+  if (wavebuffer) {
+	  GtkWidget *tb, *vbox;
+	  int cnt;
+#if 0
+	  GList *list;
+	  list = gtk_container_children(waveview->vbox1);
+	  while (list) {
+		  gtk_widget_destroy(list->data);
+		  list = g_list_next(list);
+	  }
+#endif
+	  cnt = gtk_wave_buffer_get_num_channels(wavebuffer);
+	  while (cnt--) {
+#if 0
+		  GdkPixmap *rec_pixmap, *mute_pixmap;
+		  GdkBitmap *rec_bitmap, *mute_bitmap;
+#endif
+		  vbox = gtk_vbox_new(FALSE, 0);
+		  tb = gtk_toggle_button_new();
+#if 0
+		  rec_pixmap = gdk_pixmap_create_from_xpm_d(
+			  GTK_WIDGET(tb)->window, &rec_bitmap, NULL, rec_xpm);
+#endif
+		  gtk_container_set_border_width(GTK_CONTAINER(tb), 0);
+#if 0
+		  gtk_container_add(GTK_CONTAINER(tb),
+				    gtk_pixmap_new(rec_pixmap, rec_bitmap));
+#endif
+		  gtk_object_set(GTK_OBJECT(tb), "can_focus", FALSE, NULL);
+		  gtk_box_pack_start(GTK_BOX(vbox), tb, FALSE, FALSE, 0);
+		  gtk_signal_connect(GTK_OBJECT(tb), "realize",
+				     on_tb_realize_cb, rec_xpm);
+		  tb = gtk_toggle_button_new();
+#if 0
+		  mute_pixmap = gdk_pixmap_create_from_xpm_d(
+			  GTK_WIDGET(tb)->window, &mute_bitmap, NULL, mute_xpm);
+#endif
+		  gtk_container_set_border_width(GTK_CONTAINER(tb), 0);
+#if 0
+		  gtk_container_add(GTK_CONTAINER(tb),
+				    gtk_pixmap_new(mute_pixmap, mute_bitmap));
+#endif
+		  gtk_object_set(GTK_OBJECT(tb), "can_focus", FALSE, NULL);
+		  gtk_box_pack_start(GTK_BOX(vbox), tb, FALSE, FALSE, 0);
+		  gtk_signal_connect(GTK_OBJECT(tb), "realize",
+				     on_tb_realize_cb, mute_xpm);
+		  gtk_box_pack_start(GTK_BOX(waveview->vbox1), vbox, TRUE, FALSE, 0);
+		  gtk_widget_show_all(vbox);
+	  }
+  }
 }
 
+gboolean gtk_wave_view_get_flag(GtkWaveView *waveview, int track, int flag)
+{
+	GList *list;
+
+	/* track */
+	list = gtk_container_children(GTK_CONTAINER(waveview->vbox1));
+	while (list && track--)
+		list = g_list_next(list);
+	if (!list)
+		return FALSE;
+
+	/* flag */
+	list = gtk_container_children(GTK_CONTAINER(list->data));
+	while (list && flag--)
+		list = g_list_next(list);
+	if (!list)
+		return FALSE;
+
+	return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(list->data));
+}
+
+void gtk_wave_view_set_flag(GtkWaveView *waveview, int track, int flag,
+			    gboolean value)
+{
+	GList *list;
+
+	/* track */
+	list = gtk_container_children(GTK_CONTAINER(waveview->vbox1));
+	while (list && track--)
+		list = g_list_next(list);
+	if (!list)
+		return;
+
+	/* flag */
+	list = gtk_container_children(GTK_CONTAINER(list->data));
+	while (list && flag--)
+		list = g_list_next(list);
+	if (!list)
+		return;
+
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(list->data), value);
+}
+
+
 
 /* return < 0 means no further action necessary
  * return > 0 means further action necessary
@@ -1706,6 +1933,7 @@ gtk_wave_view_set_marker (GtkWaveView *waveview, gint32 frame)
       if (waveview->marker >= 0)
         draw_marker_from_frame_pos (waveview, waveview->marker);
       waveview->marker = frame;
+      gtk_wave_view_update_label(waveview);
     }
 }
 
@@ -1807,6 +2035,8 @@ gtk_wave_view_set_marker_and_scroll (GtkWaveView *waveview,
 
   /* set the new marker */
   waveview->marker = frame;
+ 
+  gtk_wave_view_update_label(waveview);
 }
   
 
@@ -1923,8 +2153,15 @@ gtk_wave_view_set_selection (GtkWaveView *waveview,
   waveview->select_right = start + length - 1;
 
   gtk_widget_queue_draw (waveview->area);
+  gtk_wave_view_update_label(waveview);
 }
 
+gboolean
+gtk_wave_view_range_visible (GtkWaveView *waveview, gint32 start, gint32 length)
+{
+  return calc_win_pel_pos(waveview, start) > 0
+	 && calc_win_pel_pos(waveview, start+length) < GTK_WIDGET(waveview->area)->allocation.width;
+}
 
 guint32
 gtk_wave_view_get_select_channels (GtkWaveView *waveview)
