@@ -1,7 +1,7 @@
 /*
  * main.c
  *
- * $Id: main.c,v 1.97.2.8 2002/03/11 13:02:18 richi Exp $
+ * $Id: main.c,v 1.114.2.3 2003/12/31 22:13:30 richi Exp $
  *
  * Copyright (C) 2001 Johannes Hirche, Richard Guenther
  *
@@ -35,9 +35,12 @@
 #include "glmid.h"
 #include "swapfilegui.h"
 #include "waveeditgui.h"
+#ifdef ENABLE_EXPERIMENTAL
 #include "timeline/timeline.h"
+#endif
 #include "edit_filter/filtereditgui.h"
 #include "util/glame_gui_utils.h"
+#include "util/gtknob.h"
 #include "gltreeitem.h"
 #include "gpsm.h"
 #include "clipboard.h"
@@ -174,7 +177,9 @@ static void create_new_project_cb(GtkWidget *menu, void * blah)
 	/* Create new gpsm group. */
 	grp = gpsm_newgrp(_("Unnamed"));
 	if (gpsm_item_place(gpsm_root(), (gpsm_item_t *)grp,
-			    0, gpsm_item_vsize(gpsm_root())) == -1)
+			    0, gpsm_item_vsize(gpsm_root())) == -1
+	    && gpsm_item_place(gpsm_root(), (gpsm_item_t *)grp,
+			       0, gpsm_item_vsize(gpsm_root())+1) == -1)
 		DPRINTF("Cannot insert new group!?\n");
 	if (deleted)
 		gpsm_item_place(gpsm_root(), (gpsm_item_t *)deleted,
@@ -205,7 +210,7 @@ static void edit_file_cb(GtkWidget *menu, void *data)
 	WaveeditGui *we;
 
 	/* Run the import dialog. */
-	file = glame_import_dialog(NULL);
+	file = glame_import_dialog(GTK_WINDOW(app));
 	if (!file)
 		return;
 
@@ -235,7 +240,7 @@ static void import_cb(GtkWidget *menu, void *data)
 	int res;
 
 	/* Run the import dialog. */
-	file = glame_import_dialog(NULL);
+	file = glame_import_dialog(GTK_WINDOW(app));
 	if (!file)
 		return;
 
@@ -362,13 +367,15 @@ static int update_preferences()
 	DPRINTF(
 "Preferences:\n"
 "\tSwapfile directory %s\n"
+"\tVM parameters are: lru %li, fds %li, maps %li, vm %li\n"
 "\tUndo stack depth is %li\n"
 "\tAudio input plugin %s, device \"%s\", rate %li\n"
 "\tAudio output plugin %s, device \"%s\"\n"
 "\tGLAME_WBUFSIZE %i\n"
 "\tPopup timeout is %lims\n"
 "\tMac mode is %s\n",
-                swappath, maxundo, ainplugin, aindev, rate, aoutplugin, aoutdev, _GLAME_WBUFSIZE,
+                swappath, maxlru, maxfds, maxmaps, maxvm,
+		maxundo, ainplugin, aindev, rate, aoutplugin, aoutdev, _GLAME_WBUFSIZE,
                 nPopupTimeout, bMac ? "on" : "off");
 
 	/* Free temp. storage. */
@@ -395,6 +402,82 @@ toggle_cb(GtkWidget*foo, gboolean *bar)
 	*bar = (*bar)?FALSE:TRUE;
 }
 
+#if 0
+/* This widget should build a widget with a list of the current
+ * (configured) audio I/O plugins and allow to add custom ones
+ * and configure existing ones. */
+static GtkWidget *audio_io_preferences_widget_new()
+{
+	GtkWidget *vbox;
+	GtkWidget *hbox1;
+	GtkWidget *vbox2;
+	GtkWidget *label1;
+	GtkWidget *list1;
+	GtkWidget *vbox1;
+	GtkWidget *hbox2;
+	GtkWidget *entry1;
+	GtkWidget *combo_entry2;
+	GtkWidget *combo1;
+	GtkWidget *combo_entry1;
+	GtkWidget *hbox3;
+	GtkWidget *button1;
+	GtkWidget *button2;
+
+	vbox = gtk_vbox_new(FALSE, 0);
+
+	hbox1 = gtk_hbox_new (FALSE, 0);
+	gtk_widget_show (hbox1);
+	gtk_container_add (GTK_CONTAINER (vbox), hbox1);
+
+	vbox2 = gtk_vbox_new (FALSE, 0);
+	gtk_widget_show (vbox2);
+	gtk_box_pack_start (GTK_BOX (hbox1), vbox2, TRUE, TRUE, 0);
+
+	label1 = gtk_label_new (_("Audio plugins"));
+	gtk_widget_show (label1);
+	gtk_box_pack_start (GTK_BOX (vbox2), label1, FALSE, FALSE, 0);
+
+	list1 = gtk_list_new ();
+	gtk_widget_show (list1);
+	gtk_box_pack_start (GTK_BOX (vbox2), list1, TRUE, TRUE, 0);
+
+	vbox1 = gtk_vbox_new (FALSE, 0);
+	gtk_widget_show (vbox1);
+	gtk_box_pack_start (GTK_BOX (hbox1), vbox1, TRUE, TRUE, 0);
+
+	hbox2 = gtk_hbox_new (FALSE, 0);
+	gtk_widget_show (hbox2);
+	gtk_box_pack_start (GTK_BOX (vbox1), hbox2, TRUE, TRUE, 0);
+
+	entry1 = gnome_entry_new (NULL);
+	gtk_widget_show (entry1);
+	gtk_box_pack_start (GTK_BOX (hbox2), entry1, TRUE, TRUE, 0);
+
+	combo_entry2 = gnome_entry_gtk_entry (GNOME_ENTRY (entry1));
+	gtk_widget_show (combo_entry2);
+
+	combo1 = gtk_combo_new ();
+	gtk_widget_show (combo1);
+	gtk_box_pack_start (GTK_BOX (hbox2), combo1, TRUE, TRUE, 0);
+
+	combo_entry1 = GTK_COMBO (combo1)->entry;
+	gtk_widget_show (combo_entry1);
+
+	hbox3 = gtk_hbox_new (FALSE, 0);
+	gtk_widget_show (hbox3);
+	gtk_box_pack_start (GTK_BOX (vbox1), hbox3, TRUE, TRUE, 0);
+
+	button1 = gtk_button_new_with_label (_("Add"));
+	gtk_widget_show (button1);
+	gtk_box_pack_start (GTK_BOX (hbox3), button1, TRUE, TRUE, 0);
+
+	button2 = gtk_button_new_with_label (_("Apply"));
+	gtk_widget_show (button2);
+	gtk_box_pack_start (GTK_BOX (hbox3), button2, TRUE, TRUE, 0);
+
+	return vbox;
+}
+#endif
 
 static GtkWidget *preferences_tab_filteredit(long *fe_popup_timeout, long *fe_mac)
 {
@@ -445,6 +528,7 @@ static GtkWidget *preferences_tab_swapfile(char *sw_path,
             _("GLAME doesnt handle running out of disk space very well.")));
 
         entry = gnome_file_entry_new("swapfilepath", _("Swapfile Path"));
+	gnome_file_entry_set_modal(GNOME_FILE_ENTRY(entry), TRUE);
         create_label_widget_pair(vbox, _("Swapfile Path"), entry);
 	glame_config_get_string("swapfile/defaultpath", &cfg);
 	strncpy(sw_path, cfg, 255);
@@ -559,6 +643,11 @@ static GtkWidget *preferences_tab_audioio(char *ainplugin, char *aindev,
 	return vbox;
 }
 
+static void pref_help_cb(GnomePropertyBox *box, gint page)
+{
+	gnome_help_goto(NULL, "info:glame#The_Preferences_Dialog");
+}
+
 static int
 preferences_cb(GtkWidget * wid, void * bla)
 {
@@ -610,12 +699,12 @@ preferences_cb(GtkWidget * wid, void * bla)
 	/* Finish. */
         gtk_signal_connect(GTK_OBJECT(GNOME_PROPERTY_BOX(prop_box)->ok_button),"clicked",(GtkSignalFunc)setBoolean,&ok);
         gtk_object_destroy(GTK_OBJECT(GNOME_PROPERTY_BOX(prop_box)->apply_button));
-        gtk_object_destroy(GTK_OBJECT(GNOME_PROPERTY_BOX(prop_box)->help_button));
+	gtk_signal_connect(GTK_OBJECT(prop_box), "help", GTK_SIGNAL_FUNC(pref_help_cb), NULL);
         gtk_widget_show(prop_box);
 
 
 	/* Run the dialog. */
-        gnome_dialog_run_and_close(GNOME_DIALOG(prop_box));
+        gnome_dialog_run(GNOME_DIALOG(prop_box));
         if(!ok)
 		return 0;
 
@@ -623,9 +712,12 @@ preferences_cb(GtkWidget * wid, void * bla)
 	 */
 	glame_config_set_string("swapfile/defaultpath", sw_path);
 	glame_config_set_long("swapfile/maxundo", sw_maxundo);
-	/* Do some magic with sw_maxvm number to derive the rest. */
+	/* Do some magic with sw_maxvm number to derive the rest.
+	 * This is now more safe, but also too low for 64bit large
+	 * memory machines. */
+	sw_maxvm = MIN(3072, sw_maxvm);
 	sw_maxlru = 2048 * sw_maxvm / 256;
-	sw_maxfds = 128 * sw_maxvm / 256;
+	sw_maxfds = MIN(256, 128 * sw_maxvm / 256);
 	sw_maxmaps = 256 * sw_maxvm / 256;
 	sw_maxvm *= 1024*1024;
 	glame_config_set_long("swapfile/maxlru", sw_maxlru);
@@ -700,7 +792,7 @@ static GtkWidget* glame_about(void)
 "Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA"),
 				 logo);
 	gtk_object_set_data (GTK_OBJECT (about), "about", about);
-	gtk_window_set_modal (GTK_WINDOW (about), TRUE);
+	gtk_window_set_modal (GTK_WINDOW (about), FALSE);
 	gtk_window_set_wmclass (GTK_WINDOW (about), "Glameabout", "Glame");
 	gtk_window_set_position (GTK_WINDOW (about), GTK_WIN_POS_CENTER_ALWAYS);
 	gtk_widget_show(about);
@@ -788,7 +880,9 @@ static void gui_main()
 	glame_swapfilegui_init();
 	glame_waveeditgui_init();
 	glame_filtereditgui_init();
+#ifdef ENABLE_EXPERIMENTAL
 	glame_timeline_init();
+#endif
 
 	/* Update preferences. */
 	if (update_preferences() == -1) {
@@ -846,7 +940,7 @@ _("Welcome first-time user of GLAME.\n"
 	gtk_widget_show(scrollview);
 
 	/* Create main window. */
-	app = gnome_app_new ("glame0.5", NULL);
+	app = gnome_app_new ("glame0.7", NULL);
 	gtk_object_set_data (GTK_OBJECT (app), "app", app);
 	dock = GNOME_APP (app)->dock;
 	gtk_widget_ref (dock);
@@ -910,10 +1004,11 @@ int main(int argc, char **argv)
 	textdomain("glame");
 
 	/* setup gnome/gtk  */
-	gnome_init("glame", VERSION, argc, argv);
+	gnome_init("glame0.7", VERSION, argc, argv);
 
 #ifdef HAVE_LIBGLADE
 	glade_init();
+	gtk_knob_glade_register();
 #endif
 
 	/* init glame */
