@@ -1,7 +1,7 @@
 /*
  * swapfilegui.c
  *
- * $Id: swapfilegui.c,v 1.70.2.5 2002/01/16 21:08:02 richi Exp $
+ * $Id: swapfilegui.c,v 1.88.2.2 2003/04/12 12:08:38 richi Exp $
  * 
  * Copyright (C) 2001 Richard Guenther, Johannes Hirche, Alexander Ehlert
  *
@@ -25,12 +25,12 @@
 #include <config.h>
 #endif
 
+#include <libintl.h>
 #include <stdio.h>
 #include <errno.h>
 #include <gnome.h>
 #include <xmlmemory.h>
 #include <parser.h>
-#include <libintl.h>
 #include "glame_types.h"
 #include "swapfile.h"
 #include "gltreeitem.h"
@@ -38,7 +38,9 @@
 #include "filter.h"
 #include "util/glame_gui_utils.h"
 #include "clipboard.h"
+#ifdef ENABLE_EXPERIMENTAL
 #include "timeline/timeline.h"
+#endif
 #include "glscript.h"
 #include "network_utils.h"
 #include "importexport.h"
@@ -52,8 +54,8 @@ static SwapfileGui *active_swapfilegui = NULL;
 
 
 /* Forward declarations. */
-static int click_cb(GtkWidget *item, GdkEventButton *event,
-		    gpointer data);
+static gint click_cb(GtkWidget *item, GdkEventButton *event,
+		     gpointer data);
 static void copyselected_cb(GtkWidget *menu, GlameTreeItem *item);
 static void linkselected_cb(GtkWidget *menu, GlameTreeItem *item);
 static void mergeparent_cb(GtkWidget *menu, GlameTreeItem *item);
@@ -63,12 +65,16 @@ static void addclipboard_cb(GtkWidget *menu, GlameTreeItem *item);
 static void addfile_cb(GtkWidget *menu, GlameTreeItem *item);
 static void addstereo_cb(GtkWidget *menu, GlameTreeItem *item);
 static void edit_cb(GtkWidget *menu, GlameTreeItem *item);
+#ifdef ENABLE_EXPERIMENTAL
 static void timeline_cb(GtkWidget *menu, GlameTreeItem *item);
+#endif
 static void import_cb(GtkWidget *menu, GlameTreeItem *item);
 static void export_cb(GtkWidget *menu, GlameTreeItem *item);
 static void delete_cb(GtkWidget *menu, GlameTreeItem *item);
 static void handle_grp(glsig_handler_t *handler, long sig, va_list va);
 static void group_cb(GtkWidget *menu, GlameTreeItem *item);
+static void file_property_cb(GtkMenu *menu, GlameTreeItem *item);
+static void group_property_cb(GtkMenu *menu, GlameTreeItem *item);
 
 static GnomeUIInfo dummy1_menu[] = {
 	GNOMEUIINFO_END
@@ -80,7 +86,10 @@ static GnomeUIInfo dummy2_menu[] = {
 static GnomeUIInfo group_menu_data[] = {
         GNOMEUIINFO_SEPARATOR,
         GNOMEUIINFO_ITEM(N_("Edit"), NULL, edit_cb, NULL),
+	GNOMEUIINFO_ITEM(N_("Properties..."), NULL, group_property_cb, NULL),
+#ifdef ENABLE_EXPERIMENTAL
         GNOMEUIINFO_ITEM(N_("Timeline"), NULL, timeline_cb, NULL),
+#endif
         GNOMEUIINFO_SEPARATOR,
         GNOMEUIINFO_ITEM(N_("Delete"), NULL, delete_cb, NULL),
         GNOMEUIINFO_SEPARATOR,
@@ -101,15 +110,26 @@ static GnomeUIInfo group_menu_data[] = {
         GNOMEUIINFO_SEPARATOR,
         GNOMEUIINFO_END
 };
+#ifdef ENABLE_EXPERIMENTAL
+#define GROUP_MENU_ADDGROUP_INDEX 7
+#define GROUP_MENU_ADDCLIPBOARD_INDEX 8
+#define GROUP_MENU_ADDMONO_INDEX 9
+#define GROUP_MENU_ADDSTEREO_INDEX 10
+#define GROUP_MENU_APPLYOP_INDEX 17
+#define GROUP_MENU_IMPORT_INDEX 19
+#else
 #define GROUP_MENU_ADDGROUP_INDEX 6
 #define GROUP_MENU_ADDCLIPBOARD_INDEX 7
 #define GROUP_MENU_ADDMONO_INDEX 8
 #define GROUP_MENU_ADDSTEREO_INDEX 9
 #define GROUP_MENU_APPLYOP_INDEX 16
 #define GROUP_MENU_IMPORT_INDEX 18
+#endif
+
 static GnomeUIInfo file_menu_data[] = {
         GNOMEUIINFO_SEPARATOR,
         GNOMEUIINFO_ITEM(N_("Edit"), NULL, edit_cb, NULL),
+	GNOMEUIINFO_ITEM(N_("Properties..."), NULL, file_property_cb, NULL),
 	GNOMEUIINFO_SEPARATOR,
         GNOMEUIINFO_ITEM(N_("Delete"), NULL, delete_cb, NULL),
 	GNOMEUIINFO_SEPARATOR,
@@ -121,7 +141,7 @@ static GnomeUIInfo file_menu_data[] = {
 	GNOMEUIINFO_SEPARATOR,
         GNOMEUIINFO_END
 };
-#define FILE_MENU_APPLYOP_INDEX 7
+#define FILE_MENU_APPLYOP_INDEX 8
 
 
 static void edit_tree_label_cb(GtkEntry* entry, GlameTreeItem* item)
@@ -130,8 +150,8 @@ static void edit_tree_label_cb(GtkEntry* entry, GlameTreeItem* item)
 	/* Unlock accelerators. HACK(?) */
 	gtk_signal_handler_unblock(GTK_OBJECT(active_swapfilegui->app),
 				   active_swapfilegui->accel_handler);
-	gtk_container_remove(GTK_CONTAINER(item), GTK_WIDGET(entry));
 	gtk_widget_destroy(GTK_WIDGET(entry));
+	gtk_widget_show_all(item->hbox);
 	if (text) {
 		gpsm_item_set_label(item->item, text);
 		g_free(text);
@@ -139,19 +159,18 @@ static void edit_tree_label_cb(GtkEntry* entry, GlameTreeItem* item)
 }
 void edit_tree_label(GlameTreeItem * item)
 {
-	GtkWidget *label;
 	GtkWidget *entry;
 
 	/* Deselect item and replace GtkLabel with GtkEntry. */
 	gtk_tree_item_deselect(GTK_TREE_ITEM(item));
-	label = GTK_WIDGET((g_list_first(gtk_container_children(GTK_CONTAINER(item))))->data);
-	gtk_container_remove(GTK_CONTAINER(item), label);
 	entry = gtk_entry_new();
 	gtk_entry_set_text(GTK_ENTRY(entry), gpsm_item_label(item->item));
 	gtk_signal_connect(GTK_OBJECT(entry), "activate",
 			   (GtkSignalFunc)edit_tree_label_cb, item);
+	gtk_container_add(GTK_CONTAINER(item->hbox), entry);
+	gtk_widget_hide_all(item->hbox);
 	gtk_widget_show(entry);
-	gtk_container_add(GTK_CONTAINER(item), entry);
+	gtk_widget_show(item->hbox);
 	gtk_container_check_resize(GTK_CONTAINER(item));
 	gtk_widget_grab_focus(GTK_WIDGET(entry));
 	/* Block accelerators. HACK(?) */
@@ -205,15 +224,15 @@ static int choose_ops(plugin_t *plugin)
 	return 1;
 }
 
-static int click_cb(GtkWidget *item, GdkEventButton *event,
-		    gpointer data)
+static gint click_cb(GtkWidget *item, GdkEventButton *event,
+		     gpointer data)
 {
 	GlameTreeItem *i = GLAME_TREE_ITEM(item);
 	GtkWidget *menu, *op_menu;
 
 	if (event->button == 1
 	    && event->type == GDK_2BUTTON_PRESS) {
-		edit_tree_label(i);
+		edit_cb(NULL, i);
 		return TRUE;
 	}
 
@@ -243,7 +262,7 @@ static int click_cb(GtkWidget *item, GdkEventButton *event,
 		gtk_widget_set_sensitive(group_menu_data[GROUP_MENU_IMPORT_INDEX].widget,
 					 gpsm_grp_is_vbox((gpsm_grp_t *)i->item) || gpsm_grp_is_hbox((gpsm_grp_t *)i->item) ? TRUE : FALSE);
         } else
-		return TRUE;
+		return FALSE;
 
 	gnome_popup_menu_do_popup(menu, NULL, NULL, event, i);
 	return TRUE;
@@ -547,7 +566,7 @@ static void delete_cb(GtkWidget *menu, GlameTreeItem *item)
 		return;
 	}
 	gpsm_item_place(deleted, item->item,
-			0, gpsm_item_vsize(deleted));
+			0, gpsm_item_vsize(deleted)+1);
 }
 
 static void edit_cb(GtkWidget *menu, GlameTreeItem *item)
@@ -565,6 +584,7 @@ static void edit_cb(GtkWidget *menu, GlameTreeItem *item)
 	deselect_all(active_swapfilegui);
 }
 
+#ifdef ENABLE_EXPERIMENTAL
 static void timeline_cb(GtkWidget *menu, GlameTreeItem *item)
 {
 	static int warning_shown = 0;
@@ -586,14 +606,11 @@ static void timeline_cb(GtkWidget *menu, GlameTreeItem *item)
 		warning_shown = 1;
 	}
 }
+#endif
 
-void changeString_cb(GtkEditable *wid, char *returnbuffer)
-{
-	strncpy(returnbuffer, gtk_editable_get_chars(wid, 0, -1), 100);
-}
 static void export_cb(GtkWidget *menu, GlameTreeItem *item)
 {
-	glame_export_dialog(item->item, NULL);
+	gnome_dialog_run_and_close(glame_export_dialog(item->item, GTK_WINDOW(active_swapfilegui->app)));
 	deselect_all(active_swapfilegui);
 }
 
@@ -601,7 +618,7 @@ static void import_cb(GtkWidget *menu, GlameTreeItem *item)
 {
 	gpsm_item_t *imported;
 
-	imported = glame_import_dialog(NULL);
+	imported = glame_import_dialog(GTK_WINDOW(active_swapfilegui->app));
 	if (!imported)
 		return;
 
@@ -619,17 +636,89 @@ static void import_cb(GtkWidget *menu, GlameTreeItem *item)
 	gpsm_item_destroy(imported);
 }
 
+static void file_property_cb(GtkMenu *menu, GlameTreeItem *item)
+{
+	GtkWidget *dialog, *vbox;
+	char f_name[1024];
+	double f_pos;
+	long f_rate;
+
+	strncpy(f_name, gpsm_item_label(item->item), 1024);
+	f_pos = gpsm_swfile_position(item->item);
+	f_rate = gpsm_swfile_samplerate(item->item);
+
+	dialog = gtk_type_new(gnome_dialog_get_type());
+	gnome_dialog_set_parent(GNOME_DIALOG(dialog),
+				GTK_WINDOW(active_swapfilegui->app));
+	gnome_dialog_set_close(GNOME_DIALOG(dialog), FALSE);
+	gnome_dialog_append_button(
+		GNOME_DIALOG(dialog), GNOME_STOCK_BUTTON_OK);
+	gnome_dialog_append_button(
+		GNOME_DIALOG(dialog), GNOME_STOCK_BUTTON_CANCEL);
+	gnome_dialog_set_default(GNOME_DIALOG(dialog), 0);
+
+	vbox = GNOME_DIALOG(dialog)->vbox;
+
+	/* file name */
+	create_label_edit_pair(vbox, "Track name:",
+			       "swapfile::fileprop::name", f_name);
+
+	/* file samplerate */
+	create_label_long_pair(vbox, "Samplerate:", &f_rate, 1, 96000);
+
+	/* file position */
+	create_label_double_pair(vbox, "Position:", &f_pos, -M_PI, M_PI);
+
+	if (gnome_dialog_run_and_close(GNOME_DIALOG(dialog)) == 1)
+		return;
+
+	/* update file */
+	gpsm_item_set_label(item->item, f_name);
+	gpsm_swfile_set((gpsm_swfile_t *)item->item,
+			f_rate, f_pos);
+}
+
+static void group_property_cb(GtkMenu *menu, GlameTreeItem *item)
+{
+	GtkWidget *dialog, *vbox;
+	char g_name[1024];
+
+	strncpy(g_name, gpsm_item_label(item->item), 1024);
+
+	dialog = gtk_type_new(gnome_dialog_get_type());
+	gnome_dialog_set_parent(GNOME_DIALOG(dialog),
+				GTK_WINDOW(active_swapfilegui->app));
+	gnome_dialog_set_close(GNOME_DIALOG(dialog), FALSE);
+	gnome_dialog_append_button(
+		GNOME_DIALOG(dialog), GNOME_STOCK_BUTTON_OK);
+	gnome_dialog_append_button(
+		GNOME_DIALOG(dialog), GNOME_STOCK_BUTTON_CANCEL);
+	gnome_dialog_set_default(GNOME_DIALOG(dialog), 0);
+
+	vbox = GNOME_DIALOG(dialog)->vbox;
+
+	/* group name */
+	create_label_edit_pair(vbox, "Group name:",
+			       "swapfile::groupprop::name", g_name);
+
+	if (gnome_dialog_run_and_close(GNOME_DIALOG(dialog)) == 1)
+		return;
+
+	/* update group */
+	gpsm_item_set_label(item->item, g_name);
+}
+
 
 /*
  * Drag & Drop state-machine.
  * FIXME: better grab/release the pointer
  */
 extern GtkWidget *glame_appbar;
-static void drag_start_stop_cb(GtkWidget *widget, GdkEventButton *event,
+static gint drag_start_stop_cb(GtkWidget *widget, GdkEventButton *event,
 			       GlameTreeItem *item)
 {
 	static GlameTreeItem *drag_widget = NULL;
-	static int mode = -1;
+	static int mode = -1, drop_replace = 0;
 	static int cursor_type = -1;
 	static GdkCursor *cursor = NULL;
 	GdkEventButton *bevent = (GdkEventButton *)event;
@@ -639,34 +728,36 @@ static void drag_start_stop_cb(GtkWidget *widget, GdkEventButton *event,
 
 	if (event->type == GDK_BUTTON_PRESS) {
 		if (bevent->button != 1)
-			return;
+			return FALSE;
 
 		/* drag&drop start */
 		drag_widget = NULL;
 		mode = -1;
+		drop_replace = 0; /* auto */
 		if (bevent->state & GDK_SHIFT_MASK) {
 			DPRINTF("SHIFT modifier\n");
 			mode = 1;
+			drop_replace = !!(bevent->state & GDK_MOD1_MASK);
 			gnome_appbar_push(GNOME_APPBAR(glame_appbar),
 					  _("Drop into hbox"));
 		} else if (bevent->state & GDK_CONTROL_MASK) {
 			DPRINTF("CTRL modifier\n");
 			mode = 2;
+			drop_replace = !!(bevent->state & GDK_MOD1_MASK);
 			gnome_appbar_push(GNOME_APPBAR(glame_appbar),
 					  _("Drop into vbox"));
 		} else {
-			DPRINTF("illegal modifier\n");
-			return; /* modifier not valid */
+			return FALSE; /* modifier not valid */
 		}
 		drag_widget = item;
 
 	} else if (event->type == GDK_BUTTON_RELEASE) {
 		if (bevent->button != 1)
-			return;
+			return FALSE;
 
 		/* drag&drop end */
 		if (!drag_widget)
-			return; /* spurious event - ignore */
+			return FALSE; /* spurious event - ignore */
 		gnome_appbar_pop(GNOME_APPBAR(glame_appbar));
 		if (cursor) {
 			cursor_type = -1;
@@ -675,7 +766,7 @@ static void drag_start_stop_cb(GtkWidget *widget, GdkEventButton *event,
 			cursor = NULL;
 		}
 		if (drag_widget == item)
-			return; /* nop */
+			return TRUE; /* nop */
 
 		source = drag_widget->item;
 		dest = item->item;
@@ -687,7 +778,8 @@ static void drag_start_stop_cb(GtkWidget *widget, GdkEventButton *event,
 			/* Mode 1 - hbox insertion either before
 			 * dropped item or at tail (if dropped
 			 * on group). */
-			if (GPSM_ITEM_IS_GRP(dest)) {
+			if (GPSM_ITEM_IS_GRP(dest)
+			    && !drop_replace) {
 				if (gpsm_hbox_insert((gpsm_grp_t *)dest, source,
 						     gpsm_item_hsize(dest), 0) == -1)
 					DPRINTF("insertion failed\n");
@@ -700,7 +792,8 @@ static void drag_start_stop_cb(GtkWidget *widget, GdkEventButton *event,
 			/* Mode 2 - vbox insertion either before
 			 * dropped item or at tail (if dropped
 			 * on group). */
-			if (GPSM_ITEM_IS_GRP(dest)) {
+			if (GPSM_ITEM_IS_GRP(dest)
+			    && !drop_replace) {
 				if (gpsm_vbox_insert((gpsm_grp_t *)dest, source,
 						     0, gpsm_item_vsize(dest)) == -1)
 					DPRINTF("insertion failed\n");
@@ -731,9 +824,10 @@ static void drag_start_stop_cb(GtkWidget *widget, GdkEventButton *event,
 			mode = -1;
 		}
 		if (!drag_widget)
-			return;
+			return FALSE;
 
-		if (GPSM_ITEM_IS_GRP(item->item))
+		if (GPSM_ITEM_IS_GRP(item->item)
+		    && !drop_replace)
 			parent = (gpsm_grp_t *)item->item;
 		else
 			parent = gpsm_item_parent(item->item);
@@ -742,7 +836,7 @@ static void drag_start_stop_cb(GtkWidget *widget, GdkEventButton *event,
 			ok = 0;
 
 		if (cursor_type == ok)
-			return;
+			return FALSE;
 
 		cursor_type = ok;
 		if (ok) {
@@ -761,6 +855,7 @@ static void drag_start_stop_cb(GtkWidget *widget, GdkEventButton *event,
 			cursor = c;
 		}
 	}
+	return FALSE;
 }
 
 
@@ -902,11 +997,12 @@ static void handle_grp(glsig_handler_t *handler, long sig, va_list va)
 	}
 }
 
-static void handle_enterleave(GtkWidget *tree, GdkEventCrossing *event,
+static gint handle_enterleave(GtkWidget *tree, GdkEventCrossing *event,
 			      SwapfileGui *swapfile)
 {
 	if (event->type == GDK_ENTER_NOTIFY)
 		active_swapfilegui = swapfile;
+	return FALSE;
 }
 
 
@@ -933,7 +1029,7 @@ static SCM gls_swapfilegui_active_item()
 static SCM gls_swapfilegui_selected_items()
 {
 	GList *selected;
-	SCM s_items = SCM_LIST0;
+	SCM s_items = SCM_EOL;
 	if (!active_swapfilegui)
 		return s_items;
 	selected = GTK_TREE_SELECTION(active_swapfilegui->tree);
@@ -1040,6 +1136,7 @@ SwapfileGui *glame_swapfile_widget_new(gpsm_grp_t *root)
 		handle_grp_add_treeitem(GTK_OBJECT(swapfile->tree), item);
 
 	gtk_widget_show(swapfile->tree);
-	active_swapfilegui = swapfile;
+	if (!active_swapfilegui)
+		active_swapfilegui = swapfile;
 	return swapfile;
 }
