@@ -1,7 +1,7 @@
 /*
  * glame_gui_utils.c
  *
- * $Id: glame_gui_utils.c,v 1.20.2.1 2002/03/04 19:17:14 richi Exp $
+ * $Id: glame_gui_utils.c,v 1.27.2.2 2003/04/27 22:14:08 nold Exp $
  *
  * Copyright (C) 2001 Johannes Hirche
  *
@@ -22,7 +22,6 @@
  */
 
 #include <sys/param.h>
-#include <values.h>
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -38,171 +37,6 @@
 
 
 
-/*
- * The generic play bar.
- */
-
-typedef struct {
-	GnomeDialog * dia;
-	filter_t *net;
-	gboolean playing;
-	guint handler_id;
-	gint play_index, pause_index, stop_index, cancel_index;
-	int close_on_stop;
-	GtkFunction atExitFunc;
-	gpointer data;
-} play_struct_t;
-
-static gint network_finish_check_cb(play_struct_t *play)
-{
-	if (filter_is_ready(play->net) && play->playing) {
-		gtk_timeout_remove(play->handler_id);
-		filter_terminate(play->net);
-		gnome_dialog_close(play->dia);
-		return FALSE;
-	}
-	return TRUE;
-}
-
-static void play_cb(GnomeDialog * dia, play_struct_t* play)
-{
-	if (filter_launch(play->net, _GLAME_WBUFSIZE) == -1
-	    || filter_start(play->net) == -1) {
-		glame_network_error_dialog(play->net,
-					   "Error processing network");
-		gnome_dialog_close(play->dia);
-		return;
-	}
-	play->playing = TRUE;
-	gnome_dialog_set_sensitive(play->dia, play->play_index, FALSE);
-	if (play->pause_index != -1)
-		gnome_dialog_set_sensitive(play->dia, play->pause_index, TRUE);
-	if (play->stop_index != -1)
-		gnome_dialog_set_sensitive(play->dia, play->stop_index, TRUE);
-	play->handler_id = gtk_timeout_add(100, (GtkFunction)network_finish_check_cb, play);
-}
-
-static void pause_cb(GnomeDialog * dia, play_struct_t * play)
-{
-	if(FILTER_IS_LAUNCHED(play->net)){
-		if(play->playing){
-			play->playing = FALSE;
-			filter_pause(play->net);
-		} else {
-			filter_start(play->net);
-			play->playing = TRUE;
-		}
-	}
-}
-
-static void stop_cb(GnomeDialog *dia, play_struct_t* play)
-{
-	if(play->handler_id)
-		gtk_timeout_remove(play->handler_id);
-	if(FILTER_IS_LAUNCHED(play->net)){
-		if (play->pause_index != -1)
-			gnome_dialog_set_sensitive(play->dia,
-						   play->pause_index, FALSE);
-		gnome_dialog_set_sensitive(play->dia, play->stop_index, FALSE);
-		filter_terminate(play->net);
-		play->playing = FALSE;
-		gnome_dialog_set_sensitive(play->dia, play->play_index, TRUE);
-		if (play->close_on_stop)
-			gnome_dialog_close(play->dia);
-	}
-}
-
-static void cancel_cb(GnomeDialog *dia, play_struct_t *play)
-{
-	if(play->handler_id)
-		gtk_timeout_remove(play->handler_id);
-	if(FILTER_IS_LAUNCHED(play->net))
-		filter_terminate(play->net);
-	gnome_dialog_close(play->dia);
-}
-
-
-static void play_destroy_cb(GtkWidget *widget, play_struct_t *play)
-{
-	if (FILTER_IS_LAUNCHED(play->net))
-		filter_terminate(play->net);
-	if (play->atExitFunc)
-		play->atExitFunc(play->data);
-	free(play);
-}
-
-int glame_gui_play_network(filter_t *network, int modal,
-			   GtkFunction atExit, gpointer data,
-			   const char *start_label,
-			   const char *pause_label,
-			   const char *stop_label, int close_on_stop)
-{
-	play_struct_t *play;
-	int i;
-	if (!network || !FILTER_IS_NETWORK(network))
-		return -1;
-
-	if (!(play = malloc(sizeof(play_struct_t))))
-		return -1;
-
-	play->net = network;
-	play->atExitFunc = atExit;
-	play->data = data;
-	play->dia = GNOME_DIALOG(gnome_dialog_new(filter_name(network),NULL));
-	play->playing = FALSE;
-	play->handler_id = 0;
-	play->play_index = -1;
-	play->pause_index = -1;
-	play->stop_index = -1;
-	play->cancel_index = -1;
-	play->close_on_stop = close_on_stop;
-
-	i = 0;
-	if (!start_label)
-		start_label = "Play";
-	play->play_index = i++;
-	gnome_dialog_append_button_with_pixmap(play->dia, start_label,
-					       GNOME_STOCK_PIXMAP_FORWARD);
-	gnome_dialog_set_default(play->dia, play->play_index);
-	gnome_dialog_button_connect(play->dia, play->play_index,
-				    play_cb, play);
-
-	if (pause_label) {
-		play->pause_index = i++;
-		gnome_dialog_append_button_with_pixmap(play->dia, pause_label, GNOME_STOCK_PIXMAP_TIMER_STOP);
-		gnome_dialog_set_sensitive(play->dia, play->pause_index, FALSE);
-		gnome_dialog_button_connect(play->dia, play->pause_index,
-					    pause_cb, play);
-	}
-
-	if (stop_label) {
-		play->stop_index = i++;
-		gnome_dialog_append_button_with_pixmap(play->dia, stop_label,
-						       GNOME_STOCK_PIXMAP_STOP);
-		gnome_dialog_set_sensitive(play->dia, play->stop_index, FALSE);
-		gnome_dialog_button_connect(play->dia, play->stop_index,
-					    stop_cb, play);
-	}
-
-	play->cancel_index = i++;
-	gnome_dialog_append_button(play->dia,GNOME_STOCK_BUTTON_CANCEL);
-	gnome_dialog_button_connect(play->dia, play->cancel_index,
-				    cancel_cb, play);
-
-	if(modal)
-		gtk_window_set_modal(GTK_WINDOW(play->dia),TRUE);
-
-	/* Finally register cleanup stuff. */
-	gtk_signal_connect(GTK_OBJECT(play->dia), "destroy",
-			   (GtkSignalFunc)play_destroy_cb, play);
-
-	gtk_widget_show(GTK_WIDGET(play->dia));
-
-	return 0;
-}
-
-
-
 /*
  * Widget creation helpers. GNOME/gtk+ suck...
  */
@@ -222,6 +56,11 @@ static void update_long_from_adjustment_cb(GtkAdjustment *adj, long *value)
 	*value = adj->value;
 }
 
+static void update_double_from_adjustment_cb(GtkAdjustment *adj, double *value)
+{
+	*value = adj->value;
+}
+
 void create_label_widget_pair(GtkWidget *vbox,
 			      const char *clabel, GtkWidget *w)
 {
@@ -280,6 +119,28 @@ void create_label_long_pair(GtkWidget *vbox,
 	gtk_container_add(GTK_CONTAINER(vbox), hbox);
 }
 
+void create_label_double_pair(GtkWidget *vbox,
+			      const char *label, double *value,
+			      double vmin, double vmax)
+{
+	GtkWidget *hbox, *l, *sbutton;
+	GtkObject *adj;
+
+	*value = MAX(vmin, MIN(vmax, *value));
+
+	hbox = gtk_hbox_new(TRUE, 5);
+	l = gtk_label_new(label);
+	gtk_misc_set_alignment(GTK_MISC(l), 1.0, 0.5);
+	adj = gtk_adjustment_new(*value, vmin, vmax, 0.1, 1.0, 0.0);
+	sbutton = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1.0, 3);
+	gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
+			   (GtkSignalFunc)update_double_from_adjustment_cb, value);
+	gtk_container_add(GTK_CONTAINER(hbox), l);
+	gtk_container_add(GTK_CONTAINER(hbox), sbutton);
+	gtk_widget_show_all(hbox);
+	gtk_container_add(GTK_CONTAINER(vbox), hbox);
+}
+
 
 static void glame_dialog_file_request_browse_cb(GnomeFileEntry *fileentry,
 						const char *pattern)
@@ -302,6 +163,7 @@ GtkWidget *glame_dialog_file_request(const char *windowtitle,
         dialogVbox = GTK_WIDGET(GTK_VBOX(GNOME_DIALOG(dialog)->vbox));
 
         fileEntry = gnome_file_entry_new(history_id, label);
+	gnome_file_entry_set_modal(GNOME_FILE_ENTRY(fileEntry), TRUE);
         gtk_signal_connect(GTK_OBJECT(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(fileEntry))),
                            "changed", update_string_from_editable_cb,
 			   returnbuffer);
@@ -686,7 +548,14 @@ GdkImlibImage* glame_load_icon(const char *filename, int x, int y)
 
 GtkWidget* glame_load_icon_widget(const char* filename,int x, int y)
 {
-	return gnome_pixmap_new_from_imlib(glame_load_icon(filename,x,y));
+	GdkImlibImage *icon;
+	GtkWidget *pixmap;
+
+	icon = glame_load_icon(filename,x,y);
+	pixmap = gnome_pixmap_new_from_imlib(icon);
+	gdk_imlib_destroy_image(icon);
+
+	return pixmap;
 }
 
 
@@ -701,6 +570,8 @@ struct network_notificator {
 	guint timeout_handler_id;
 	glsig_handler_t *delete_handler;
 	filter_t *net;
+	int wbufsize;
+	filter_launchcontext_t *context;
 };
 
 static void network_notificator_delete(glsig_handler_t *handler, long sig,
@@ -720,12 +591,14 @@ static void network_notificator_delete(glsig_handler_t *handler, long sig,
 
 	/* Cleanup notificator. */
 	glsig_delete_all(&n->emitter);
+
+	filter_launchcontext_unref(&n->context);
 	free(n);
 }
 
 static gint network_notificator_timeout(struct network_notificator *n)
 {
-	if (!filter_is_ready(n->net)) {
+	if (!filter_is_ready(n->context)) {
 		/* Run tick handlers. */
 		glsig_emit(&n->emitter, GLSIG_NETWORK_TICK, n->net);
 		return TRUE;
@@ -739,6 +612,8 @@ static gint network_notificator_timeout(struct network_notificator *n)
 
 	/* Cleanup notificator. */
 	glsig_delete_all(&n->emitter);
+
+	filter_launchcontext_unref(&n->context);
 	free(n);
 
 	/* Remove timeout handler. */
@@ -753,17 +628,26 @@ glsig_emitter_t *glame_network_notificator_creat(filter_t *net)
 	n = ALLOC(struct network_notificator);
 	INIT_GLSIG_EMITTER(&n->emitter);
 	n->net = net;
+	n->wbufsize = _GLAME_WBUFSIZE;
+	n->context = NULL;
 
 	return &n->emitter;
 }
 
+void glame_network_notificator_set_wbufsize(glsig_emitter_t *emitter,
+					    int wbufsize)
+{
+	struct network_notificator *n = (struct network_notificator *)emitter;
+	n->wbufsize = wbufsize;
+}
+
 int glame_network_notificator_run(glsig_emitter_t *emitter, int timeout)
 {
 	struct network_notificator *n = (struct network_notificator *)emitter;
 	/* First launch & start the network. Cleanup in case of
 	 * errors. */
-	if (filter_launch(n->net, _GLAME_WBUFSIZE) == -1
-	    || filter_start(n->net) == -1) {
+	if (!(n->context = filter_launch(n->net, n->wbufsize))
+	    || filter_start(n->context) == -1) {
 		free(n);
 		return -1;
 	}
@@ -786,7 +670,7 @@ void glame_network_notificator_delete_network(glsig_handler_t *handler,
 
 	GLSIGH_GETARGS1(va, net);
 
-	filter_terminate(net);
+	filter_terminate(net->launch_context);
 	filter_delete(net);
 }
 
