@@ -1,6 +1,6 @@
 /*
  * basic_sample.c
- * $Id: basic_sample.c,v 1.54.2.4 2002/03/05 20:24:03 richi Exp $
+ * $Id: basic_sample.c,v 1.65.4.1 2003/04/19 18:23:20 richi Exp $
  *
  * Copyright (C) 2000 Richard Guenther
  *
@@ -43,7 +43,7 @@
 #include "util.h"
 #include "glplugin.h"
 
-PLUGIN_SET(basic_sample, "mix render volume_adjust delay extend repeat pan2")
+PLUGIN_SET(basic_sample, "mix render volume_adjust delay extend repeat pan2 clip")
 
 
 
@@ -124,7 +124,7 @@ static int mix_f(filter_t *n)
 	    || !(jf = ALLOCN(nr, float)))
 		FILTER_ERROR_RETURN("no memory");
 
-	gain = filterparam_val_float(filterparamdb_get_param(filter_paramdb(n), "gain"));
+	gain = filterparam_val_double(filterparamdb_get_param(filter_paramdb(n), "gain"));
 
 	rate = -1;
 	i = 0;
@@ -152,12 +152,12 @@ static int mix_f(filter_t *n)
 
 		/* FIXME! external factor */
 		param = filterparamdb_get_param(filterpipe_destparamdb(in), "gain");
-		p[i].factor *= filterparam_val_float(param);
+		p[i].factor *= filterparam_val_double(param);
 
 		/* FIXME! "offset" */
 		p[i].out_pos = 0;
 		param = filterparamdb_get_param(filterpipe_destparamdb(in), "offset");
-		p[i].out_pos = rate*filterparam_val_float(param);
+		p[i].out_pos = rate*filterparam_val_double(param);
 		p[i].fifo_pos = p[i].out_pos;
 
 		cnt = MIN(cnt, p[i].out_pos);
@@ -247,6 +247,8 @@ static int mix_f(filter_t *n)
 			     (!out || !output_ready) && !fifo_full ? NULL : &wset,
 			     NULL, &timeout);
 		if (res == -1) {
+			if (errno == EPIPE)
+				break;
 			if (errno != EINTR)
 				perror("select");
 			continue;
@@ -254,7 +256,7 @@ static int mix_f(filter_t *n)
 #ifdef FIFO_AUTOADJUST
 		if (res == 0) {
 			/* Network paused? */
-			if (filter_is_ready(n))
+			if (filter_is_ready(n->launch_context))
 				continue;
 			/* If we have a ready-to-read (or full fifo?) feedback
 			 * input and a not-ready-to-write output we have to
@@ -273,7 +275,7 @@ static int mix_f(filter_t *n)
 #else
 		if (res == 0) {
 			/* Network paused? */
-			if (filter_is_ready(n))
+			if (filter_is_ready(n->launch_context))
 				continue;
 			FILTER_ERROR_STOP("Deadlock");
 		}
@@ -497,7 +499,7 @@ static int mix_fixup(filter_t *n, filter_pipe_t *out)
 	}
 
 	/* phi is overridden by parameter */
-	phi = filterparam_val_float(filterparamdb_get_param(filter_paramdb(n), "position"));
+	phi = filterparam_val_double(filterparamdb_get_param(filter_paramdb(n), "position"));
 
 	if (rate != filterpipe_sample_rate(out)
 	    || phi != filterpipe_sample_hangle(out)) {
@@ -558,12 +560,12 @@ int mix_register(plugin_t *p)
 				   FILTER_PORTFLAG_INPUT,
 				   FILTERPORT_DESCRIPTION, "input stream",
 				   FILTERPORT_END);
-	filterparamdb_add_param_float(filterport_paramdb(in), "gain",
-				      FILTER_PARAMTYPE_FLOAT, 1.0,
-				      FILTERPARAM_END);
-	filterparamdb_add_param_float(filterport_paramdb(in), "offset",
-				      FILTER_PARAMTYPE_TIME_MS, 0.0,
-				      FILTERPARAM_END);
+	filterparamdb_add_param_double(filterport_paramdb(in), "gain",
+				       FILTER_PARAMTYPE_DOUBLE, 1.0,
+				       FILTERPARAM_END);
+	filterparamdb_add_param_double(filterport_paramdb(in), "offset",
+				       FILTER_PARAMTYPE_TIME_MS, 0.0,
+				       FILTERPARAM_END);
 
 	out = filterportdb_add_port(filter_portdb(f), PORTNAME_OUT,
 				    FILTER_PORTTYPE_SAMPLE,
@@ -571,12 +573,12 @@ int mix_register(plugin_t *p)
 				    FILTERPORT_DESCRIPTION, "mixed stream",
 				    FILTERPORT_END);
 
-	filterparamdb_add_param_float(filter_paramdb(f), "gain",
-				      FILTER_PARAMTYPE_FLOAT, 1.0,
-				      FILTERPARAM_END);
-	filterparamdb_add_param_float(filter_paramdb(f), "position",
-				      FILTER_PARAMTYPE_POSITION, FILTER_PIPEPOS_DEFAULT,
-				      FILTERPARAM_END);
+	filterparamdb_add_param_double(filter_paramdb(f), "gain",
+				       FILTER_PARAMTYPE_DOUBLE, 1.0,
+				       FILTERPARAM_END);
+	filterparamdb_add_param_double(filter_paramdb(f), "position",
+				       FILTER_PARAMTYPE_POSITION, FILTER_PIPEPOS_DEFAULT,
+				       FILTERPARAM_END);
 
 	f->f = mix_f;
 	in->connect = mix_connect_in;
@@ -693,11 +695,12 @@ static int render_f(filter_t *n)
 		nr_active -= nto1_tail(I, nr_in);
 	} while (nr_active>0);
 
+	FILTER_BEFORE_STOPCLEANUP;
+
 	/* Queue EOFs. */
 	filterport_foreach_pipe(out, p)
 		sbuf_queue(p, NULL);
 
-	FILTER_BEFORE_STOPCLEANUP;
 	FILTER_BEFORE_CLEANUP;
 
 	for (i=0; i<nr_out; i++)
@@ -748,7 +751,7 @@ static void render_fixup_param(glsig_handler_t *h, long sig, va_list va)
 	pipe = filterparam_get_sourcepipe(param);
 	filterpipe_settype_sample(pipe,
 				  filterpipe_sample_rate(pipe),
-				  filterparam_val_float(param));
+				  filterparam_val_double(param));
 	glsig_emit(filterpipe_emitter(pipe), GLSIG_PIPE_CHANGED, pipe);
 }
 static int render_connect_in(filter_port_t *port, filter_pipe_t *p)
@@ -803,7 +806,7 @@ int render_register(plugin_t *p)
 				    FILTER_PORTFLAG_OUTPUT,
 				    FILTERPORT_DESCRIPTION, "mixed stream",
 				    FILTERPORT_END);
-	filterparamdb_add_param_float(filterport_paramdb(out), "position",
+	filterparamdb_add_param_double(filterport_paramdb(out), "position",
 				      FILTER_PARAMTYPE_POSITION,
 				      FILTER_PIPEPOS_DEFAULT,
 				      FILTERPARAM_END);
@@ -832,7 +835,7 @@ int render_register(plugin_t *p)
 
 static int vadjust_set_param(filter_param_t *param, const void *val)
 {
-	float ngain, gain;
+	double ngain, gain;
 	filter_param_t *anyscale;
 	filter_t *n;
 
@@ -840,7 +843,7 @@ static int vadjust_set_param(filter_param_t *param, const void *val)
 	if (n->priv)
 		return 0;
 
-	gain = *((float*)val);
+	gain = *((double *)val);
 	
 	if (strcmp("factor", filterparam_label(param))==0) {
 		anyscale = filterparamdb_get_param(filter_paramdb(n), "dbgain");
@@ -890,16 +893,17 @@ static int volume_adjust_f(filter_t *n)
 
 		/* adjust the amplitude by scale */
 		glsimd.scalar_product_1dI(buf, cnt,
-					  filterparam_val_float(scale));
+					  filterparam_val_double(scale));
 
 		/* queue the modified buffer */
 		sbuf_queue(out, b);
 	};
 
+	FILTER_BEFORE_STOPCLEANUP;
+
 	/* forward the EOF mark */
 	sbuf_queue(out, NULL);
 
-	FILTER_BEFORE_STOPCLEANUP;
 	FILTER_BEFORE_CLEANUP;
 
 	FILTER_RETURN;
@@ -925,16 +929,16 @@ int volume_adjust_register(plugin_t *p)
 			      FILTERPORT_DESCRIPTION, "scaled stream",
 			      FILTERPORT_END);
 
-	param = filterparamdb_add_param_float(filter_paramdb(f), "factor",
-					      FILTER_PARAMTYPE_FLOAT, 1.0,
-					      FILTERPARAM_LABEL, "Gain",
-					      FILTERPARAM_END);	
+	param = filterparamdb_add_param_double(filter_paramdb(f), "factor",
+					       FILTER_PARAMTYPE_DOUBLE, 1.0,
+					       FILTERPARAM_LABEL, "Gain",
+					       FILTERPARAM_END);	
 	param->set = vadjust_set_param;
 
-	param = filterparamdb_add_param_float(filter_paramdb(f), "dbgain",
-					      FILTER_PARAMTYPE_FLOAT, 0.0,
-					      FILTERPARAM_LABEL, "Gain [dB]",
-					      FILTERPARAM_END);
+	param = filterparamdb_add_param_double(filter_paramdb(f), "dbgain",
+					       FILTER_PARAMTYPE_DOUBLE, 0.0,
+					       FILTERPARAM_LABEL, "Gain [dB]",
+					       FILTERPARAM_END);
 	param->set = vadjust_set_param;
 	
 	plugin_set(p, PLUGIN_DESCRIPTION, "adjust the volume of a stream");
@@ -964,7 +968,7 @@ static int delay_f(filter_t *n)
 		FILTER_ERROR_RETURN("no input or no output");
 
 	delay = (int)(filterpipe_sample_rate(in)
-		      * filterparam_val_float(filterparamdb_get_param(filter_paramdb(n), "delay"))/1000.0);
+		      * filterparam_val_double(filterparamdb_get_param(filter_paramdb(n), "delay"))/1000.0);
 	if (delay < 0)
 		FILTER_ERROR_RETURN("weird delay time");
 
@@ -992,9 +996,11 @@ static int delay_f(filter_t *n)
 		FILTER_CHECK_STOP;
 		sbuf_queue(out, buf);
 	}
-	sbuf_queue(out, buf);
 
 	FILTER_BEFORE_STOPCLEANUP;
+
+	sbuf_queue(out, NULL);
+
 	FILTER_BEFORE_CLEANUP;
 
 	FILTER_RETURN;
@@ -1020,7 +1026,7 @@ int delay_register(plugin_t *p)
 			      FILTERPORT_DESCRIPTION, "delayed output stream",
 			      FILTERPORT_END);
 
-	param = filterparamdb_add_param_float(filter_paramdb(f), "delay",
+	param = filterparamdb_add_param_double(filter_paramdb(f), "delay",
 					      FILTER_PARAMTYPE_TIME_MS, 0.0,
 					      FILTERPARAM_END);
 
@@ -1050,7 +1056,7 @@ static int extend_f(filter_t *n)
 		FILTER_ERROR_RETURN("no input or no output");
 
 	time = (int)(filterpipe_sample_rate(in)
-		     * filterparam_val_float(filterparamdb_get_param(filter_paramdb(n), "time"))/1000.0);
+		     * filterparam_val_double(filterparamdb_get_param(filter_paramdb(n), "time"))/1000.0);
 	if (time < 0)
 		FILTER_ERROR_RETURN("weird extend time");
 
@@ -1080,10 +1086,11 @@ static int extend_f(filter_t *n)
 		sbuf_queue(out, buf);
 	}
 
+	FILTER_BEFORE_STOPCLEANUP;
+
 	/* send EOF */
 	sbuf_queue(out, NULL);
 
-	FILTER_BEFORE_STOPCLEANUP;
 	FILTER_BEFORE_CLEANUP;
 
 	FILTER_RETURN;
@@ -1108,7 +1115,7 @@ int extend_register(plugin_t *p)
 			      FILTERPORT_DESCRIPTION, "extended output stream",
 			      FILTERPORT_END);
 
-	filterparamdb_add_param_float(filter_paramdb(f), "time",
+	filterparamdb_add_param_double(filter_paramdb(f), "time",
 				      FILTER_PARAMTYPE_TIME_MS, 0.0,
 				      FILTERPARAM_DESCRIPTION, "extend time in ms",
 				      FILTERPARAM_END);
@@ -1140,7 +1147,7 @@ static int repeat_f(filter_t *n)
 		FILTER_ERROR_RETURN("no input or no output");
 
 	endless = 0;
-	duration = filterpipe_sample_rate(in)*filterparam_val_float(filterparamdb_get_param(filter_paramdb(n), "duration"));
+	duration = filterpipe_sample_rate(in)*filterparam_val_double(filterparamdb_get_param(filter_paramdb(n), "duration"));
 	if (duration < 0)
 		FILTER_ERROR_RETURN("weird time");
 	if (duration == 0)
@@ -1232,7 +1239,7 @@ int repeat_register(plugin_t *p)
 			      FILTER_PORTFLAG_OUTPUT,
 			      FILTERPORT_DESCRIPTION, "repeated stream",
 			      FILTERPORT_END);
-	filterparamdb_add_param_float(filter_paramdb(f), "duration",
+	filterparamdb_add_param_double(filter_paramdb(f), "duration",
 				  FILTER_PARAMTYPE_TIME_S, 0.0,
 				  FILTERPARAM_DESCRIPTION, "total duration in seconds",
 				  FILTERPARAM_END);
@@ -1265,18 +1272,17 @@ static int pan2_f(filter_t *n)
 
 	FILTER_AFTER_INIT;
 
-	/* The loop condition is at the end to get and
-	 * forward the EOF mark. */
-	do {
+	while ((buf = fbuf_get(in))) {
 		FILTER_CHECK_STOP;
-		/* get an input buffer */
-		buf = fbuf_get(in);
 
 		/* just forward every buffer */
 		fbuf_queue(out, buf);
-	} while (buf);
+	}
 
 	FILTER_BEFORE_STOPCLEANUP;
+
+	fbuf_queue(out, NULL);
+
 	FILTER_BEFORE_CLEANUP;
 
 	FILTER_RETURN;
@@ -1285,7 +1291,7 @@ static int pan2_set(filter_param_t *param, const void *val)
 {
 	filter_pipe_t *out;
 	filter_t *n;
-	float pos = *(float *)val;
+	double pos = *(double *)val;
 
 	if (pos < -M_PI || pos > M_PI)
 		return -1;
@@ -1294,8 +1300,7 @@ static int pan2_set(filter_param_t *param, const void *val)
 	out = filterport_get_pipe(filterportdb_get_port(filter_portdb(n), PORTNAME_OUT));
 	if (!out)
 		return 0;
-	filterpipe_settype_sample(out, filterpipe_sample_rate(out),
-				  *(float *)val);
+	filterpipe_settype_sample(out, filterpipe_sample_rate(out), pos);
 	glsig_emit(filterpipe_emitter(out), GLSIG_PIPE_CHANGED, out);
 
 	return 0;
@@ -1334,7 +1339,7 @@ static int pan2_connect_out(filter_port_t *port, filter_pipe_t *pipe)
 	if (in)
 		rate = filterpipe_sample_rate(in);
 	filterpipe_settype_sample(pipe, rate,
-				  filterparam_val_float(filterparamdb_get_param(filter_paramdb(n), "position")));
+				  filterparam_val_double(filterparamdb_get_param(filter_paramdb(n), "position")));
 	
 	return 0;
 }
@@ -1362,7 +1367,7 @@ int pan2_register(plugin_t *p)
 		FILTERPORT_DESCRIPTION, "panned stream",
 		FILTERPORT_END);
 	port->connect = pan2_connect_out;
-	param = filterparamdb_add_param_float(
+	param = filterparamdb_add_param_double(
 		filter_paramdb(f), "position",
 		FILTER_PARAMTYPE_POSITION, FILTER_PIPEPOS_DEFAULT,
 		FILTERPARAM_DESCRIPTION, "new position in stereo field",
@@ -1377,3 +1382,88 @@ int pan2_register(plugin_t *p)
   
 	return filter_register(f, p);
 }
+
+
+static int clip_f(filter_t *n)
+{
+	filter_pipe_t *in, *out;
+	filter_port_t *inp, *outp;
+	filter_param_t *clip;
+	filter_buffer_t *b;
+	SAMPLE *buf;
+	SAMPLE clipval;
+	int cnt;
+
+	inp = filterportdb_get_port(filter_portdb(n), PORTNAME_IN);
+	outp = filterportdb_get_port(filter_portdb(n), PORTNAME_OUT);
+	if (!(in = filterport_get_pipe(inp))
+	    || !(out = filterport_get_pipe(outp)))
+		FILTER_ERROR_RETURN("no input or no output");
+	clip = filterparamdb_get_param(filter_paramdb(n), "clip");
+	
+	FILTER_AFTER_INIT;
+
+	/* do the actual work */
+	while ((b = sbuf_get(in))) {
+		FILTER_CHECK_STOP;
+
+		/* get working copy, if necessary,
+		 * prepare for streamed loops. */
+		b = sbuf_make_private(b);
+		buf = sbuf_buf(b);
+		cnt = sbuf_size(b);
+		clipval = filterparam_val_double(clip);
+
+		while (cnt--)
+			buf[cnt] = buf[cnt] < -clipval ? -clipval
+				   : buf[cnt] > clipval ? clipval : buf[cnt];
+
+		/* queue the modified buffer */
+		sbuf_queue(out, b);
+	};
+
+	FILTER_BEFORE_STOPCLEANUP;
+
+	/* forward the EOF mark */
+	sbuf_queue(out, NULL);
+
+	FILTER_BEFORE_CLEANUP;
+
+	FILTER_RETURN;
+}
+int clip_register(plugin_t *p)
+{
+	filter_t *f;
+	filter_param_t *param;
+	filter_port_t *port;
+
+	if (!(f = filter_creat(NULL)))
+		return -1;
+	f->f = clip_f;
+
+	port = filterportdb_add_port(
+		filter_portdb(f), PORTNAME_IN,
+		FILTER_PORTTYPE_SAMPLE,
+		FILTER_PORTFLAG_INPUT,
+		FILTERPORT_DESCRIPTION, "input stream",
+		FILTERPORT_END);
+	port = filterportdb_add_port(
+		filter_portdb(f), PORTNAME_OUT,
+		FILTER_PORTTYPE_SAMPLE,
+		FILTER_PORTFLAG_OUTPUT,
+		FILTERPORT_DESCRIPTION, "clipped stream",
+		FILTERPORT_END);
+	param = filterparamdb_add_param_double(
+		filter_paramdb(f), "clip",
+		FILTER_PARAMTYPE_SAMPLE, 1.0,
+		FILTERPARAM_DESCRIPTION, "clip value",
+		FILTERPARAM_END);
+
+	plugin_set(p, PLUGIN_DESCRIPTION,
+		   "clips the stream to the specified value");
+	plugin_set(p, PLUGIN_PIXMAP, "clip.png");
+	plugin_set(p, PLUGIN_CATEGORY, "Filter");
+	plugin_set(p, PLUGIN_GUI_HELP_PATH, "Mangling_Data_Streams");
+  
+	return filter_register(f, p);
+}
