@@ -1,6 +1,6 @@
 /*
  * fft.c
- * $Id: fft_plugins.c,v 1.13.2.1 2002/03/24 18:48:59 richi Exp $
+ * $Id: fft_plugins.c,v 1.19 2002/05/25 15:08:03 richi Exp $
  *
  * Copyright (C) 2000 Alexander Ehlert
  *
@@ -43,7 +43,7 @@ PLUGIN_SET(fft_plugins,"fft ifft fft_resample fft_equalizer fft_bandpass")
 
 pthread_mutex_t planlock = PTHREAD_MUTEX_INITIALIZER;
 
-SAMPLE *hanning(int n)
+static SAMPLE *hanning(int n)
 {
 	SAMPLE *win;
 	int i;
@@ -55,7 +55,7 @@ SAMPLE *hanning(int n)
 	return win;
 }
 
-SAMPLE window_gain(SAMPLE *win, int n, int osamp)
+static SAMPLE window_gain(SAMPLE *win, int n, int osamp)
 {
 	SAMPLE *s;
 	double max;
@@ -94,10 +94,10 @@ static void fft_update_pipes(filter_t *n, filter_pipe_t *opipe)
 	in = filterport_get_pipe(in_port);
 	
 	if ((param=filterparamdb_get_param(filter_paramdb(n), "blocksize")))
-		bsize=filterparam_val_int(param);
+		bsize=filterparam_val_long(param);
 	
 	if ((param=filterparamdb_get_param(filter_paramdb(n), "oversamp")))
-		osamp=filterparam_val_int(param);
+		osamp=filterparam_val_long(param);
 
 	if (in) {	
 		rate=filterpipe_sample_rate(in);
@@ -144,39 +144,51 @@ static void fft_fixup_pipe(glsig_handler_t *h, long sig, va_list va)
 	glsig_emit(filterpipe_emitter(opipe), GLSIG_PIPE_CHANGED, opipe);
 }
 
-static int fft_fixup_param(filter_param_t *param, const void *val)
+static int fft_blocksize_set(filter_param_t *param, const void *val)
 {
-	filter_t *n = filterparam_filter(param);
-	filter_pipe_t *pipe;
-	int stuff = *((int*)val);
-	
-	pipe = filterport_get_pipe(filterportdb_get_port(filter_portdb(n), PORTNAME_OUT));
+	filter_pipe_t *out;
 
-	if (strcmp("blocksize", filterparam_label(param))==0) {
-		if (stuff<2)
-			return -1;
-		if (pipe) 
-		filterpipe_fft_bsize(pipe)=stuff;
-	}
-		
-	if(strcmp("oversamp", filterparam_label(param))==0) {
-		if (stuff<1)
-			return -1;
-		if (pipe)
-			filterpipe_fft_osamp(pipe)=stuff;
-	}
+	if (*((long *)val) < 2)
+		return -1;
 
-	if (pipe)
-		glsig_emit(&pipe->emitter, GLSIG_PIPE_CHANGED, pipe);
-	
+	out = filterport_get_pipe(filterportdb_get_port(
+		filter_portdb(filterparam_filter(param)), PORTNAME_OUT));
+	if (out) {
+		filterpipe_settype_fft(out,
+				       filterpipe_fft_rate(out),
+				       filterpipe_fft_hangle(out),
+				       *((long *)val),
+				       filterpipe_fft_osamp(out));
+		glsig_emit(filterpipe_emitter(out), GLSIG_PIPE_CHANGED, out);
+	}
 	return 0;
 }
 
-static int fft_f(filter_t *n){
-	queue_t		queue;
+static int fft_oversamp_set(filter_param_t *param, const void *val)
+{
+	filter_pipe_t *out;
+
+	if (*((long *)val) < 1)
+		return -1;
+
+	out = filterport_get_pipe(filterportdb_get_port(
+		filter_portdb(filterparam_filter(param)), PORTNAME_OUT));
+	if (out) {
+		filterpipe_settype_fft(out,
+				       filterpipe_fft_rate(out),
+				       filterpipe_fft_hangle(out),
+				       filterpipe_fft_bsize(out),
+				       *((long *)val));
+		glsig_emit(filterpipe_emitter(out), GLSIG_PIPE_CHANGED, out);
+	}
+	return 0;
+}
+
+static int fft_f(filter_t *n)
+{
+	in_queue_t    queue;
 	filter_pipe_t *in,*out;
 	filter_buffer_t *outb,*outb2;
-	filter_param_t *param;
 	rfftw_plan p;
 	SAMPLE *overlap, *s, *win;
 	int osamp, bsize = 2048;
@@ -187,25 +199,22 @@ static int fft_f(filter_t *n){
 	if (!(out=filterport_get_pipe(filterportdb_get_port(filter_portdb(n), PORTNAME_OUT))))
 		FILTER_ERROR_RETURN("no output");
 
-	if ((param=filterparamdb_get_param(filter_paramdb(n), "blocksize")))
-		bsize=filterparam_val_int(param);
-	
-	if ((param=filterparamdb_get_param(filter_paramdb(n), "oversamp")))
-		osamp=filterparam_val_int(param);
-	else osamp=1;
+	bsize = filterparam_val_long(
+		filterparamdb_get_param(filter_paramdb(n), "blocksize"));
+	osamp = filterparam_val_long(
+		filterparamdb_get_param(filter_paramdb(n), "oversamp"));
 
 	/* plans are not threadsafe! */
-	
 	pthread_mutex_lock(&planlock);
 	p = rfftw_create_plan(bsize, FFTW_REAL_TO_COMPLEX, FFTW_ESTIMATE | FFTW_IN_PLACE);
 	pthread_mutex_unlock(&planlock);
-	
+
 	if (!(overlap=ALLOCN(bsize,SAMPLE)))
 		FILTER_ERROR_RETURN("couldn't allocate overlap buffer");
 	if (!(win=hanning(bsize)))
 		FILTER_ERROR_RETURN("couldn't allocate window buffer");
 
-	init_queue(&queue, in, n);
+	in_queue_init(&queue, in, n);
 	
 	FILTER_AFTER_INIT;
 	
@@ -221,11 +230,11 @@ static int fft_f(filter_t *n){
 		s = sbuf_buf(outb);
 		
 		for(cnt=0; cnt<obufcnt; cnt++) {
-			if (queue_copy_pad(&queue, s, bsize)) {
+			if (in_queue_copy_pad(&queue, s, bsize)) {
 				cnt++;
 				break;
 			}
-			queue_shift(&queue, ooff);
+			in_queue_shift(&queue, ooff);
 			s += bsize;
 		}
 		
@@ -251,12 +260,11 @@ static int fft_f(filter_t *n){
 		sbuf_queue(out, outb2);
 	}
 	sbuf_queue(out, NULL);
-	queue_drain(&queue);
+	in_queue_drain(&queue);
 
 	FILTER_BEFORE_STOPCLEANUP;
 	FILTER_BEFORE_CLEANUP;
 
-
 	free(win);
 	rfftw_destroy_plan(p);
 
@@ -286,17 +294,17 @@ int fft_register(plugin_t *p)
 				    FILTERPORT_END);
 	out->connect = fft_connect_out;
 	
-	param = filterparamdb_add_param_int(filter_paramdb(f),"blocksize",
-			FILTER_PARAMTYPE_INT, 2048,
+	param = filterparamdb_add_param_long(filter_paramdb(f),"blocksize",
+			FILTER_PARAMTYPE_LONG, 2048,
 			FILTERPARAM_DESCRIPTION,"fft-block size",
 			FILTERPARAM_END);
-	param->set = fft_fixup_param;
+	param->set = fft_blocksize_set;
 	
-	param = filterparamdb_add_param_int(filter_paramdb(f),"oversamp",
-			FILTER_PARAMTYPE_INT, 8,
+	param = filterparamdb_add_param_long(filter_paramdb(f),"oversamp",
+			FILTER_PARAMTYPE_LONG, 8,
 			FILTERPARAM_DESCRIPTION,"oversampling factor",
 			FILTERPARAM_END);
-	param->set = fft_fixup_param;
+	param->set = fft_oversamp_set;
 
 	glsig_add_handler(&f->emitter, GLSIG_PIPE_CHANGED, fft_fixup_pipe, NULL);	
 
@@ -321,15 +329,19 @@ static int ifft_connect_out(filter_port_t *port, filter_pipe_t *p)
 {
 	filter_t *n = filterport_filter(port);
 	filter_pipe_t *in;
+	int rate = GLAME_DEFAULT_SAMPLERATE;
+	float hangle = 0.0;
 
 	if(filterport_get_pipe(port))
 		return -1;
 
-	if ((in = filterport_get_pipe(filterportdb_get_port(filter_portdb(n), PORTNAME_IN)))) {
-		DPRINTF("Setting rate %d hangle %f\n", 
-				filterpipe_fft_rate(in), filterpipe_fft_hangle(in));
-		filterpipe_settype_sample(p,filterpipe_fft_rate(in),filterpipe_fft_hangle(in));
+	if ((in = filterport_get_pipe(filterportdb_get_port(
+		filter_portdb(n), PORTNAME_IN)))) {
+		rate = filterpipe_fft_rate(in);
+		hangle = filterpipe_fft_hangle(in);
 	}
+	filterpipe_settype_sample(p, rate, hangle);
+
 	return 0;
 }
 
@@ -353,8 +365,9 @@ static void ifft_fixup_pipe(glsig_handler_t *h, long sig, va_list va)
 	glsig_emit(filterpipe_emitter(opipe), GLSIG_PIPE_CHANGED, opipe);
 }
 
-static int ifft_f(filter_t *n){
-	queue_t		queue;
+static int ifft_f(filter_t *n)
+{
+	out_queue_t	queue;
 	filter_pipe_t	*in, *out;
 	filter_buffer_t *inb;
 	rfftw_plan p;
@@ -394,7 +407,7 @@ static int ifft_f(filter_t *n){
 	for (j=0; j < bsize; j++)
 		win[j] *= gain;
 	
-	init_queue(&queue, out, n);
+	out_queue_init(&queue, out, n);
 
 	FILTER_AFTER_INIT;
 
@@ -420,15 +433,15 @@ static int ifft_f(filter_t *n){
 		s = sbuf_buf(inb);
 		
 		for (i=0; i<ibufcnt; i++) {
-			queue_add(&queue, s, bsize);
-			queue_add_shift(&queue, ooff);
+			out_queue_add(&queue, s, bsize);
+			out_queue_shift(&queue, ooff);
 			s += bsize;
 		}
 		sbuf_unref(inb);
 entry:		
 		inb = sbuf_make_private(sbuf_get(in));
 	}
-	queue_add_drain(&queue);
+	out_queue_drain(&queue);
 	sbuf_queue(out,NULL);
 
 	FILTER_BEFORE_STOPCLEANUP;
@@ -475,7 +488,8 @@ int ifft_register(plugin_t *p)
 }
 
 
-static int fft_resample_connect_in(filter_port_t *port, filter_pipe_t *p) {
+static int fft_resample_connect_in(filter_port_t *port, filter_pipe_t *p)
+{
 	if (filterport_get_pipe(port))
                 return -1;
 	return 0;
@@ -484,48 +498,52 @@ static int fft_resample_connect_in(filter_port_t *port, filter_pipe_t *p) {
 static int fft_resample_connect_out(filter_port_t *port, filter_pipe_t *p)
 {
 	filter_t *n = filterport_filter(port);
-	int rate = 44100, bsize = 2048;
 	filter_pipe_t *in;
-	filter_param_t *param;
+	int rate = 44100, bsize = 2048, osamp = 1;
+	float hangle = 0.0;
 
 	if(filterport_get_pipe(port))
 		return -1;
 
-	if ((in = filterport_get_pipe(filterportdb_get_port(filter_portdb(n), PORTNAME_IN)))) {
-		if ((param=filterparamdb_get_param(filter_paramdb(n), "frequency")))
-			rate=filterparam_val_int(param);
+	if ((in = filterport_get_pipe(filterportdb_get_port(
+		filter_portdb(n), PORTNAME_IN)))) {
+		rate = filterparam_val_long(filterparamdb_get_param(
+			filter_paramdb(n), "frequency"));
 	
-		bsize=((rate*filterpipe_fft_bsize(in)/filterpipe_fft_rate(in))>>2)<<2;
+		bsize = ((rate*filterpipe_fft_bsize(in)/filterpipe_fft_rate(in))>>2)<<2;
 		rate = bsize*filterpipe_fft_rate(in)/filterpipe_fft_bsize(in);
+		osamp = filterpipe_fft_osamp(in);
+		hangle = filterpipe_fft_hangle(in);
 		DPRINTF("resampling to bsize %d and frequency %d\n",bsize,rate);
-		filterpipe_settype_fft(p,rate,filterpipe_fft_hangle(in),bsize,filterpipe_fft_osamp(in));
 	}
+	filterpipe_settype_fft(p, rate, hangle, bsize, osamp);
 	
 	return 0;
 }
 
 
-static int fft_resample_fixup_param(filter_param_t *param, const void *val)
+static int fft_resample_frequency_set(filter_param_t *param, const void *val)
 {
 	filter_t *n = filterparam_filter(param);
 	filter_pipe_t *out, *in;
-	int rate = *((int*)val), bsize;
+	int rate = *((long *)val);
+	int in_rate = GLAME_DEFAULT_SAMPLERATE, in_bsize = 2048;
+	int out_rate, out_bsize;
 
-	if(strcmp("frequency", filterparam_label(param)) == 0) {
-		if(rate<=0)
-			return -1;
+	if (rate <= 0)
+		return -1;
+
+	if ((in = filterport_get_pipe(filterportdb_get_port(filter_portdb(n), PORTNAME_IN)))) {
+		in_rate = filterpipe_fft_rate(in);
+		in_bsize = filterpipe_fft_bsize(in);
 	}
-
-	if ((out = filterport_get_pipe(filterportdb_get_port(filter_portdb(n), PORTNAME_OUT)))
-	    && (in = filterport_get_pipe(filterportdb_get_port(filter_portdb(n), PORTNAME_IN)))) {
-		bsize= ((rate*filterpipe_fft_bsize(in)/filterpipe_fft_rate(in))>>2)<<2;
-		rate = bsize*filterpipe_fft_rate(in)/filterpipe_fft_bsize(in);
-		DPRINTF("rate = %d\n", rate);
-		filterpipe_settype_fft(out, 
-				       rate, 
-				       filterpipe_fft_hangle(in),
-				       bsize,
-				       filterpipe_fft_osamp(in));
+	if ((out = filterport_get_pipe(filterportdb_get_port(filter_portdb(n), PORTNAME_OUT)))) {
+		out_bsize = ((rate*in_bsize/in_rate) >> 2) << 2;
+		out_rate = out_bsize*in_rate/in_bsize;
+		DPRINTF("rate = %d\n", out_rate);
+		filterpipe_settype_fft(out, out_rate,
+				       filterpipe_fft_hangle(out), out_bsize,
+				       filterpipe_fft_osamp(out));
 
 		glsig_emit(filterpipe_emitter(out), GLSIG_PIPE_CHANGED, out);
 	}
@@ -536,30 +554,29 @@ static void fft_resample_fixup_pipe(glsig_handler_t *h, long sig, va_list va)
 {
 	filter_t	*n;
 	filter_port_t   *oport;
-	filter_pipe_t   *opipe, *pipe, *in;
-	filter_param_t  *param;
+	filter_pipe_t   *opipe, *in;
 	int rate = 44100, bsize = 2048;
 
-	GLSIGH_GETARGS1(va, pipe);
-	n = filterport_filter(filterpipe_dest(pipe));
+	GLSIGH_GETARGS1(va, in);
+	n = filterport_filter(filterpipe_dest(in));
 	oport = filterportdb_get_port(filter_portdb(n), PORTNAME_OUT);
 	opipe = filterport_get_pipe(oport);
 	if(!opipe)
 		return;
 
-	if ((in = filterport_get_pipe(filterportdb_get_port(filter_portdb(n), PORTNAME_IN)))) {
-		if ((param=filterparamdb_get_param(filter_paramdb(n), "frequency")))
-			rate=filterparam_val_int(param);
-	
-		bsize=((rate*filterpipe_fft_bsize(in)/filterpipe_fft_rate(in))>>2)<<2;
-		rate = bsize*filterpipe_fft_rate(in)/filterpipe_fft_bsize(in);
-		DPRINTF("resampling to bsize %d and frequency %d\n",bsize,rate);
-		filterpipe_settype_fft(opipe,rate,filterpipe_fft_hangle(in),bsize,filterpipe_fft_osamp(in));
-	}
+	rate = filterparam_val_long(filterparamdb_get_param(
+		filter_paramdb(n), "frequency"));
+	bsize = ((rate*filterpipe_fft_bsize(in)/filterpipe_fft_rate(in))>>2)<<2;
+	rate = bsize*filterpipe_fft_rate(in)/filterpipe_fft_bsize(in);
+	DPRINTF("resampling to bsize %d and frequency %d\n",bsize,rate);
+	filterpipe_settype_fft(opipe, rate, filterpipe_fft_hangle(in),
+			       bsize, filterpipe_fft_osamp(in));
+
 	glsig_emit(filterpipe_emitter(opipe), GLSIG_PIPE_CHANGED, opipe);
 }
 
-static int fft_resample_f(filter_t *n){
+static int fft_resample_f(filter_t *n)
+{
 	filter_pipe_t *in,*out;
 	filter_buffer_t *inb,*outb;
 	filter_param_t *param;
@@ -575,7 +592,7 @@ static int fft_resample_f(filter_t *n){
 	bsize=filterpipe_fft_bsize(in);
 	
 	if ((param=filterparamdb_get_param(filter_paramdb(n), "frequency")))
-		rate=filterparam_val_int(param);
+		rate=filterparam_val_long(param);
 
 	nbsize=((rate*filterpipe_fft_bsize(in)/filterpipe_fft_rate(in))>>2)<<2;
 	DPRINTF("Transforming blocksize %d to blocksize %d, new frequency %d\n",bsize,nbsize,rate);
@@ -635,11 +652,11 @@ int fft_resample_register(plugin_t *p)
 				    FILTERPORT_END);
 	out->connect = fft_resample_connect_out;
 	
-	param = filterparamdb_add_param_int(filter_paramdb(f),"frequency",
-			FILTER_PARAMTYPE_INT, 44100,
+	param = filterparamdb_add_param_long(filter_paramdb(f),"frequency",
+			FILTER_PARAMTYPE_LONG, 44100,
 			FILTERPARAM_DESCRIPTION,"resample frequency",
 			FILTERPARAM_END);
-	param->set = fft_resample_fixup_param;
+	param->set = fft_resample_frequency_set;
 	
 	glsig_add_handler(&f->emitter, GLSIG_PIPE_CHANGED, fft_resample_fixup_pipe, NULL);
 	f->f = fft_resample_f;
@@ -681,19 +698,19 @@ static int fft_equalizer_f(filter_t *n)
 	step = off/5;
 
 	if ((param = filterparamdb_get_param(filter_paramdb(n), "low")))
-		f[0] = filterparam_val_float(param);
+		f[0] = filterparam_val_double(param);
 
 	if ((param = filterparamdb_get_param(filter_paramdb(n), "midlow")))
-		f[1] = filterparam_val_float(param);
+		f[1] = filterparam_val_double(param);
 
 	if ((param = filterparamdb_get_param(filter_paramdb(n), "mid")))
-		f[2] = filterparam_val_float(param);
+		f[2] = filterparam_val_double(param);
 	
 	if ((param = filterparamdb_get_param(filter_paramdb(n), "midhigh")))
-		f[3] = filterparam_val_float(param);
+		f[3] = filterparam_val_double(param);
 	
 	if ((param = filterparamdb_get_param(filter_paramdb(n), "high")))
-		f[4] = filterparam_val_float(param);
+		f[4] = filterparam_val_double(param);
 
 	DPRINTF("f[0-4] = %f %f %f %f %f\n",f[0], f[1], f[2], f[3], f[4]);
 	
@@ -748,28 +765,28 @@ int fft_equalizer_register(plugin_t *p)
 			      FILTERPORT_DESCRIPTION, "equalized fft stream",
 			      FILTERPORT_END);
 	
-	filterparamdb_add_param_float(filter_paramdb(f),"low",
-			FILTER_PARAMTYPE_FLOAT, 1.0,
+	filterparamdb_add_param_double(filter_paramdb(f),"low",
+			FILTER_PARAMTYPE_DOUBLE, 1.0,
 			FILTERPARAM_DESCRIPTION,"low frequency gain",
 			FILTERPARAM_END);
 
-	filterparamdb_add_param_float(filter_paramdb(f),"midlow",
-			FILTER_PARAMTYPE_FLOAT, 1.0,
+	filterparamdb_add_param_double(filter_paramdb(f),"midlow",
+			FILTER_PARAMTYPE_DOUBLE, 1.0,
 			FILTERPARAM_DESCRIPTION,"middle low frequency gain",
 			FILTERPARAM_END);
 	
-	filterparamdb_add_param_float(filter_paramdb(f),"mid",
-			FILTER_PARAMTYPE_FLOAT, 1.0,
+	filterparamdb_add_param_double(filter_paramdb(f),"mid",
+			FILTER_PARAMTYPE_DOUBLE, 1.0,
 			FILTERPARAM_DESCRIPTION,"low frequency gain",
 			FILTERPARAM_END);
 	
-	filterparamdb_add_param_float(filter_paramdb(f),"midhigh",
-			FILTER_PARAMTYPE_FLOAT, 1.0,
+	filterparamdb_add_param_double(filter_paramdb(f),"midhigh",
+			FILTER_PARAMTYPE_DOUBLE, 1.0,
 			FILTERPARAM_DESCRIPTION,"middle high frequency gain",
 			FILTERPARAM_END);
 	
-	filterparamdb_add_param_float(filter_paramdb(f),"high",
-			FILTER_PARAMTYPE_FLOAT, 1.0,
+	filterparamdb_add_param_double(filter_paramdb(f),"high",
+			FILTER_PARAMTYPE_DOUBLE, 1.0,
 			FILTERPARAM_DESCRIPTION,"high frequency gain",
 			FILTERPARAM_END);
 
@@ -812,13 +829,13 @@ static int fft_bandpass_f(filter_t *n){
 	off = bsize/2;
 
 	if ((param = filterparamdb_get_param(filter_paramdb(n), "band minimum")))
-		fmin = filterparam_val_int(param);
+		fmin = filterparam_val_long(param);
 
 	if ((param = filterparamdb_get_param(filter_paramdb(n), "band maximum")))
-		fmax = filterparam_val_int(param);
+		fmax = filterparam_val_long(param);
 
 	if ((param = filterparamdb_get_param(filter_paramdb(n), "gain")))
-		gain = pow(10.0,filterparam_val_float(param)/20.0);
+		gain = pow(10.0,filterparam_val_double(param)/20.0);
 	
 	fmin = fmin/(freq/off);
 	fmax = MIN(fmax/(freq/off), bsize/2);
@@ -877,18 +894,18 @@ int fft_bandpass_register(plugin_t *p)
 			      FILTERPORT_DESCRIPTION, "fft stream band",
 			      FILTERPORT_END);
 	
-	filterparamdb_add_param_int(filter_paramdb(f),"band minimum",
-			FILTER_PARAMTYPE_INT, 0,
+	filterparamdb_add_param_long(filter_paramdb(f),"band minimum",
+			FILTER_PARAMTYPE_LONG, 0,
 			FILTERPARAM_DESCRIPTION,"Lower band frequency limit",
 			FILTERPARAM_END);
 
-	filterparamdb_add_param_int(filter_paramdb(f),"band maximum",
-			FILTER_PARAMTYPE_INT, 44100,
+	filterparamdb_add_param_long(filter_paramdb(f),"band maximum",
+			FILTER_PARAMTYPE_LONG, 44100,
 			FILTERPARAM_DESCRIPTION,"Upper band frequency limit",
 			FILTERPARAM_END);
 	
-	filterparamdb_add_param_float(filter_paramdb(f),"gain",
-			FILTER_PARAMTYPE_FLOAT, 0.0,
+	filterparamdb_add_param_double(filter_paramdb(f),"gain",
+			FILTER_PARAMTYPE_DOUBLE, 0.0,
 			FILTERPARAM_DESCRIPTION,"band gain [dB]",
 			FILTERPARAM_END);
 	
