@@ -88,7 +88,11 @@ static pthread_mutex_t swmx = PTHREAD_MUTEX_INITIALIZER;
 #define UNLOCK do { pthread_mutex_unlock(&swmx); } while (0)
 
 static void (* panic_handler)(const char *msg) = NULL;
-static void do_panic(const char *msg);
+#ifdef HAVE_HCC
+static void do_panic(const char *msg) __attribute__((noreturn));
+#else
+static void do_panic(const char *msg) __attribute__((noreturn));
+#endif
 #define SWPANIC(msg) do { if (swap.panic) PANIC(msg); else do_panic(msg); } while (0)
 
 
@@ -279,14 +283,14 @@ int swapfile_open(const char *name, int flags)
 	return 0;
 }
 
-void swapfile_sync()
+void swapfile_sync(void)
 {
 	file_sync();
 }
 
 /* Closes and updates a previously opened swap file/partition
  * and marks it clean. */
-void swapfile_close()
+void swapfile_close(void)
 {
 	struct swfd *f;
 	char s[256];
@@ -769,7 +773,7 @@ int sw_unlink(long name)
 /* Open the (flat) swapfile directory for reading. The stream
  * is positioned at the first file. Like opendir(3), but w/o
  * directory specification for obvious reason. */
-SWDIR *sw_opendir()
+SWDIR *sw_opendir(void)
 {
 	if (!SWAPFILE_OK()) {
 		errno = EINVAL;
@@ -1278,7 +1282,7 @@ ssize_t sw_write(swfd_t fd, const void *buf, size_t count)
 	size_t dcnt, cnt = count;
 	struct swfd *_fd;
 	struct swcluster *c;
-	s64 coff, old_size = -1, old_offset;
+	s64 coff, old_size = -1, old_offset = -1;
 	ssize_t res;
 	int err = 0;
 
@@ -1403,7 +1407,7 @@ void *sw_mmap(void *start, int prot, int flags, swfd_t fd)
 		return MAP_FAILED;
 	}
 
-	if (flags & MAP_FIXED) {
+	if (flags & ~(MAP_PRIVATE|MAP_SHARED)) {
 		errno = EINVAL;
 		return MAP_FAILED;
 	}
