@@ -98,6 +98,8 @@ static pthread_mutex_t clustersmx;
 #define LOCKCLUSTER(c) do { pthread_mutex_lock(&c->mx); } while (0)
 #define UNLOCKCLUSTER(c) do { pthread_mutex_unlock(&c->mx); } while (0)
 
+/* Flag, if we initialized the cache parameters already. */
+static int clusters_cache_set = 0;
 
 /* The internal helpers (with no locking) */
 static int __cluster_meta_open(long name, int flags);
@@ -159,6 +161,7 @@ static inline int swwrite(int fd, const void *buf, size_t count)
 
 void swcluster_set_cache(int maxlru, int maxfds, int maxmaps, size_t maxvm)
 {
+	clusters_cache_set = 1;
 	clustersmaxlru = MAX(256, MIN(32768, maxlru));
 	clusters_maplru_maxcnt = MAX(32, MIN(2048, maxmaps));
 	clusters_maxmappedsize = MAX(16*1024*1024, MIN(1024*1024*1024, maxvm));
@@ -168,27 +171,25 @@ void swcluster_set_cache(int maxlru, int maxfds, int maxmaps, size_t maxvm)
 static int cluster_init(int maxlru, int maxfds,
  			int maxmaps, size_t maxvm)
 {
+	/* Update maximas if not set before. */
+	if (!clusters_cache_set)
+		swcluster_set_cache(maxlru, maxfds, maxmaps, maxvm);
+
         /* Init the clusters lru - unused clusters only. */
-	clustersmaxlru = MAX(256, MIN(32768, maxlru));
 	clusterslrucnt = 0;
 	GLAME_INIT_LIST_HEAD(&clusterslru);
 	pthread_mutex_init(&clustersmx, NULL);
 	
 	/* Init the mappings lru - unused maps only. */
-	clusters_maplru_maxcnt = MAX(32, MIN(2048, maxmaps));
 	clusters_maplru_cnt = 0;
 	GLAME_INIT_LIST_HEAD(&clusters_maplru);
 	pthread_mutex_init(&mappingsmx, NULL);
-	clusters_maxmappedsize = MAX(16*1024*1024, MIN(1024*1024*1024, maxvm));
 
 	/* Init the fd lru - all fds. */
-	clusters_fdlru_maxcnt = MAX(16, MIN(1024, maxfds));
 	clusters_fdlru_cnt = 0;
 	GLAME_INIT_LIST_HEAD(&clusters_fdlru);
 	pthread_mutex_init(&fdsmx, NULL);
 
-	swcluster_set_cache(maxlru, maxfds, maxmaps, maxvm);
-
 	/* Initialize random number generator. */
 	srand(getpid());
 
@@ -203,7 +204,7 @@ static int cluster_init(int maxlru, int maxfds,
 	return 0;
 }
 
-static void cluster_cleanup()
+static void cluster_cleanup(void)
 {
 	struct swcluster *c;
 	int i;
@@ -483,7 +484,7 @@ static void _cluster_killmap(struct swcluster *c)
 	UNLOCKMAPPINGS;
 }
 
-static inline void __cluster_shrinkmaps()
+static inline void __cluster_shrinkmaps(void)
 {
 	while (clusters_mappedsize > clusters_maxmappedsize
 	       || clusters_maplru_cnt > clusters_maplru_maxcnt) {
@@ -801,18 +802,43 @@ static ssize_t cluster_write(struct swcluster *c, const void *buf,
  * Internal helpers. No locking.
  */
 
+/* Shrink the fdlru to honour the max size set. Caller needs to hold FD lock. */
+static void __cluster_fdlru_shrink(void)
+{
+	while (clusters_fdlru_cnt > clusters_fdlru_maxcnt) {
+		struct swcluster *c2;
+		c2 = glame_list_gettail(&clusters_fdlru, struct swcluster, fdlru);
+		close(c2->fd);
+		c2->fd = -1;
+		glame_list_del_init(&c2->fdlru);
+		clusters_fdlru_cnt--;
+	}
+}
+
 /* Helpers for opening the clusters metadata or data file on disk. */
 static int __cluster_meta_open(long name, int flags)
 {
 	char s[256];
+	int fd;
 
 	snprintf(s, 255, "%s/%lX/%lX", swap.clusters_meta_base,
 		 name & 0xff, name >> 8);
-	return open(s, flags, 0666);
+	fd = open(s, flags, 0666);
+	if (fd == -1
+	    && errno == EMFILE
+	    && clusters_fdlru_maxcnt > 16) {
+		DPRINTF("WARNING! fixing clusters_fdlru_maxcnt (%i)\n", clusters_fdlru_maxcnt);
+		clusters_fdlru_maxcnt /= 2;
+		LOCKFDS;
+		__cluster_fdlru_shrink();
+		UNLOCKFDS;
+		return __cluster_meta_open(name, flags); /* Try again. */
+	} else
+		return fd;
 }
 
 /* Alloc and init a cluster struct. */
-static struct swcluster *_cluster_new()
+static struct swcluster *_cluster_new(void)
 {
 	struct swcluster *c;
 
@@ -992,14 +1018,7 @@ static inline int __cluster_data_open(struct swcluster *c, int flags)
 		glame_list_add(&c->fdlru, &clusters_fdlru);
 		clusters_fdlru_cnt++;
 	}
-	while (clusters_fdlru_cnt > clusters_fdlru_maxcnt) {
-		struct swcluster *c2;
-		c2 = glame_list_gettail(&clusters_fdlru, struct swcluster, fdlru);
-		close(c2->fd);
-		c2->fd = -1;
-		glame_list_del_init(&c2->fdlru);
-		clusters_fdlru_cnt--;
-	}
+	__cluster_fdlru_shrink();
 	UNLOCKFDS;
 	if (c->fd == -1)
 		return __cluster_data_open(c, flags);
@@ -1036,14 +1055,16 @@ static void _cluster_truncate(struct swcluster *c, s32 size)
 		return;
 	}
 
-	/* Next, truncate the on-disk data file. */
+	/* Need the data. */
 	_cluster_needdata(c);
-	if (ftruncate(c->fd, size) == -1)
-		SWPANIC("Cannot truncate cluster");
 
 	/* Now, we have to adjust mappings of the cluster. */
 	_cluster_fixmap(c, size);
 
+	/* Next, truncate the on-disk data file. */
+	if (ftruncate(c->fd, size) == -1)
+		SWPANIC("Cannot truncate cluster");
+
 	/* Update in-memory size. */
 	c->size = size;
 }
